/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.messageeventlistener.js */
/* @Author:  Nicole Violette */
/* @Contact: nicole at slingo dot com */
/* @Created: Tue Aug 21 18:53:10 GMT 2012 */
/* @Notes:   Listen for message events from other windows. */

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') var SLINGO = {};
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.MessageEventListener)throw new Error('SLINGO.MessageEventListener already defined');

	
SLINGO.MessageEventListener = {
	
	_listenTimeout : null,
	
	/*  {
			BEACON_TYPE_1 : {
				EVENT_TYPE_1 : {function_ref_1 : function_1, function_ref_2 : function_2},
				EVENT_TYPE_2 : {function_ref_1 : function_1},
				ANY : {function_ref_1 : function_1}
			},
			BEACON_TYPE_2 : {...},
			ANY : {...}
		}
	*/
	_eventCallbacks : {},
	_relayFrame : null,
	_relayFrameSrc : '',
	_callbackCount : 0,
	isListening : false,
	/* Register functions to be called when an event occurs.
		Arguments:
		string beaconType - the type of beacon to listen to. Use "ANY" to listen for all types.
		string eventType - the type of event to listen for on a particular beacon. Use "ANY" to listen to all events broadcasted by a beacon.
		function callbackFn - the function to call when the specified event is broadcasted from the specified beacon
		
		return string - reference id for the added callback function
	*/
	setRelay : function(iframeId) {
		iframeObj = document.getElementById(iframeId);
		if(iframeObj) {
			this._relayFrame = iframeObj;
			this._relayFrameSrc = iframeObj.src;
		}
	},
	registerCallback : function(beaconType, eventType, callbackFn){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener.registerCallback(beaconType, eventType, callbackFn);
		
		if(typeof callbackFn !== 'function')
			return this._writeToConsole('MessageEventListener._registerCallback: callback is not a function');
			
		beaconType = beaconType.toUpperCase();
		
		this._callbackCount++;
		var refId = beaconType+'-'+eventType+'-'+this._callbackCount;
		
		if(!(beaconType in this._eventCallbacks)) this._eventCallbacks[beaconType] = {};
		if(!(eventType in this._eventCallbacks[beaconType])) this._eventCallbacks[beaconType][eventType] = {};

		this._eventCallbacks[beaconType][eventType][refId] = callbackFn;
		
		return refId;
	},
	/* Unregister callback registered with registerCallback */
	unregisterCallback : function(beaconType,eventType,refId) {
		if((beaconType in this._eventCallbacks) && this._eventCallbacks[beaconType][eventType][refId]) {
			delete this._eventCallbacks[beaconType][eventType][refId];
		}
	},
	/* Start listening for the beacon. Method depends on whether the browser supports the HTML5 postMessage(...) function. */
	start : function(){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener.start();
		this.isListening = true;
		return (window.postMessage && (typeof window.postMessage == "function"))
			? this._listenForEvent()
			: this._pollForEvent();
	},
	
	/* Start listening for the "onmessage" event from window.postMessage(...). For HTML5-enabled browsers only. */
	_listenForEvent : function(){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener._listenForEvent();
		
		var obj = window;
		var evt = "onmessage";
		var f_ref = SLINGO.MessageEventListener._catchEvent;
	    if(obj.addEventListener){
	        evt = evt.toLowerCase();
	        if(evt.indexOf("on") == 0) evt = evt.substring(2, evt.length);
	        obj.addEventListener(evt, f_ref, false);
	    } else if(obj.attachEvent) bSucc = obj.attachEvent(evt, f_ref);
	    else return this._writeToConsole("MessageEventListener._listenForEvent: failed to add event to the object");
	},
	
	/* Poll for updates to the current window.location.hash value. This is for non-HTML5 browsers only. */
	/* NOTE! This will NOT work for iframe embedded slingo games */
	_pollForEvent : function(){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener._pollForEvent();
		
		if(this._listenTimeout) this._listenTimeout = null;
		
		/* Get the hash value to see if messages were stored since the last timeout.
			Note #1: When clearing hash value, don't just set it to "#". This will cause the page to jump back to the top if the user has scrolled.
					 You don't have to use "#default", it just can't be "#". Also, make sure there is no element on the page that has a name
					 attribute of "default", or else the page will keep jumping to that element instead.
			Note #2: Only reset the hash value if the there was actually a message there. Otherwise, there will be a constant clicking
					 noise in IE as it updates the hash value every second.
		*/
		var msg = window.location.hash;
		if(msg !== '#default') window.location.hash = '#default';
		/* Message format: "#,msg1[,msg2[,msg3[,...]]] */
		var rem = ['#',','];
		for(var i = 0; i < rem.length; i++)
			if(msg.indexOf(rem[i]) === 0)
				msg = msg.substr(1, msg.length);
		msg = msg.split(",");
		if(msg) { 
			if(!this._relayFrame) {
	 			for(var i = 0; i < msg.length; i++) {
					if(msg[i].length) { 
						this._processEvent(msg[i]);
					}
				}
			}
		}
		
		this._listenTimeout = setTimeout(this._pollForEvent, 500);
		return true;
	},
	
	/* Called on each "onmessage" event. For HTML5-enabled browsers only. */
	_catchEvent : function(evt){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener._catchEvent(evt);
		/* Make sure the event origin is from Slingo.com. */
		if(!evt)				return this._writeToConsole("MessageEventListener._catchEvent: no event passed");
		if(!("origin" in evt))	return this._writeToConsole("MessageEventListener._catchEvent: no origin provided");
		
		/*
		var valid = false;
		var origins = ["http://www.slingo.com",
		               "http://slingo.com",
		               "https://www.slingo.com",
		               "https://slingo.com",
		               "http://heaven.slingo.com",
		               "https://heaven.slingo.com",
		               "http://adsvr.slingo.com",
		               "https://adsvr.slingo.com"
		               ];
		for(var i = 0; i < origins.length; i++)
			if(evt.origin === origins[i]){
				valid = true;
				break;
			}
		
		if(!valid) return this._writeToConsole("MessageEventListener._catchEvent: invalid origin: "+evt.origin);
		*/
		
		if(!("data" in evt)) return this._writeToConsole("MessageEventListener._catchEvent: no data provided");
		
		if(this._relayFrame) {
			this._relayFrame.contentWindow.postMessage(evt.data,'*');
		} else {
			return this._processEvent(evt.data);
		}
	},
	

	/* Process a message from the ad iframe. Can be used for both HTML5 and non-HTML5 enabled browsers. */
	_processEvent : function(data){
		if(this != SLINGO.MessageEventListener) return SLINGO.MessageEventListener._processEvent(data);

		var data = data.split("-");
		if(data.length !== 2) return false; // this._writeToConsole("MessageEventListener._processEvent: incorrect data format");
		
		var beacon = data[0].toUpperCase();
		var evt = data[1];
		
		/* Call any predefined callback functions. */
		if((beacon in this._eventCallbacks) && (typeof this._eventCallbacks[beacon] == 'object')){
			/* Notify callback functions for this event on this beacon. */
			if((evt in this._eventCallbacks[beacon]) && (typeof this._eventCallbacks[beacon][evt] == 'object'))
				for(var i in this._eventCallbacks[beacon][evt])
					if(typeof this._eventCallbacks[beacon][evt][i] == 'function')
						this._eventCallbacks[beacon][evt][i](beacon, evt);
			
			/* Notify callback functions for ANY event on this beacon. */
			if(("ANY" in this._eventCallbacks[beacon]) && (typeof this._eventCallbacks[beacon].ANY == 'object'))
				for(var i in this._eventCallbacks[beacon].ANY)
					if(typeof this._eventCallbacks[beacon].ANY[i] == 'function')
						this._eventCallbacks[beacon].ANY[i](beacon, evt);
		}
			
		/* Notify callback functions for ANY beacon. */
		if(("ANY" in this._eventCallbacks) && (typeof this._eventCallbacks.ANY == 'object')) {		
			if(evt in this._eventCallbacks.ANY) {
				for(var ref in this._eventCallbacks.ANY[evt]) {
					if(typeof this._eventCallbacks.ANY[evt][ref] == 'function') {
						this._eventCallbacks.ANY[evt][ref](beacon, evt);
					}
				}
			}					
		}

		return true;
	},
	
	_writeToConsole : function(msg){
		if(("console" in window) && ("log" in console))
			console.log(msg);
	}
	
};
/* OK, MessageEventListener plugin finished. */
SLINGO.transitLoadComplete = true;