/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.pbar.js */
/* @Author:  Gregory Patmore */
/* @Contact: greg at slingo dot com */
/* @Created: Thu Jul 17 16:51:23 EDT 2008 */
/* @Notes:   Handles functionality to show a css/js/dom scripting based progress bar */

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.PBar)throw new Error('SLINGO.PBar already defined');


SLINGO.PBar = {

	initBasicPBar : function(cont, fill, initPctg){
		SLINGO.$(cont).PBAR = new SLINGO.PBar.BasicBar(cont, fill, initPctg);
	},


	BasicBar : function(cont, fill, pctg){
		this.cont = SLINGO.$(cont);
		this.fill = SLINGO.$(fill);
		this.contWidth = parseInt(SLINGO.getElementStyle(this.cont,'width','width'));
		this.setPctgFull = function(pctg){
			this.fill.style.width = Math.ceil((this.contWidth/100) * pctg) + 'px';
		};

		if(typeof pctg != 'undefined') this.setPctgFull(pctg);
	}

};

SLINGO.PBar.ProgressBar = function(tNodeId){
	/* check node id for valid */
	if(!tNodeId || !SLINGO.$(tNodeId))
		SLINGO.triggerError('Must provide a valid node ID for construction',SLINGO.ETRIGGER);

	/* properties */
	/* node id to insert the prog. bar into */
	this.targetNodeId = tNodeId;

	/* holder for a reference to the node we create as a progress bar */
	this.barNode = null;

	/* the inner node that will hold the text displaying the percentages */
	this.targetTextNodeId='pbar_innerText';

	/* optional css class to set the progress bar to */
	this.progressBarClass = 'pBarFill';

	/* optional reference to a function to call when progress bar is complete */
	this.endCallFunction = false;

	/* the current width of the progress bar node */
	this.currWidth = 0;

	/* number of percentage points to increment each loop cycle */
	this.pctgPerLoop = 0;

	/* reference to the Interval object used for cycling */
	this.updateInterval;

	/* short textual message representing the current state of the object */
	/* used mainly for debug */
	this.status = 'IDLE';

	/* Boolean value representing whether or not an error has occurred */
	this.isError = false;

	/*  global reference to this object. */
	this.globalRef;

	/* The start time (in milliseconds from the UNIX epoch) for TIME_BASED_UPDATES */
	this.startTime = 0;

	/* the amount of milliseconds the progress bar should take to complete - for TIME_BASED_UPDATES*/
	this.durMSecs = 0;

	/* The end time (in milliseconds from the UNIX epoch) for TIME_BASED_UPDATES*/
	this.endTime = 0;
	
};
/* integer representing how many times per second the progress bar width will be updated */
SLINGO.PBar.ProgressBar.prototype.LOOP_RATE=10;

/* integer representing the total number of seconds to reach 100% */
SLINGO.PBar.ProgressBar.prototype.TOTAL_SECONDS=35;

/* loading text to display in front of your percentage */
SLINGO.PBar.ProgressBar.prototype.LOAD_TEXT = 'Loading: ';

/* Whether or not the loop mechanism is time based or loop based */
SLINGO.PBar.ProgressBar.prototype.TIME_BASED_UPDATES = false;

/* Timer Callbacks */ 
SLINGO.PBar.ProgressBar.prototype.callbacksTick = new Array();
SLINGO.PBar.ProgressBar.prototype.callbacksTickIdx = 0;
/* Methods */

/* function to initialize the progress bar object */
SLINGO.PBar.ProgressBar.prototype.init = function(){
	this.status = 'INITIALIZING';
	if(!this.targetNodeId || !SLINGO.$(this.targetNodeId)){
		this.isError = true;
		this.status = "FAILED TO INIT";
		throw new Error('Cannot intialize ProgressBar as no target has been specified');
	}
	if(!this.pctgPerLoop) this.setPctgPerLoop();
	this.createBarNode();
	if(typeof this.globalRef == 'undefined') this.setGlobalRef();
	if(!this.isError) this.status = 'INITIALIZED';
};

/* determines the per loop increment percentage */
SLINGO.PBar.ProgressBar.prototype.setPctgPerLoop = function(){
	this.pctgPerLoop=(100/this.TOTAL_SECONDS)/this.LOOP_RATE;
};

SLINGO.PBar.ProgressBar.prototype.addTickCallback = function(callback) {
	this.callbacksTick.push(callback);
	return this.callbacksTickIdx++;
}

/* Creates the progress bar node */
SLINGO.PBar.ProgressBar.prototype.createBarNode = function(){
	this.barNode = document.createElement('div');
	var dt = new Date();
	this.barNode.id = 'pBarNode_' + dt.getTime();

	if(this.progressBarClass)
		this.barNode.className = this.progressBarClass;

	this.barNode.style.width = '0';

	if(!this.targetNodeId || !SLINGO.$(this.targetNodeId)){
		this.isError = true;
		this.status = 'ERROR INITIALIZING BAR NODE';
		throw new Error('No Target Node Defined');
	}

	var t = document.createElement('div');
	t.id = this.targetTextNodeId;
	this.barNode.appendChild(t);
	SLINGO.$(this.targetNodeId).appendChild(this.barNode);
};

/* starts the progress bar in motion */
SLINGO.PBar.ProgressBar.prototype.launch = function(){
	if(this.updateInterval) clearInterval(this.updateInterval);

	/* Track this regardless of time based or not */
	var d = new Date();
	this.startTime = d.getTime();
	this.durMSecs = this.TOTAL_SECONDS * 1000
	this.endTime = this.startTime + this.durMSecs;

	/* If this pbar is time based, then we create the times here */
	if(this.TIME_BASED_UPDATES){
		/* dev only for use in testing the game cycle */
		/*SLINGO.SGS._showDebugMsg('ProgressBar.launch:: INTIALIZING TIME BASED UPDATES: TOTAL_SECONDS = ' + this.TOTAL_SECONDS);*/
		d.setTime(this.startTime);
		/* SLINGO.SGS._showDebugMsg('ProgressBar.launch:: start time set to : ' + d); */
		d.setTime(this.endTime);
		/* SLINGO.SGS._showDebugMsg('ProgressBar.launch:: end   time set to : ' + d); */
		/* start the loop */
		this.updateInterval = setInterval(this.globalRef+".updatePBar();", this.LOOP_RATE * 10);
	}
	this.status = 'RUNNING';
	return true;
};

/* sets up a global reference to itself for interval management*/
SLINGO.PBar.ProgressBar.prototype.setGlobalRef = function(){
	var dt = new Date();
	this.globalRef = 'JSProgressBar_instance_' + dt.getTime();
	window[this.globalRef] = this;
};

/* handles a full cycle for updating the progress bar */
SLINGO.PBar.ProgressBar.prototype.updatePBar = function(pboRef){
	/* update for time based updates */
	var d = new Date();
	var cs = d.getTime();
	delete d;
	var elapsedTime = (cs - this.startTime);

	if(this.TIME_BASED_UPDATES){
		if(cs >= this.endTime){
			this.currWidth = 100;
		}else{
			this.currWidth = (elapsedTime / this.durMSecs) * 100;
			// Remove psychotic math
			// var pMSecs = this.durMSecs - (cs - this.startTime);/* get how much time has passed */
			//this.currWidth = Math.round(100 - ((pMSecs*100)/this.durMSecs));
		}

	/* update for loop based updates */
	}else {
		this.currWidth += this.pctgPerLoop;
	}

	if(this.currWidth >= 100){
		this.currWidth = 100;
		clearInterval(this.updateInterval);
		this.status = 'COMPLETE';
		var endCall = true;
	}

	for(var i = 0; i < this.callbacksTick.length; i++) {
		this.callbacksTick[i](elapsedTime);
	}
	
	this.barNode.style.width = this.currWidth + '%';
	if(SLINGO.$(this.targetTextNodeId))
		SLINGO.$(this.targetTextNodeId).innerHTML = this.LOAD_TEXT + Math.floor(this.currWidth) + '%';
	if(endCall == true && this.endCallFunction)this.endCallFunction();
};

/* Explicitly set the fill for the progress bar to a given percentage */
SLINGO.PBar.ProgressBar.prototype.setFillPct = function(pct){
	/*SLINGO.triggerError('SLINGO.PBar.ProgressBar.prototype.setFillPct : pct('+pct+') for targetnode('+this.targetNodeId+')',SLINGO.EWARN);*/
	pct = Math.round(pct) + '%';
	this.barNode.style.width = pct;
	SLINGO.$(this.targetTextNodeId).innerHTML = this.LOAD_TEXT + pct;
	if(parseInt(pct) == 100 && this.endCallFunction){
		SLINGO.triggerError('End Call function is of type: ' + typeof(this.endCallFunction), SLINGO.EWARN);
		SLINGOGLOBAL.PBAREndCallFunc = this.endCallFunction;
		setTimeout('SLINGOGLOBAL.PBAREndCallFunc(); SLINGOGLOBAL.PBAREndCallFunc=false;',500);
	}
};

/* resets the progress bar to 0 for relaunch */
SLINGO.PBar.ProgressBar.prototype.reset = function(){
	this.currWidth = 0;
	if(typeof this.updateInterval != 'undefined')
		clearInterval(this.updateInterval);

	this.barNode.style.width = '0';
	this.status = 'INITIALIZED';
	return true;
};
/************************************************************************
/************************************************************************
/* Definitions of PBar class constants
/************************************************************************
/************************************************************************/





/************************************************************************
/************************************************************************
/* Definitions of PBar class methods
/************************************************************************
/************************************************************************/



/* OK, PBar plugin finished. */
SLINGO.transitLoadComplete = true;
