/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.overlay.js */
/* @Author:  Nicole Violette */
/* @Contact: nicole at slingo dot com */
/* @Created: Tue Jun 22 16:39:51 EDT 2010 */
/* @Notes:   */

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.Overlay)throw new Error('SLINGO.Overlay already defined');

/************************************************************************
/************************************************************************
/* Overlay Object Constructor
/************************************************************************
/* @PROTO:  RETURN SLINGO.Overlay(PARAMS)
/* @DESC:   FUNCTION_DESCRIPTION
/* @RETURN: RETURN_VALUE_DESCRIPTION
/* @PARAM:  PARAM_VALUE_DESCRIPTION
/************************************************************************
/************************************************************************/
SLINGO.Overlay = {

	/* The class given to an overlay element. */
	overlayClass 	: 'overlayItem',

	/* The id of the overlay background that greys out the rest of the page. */
	overlayBgId		: 'overlayBg',
	
	/* The class for all ad content areas. */
	adClass 		: 'adContent',
	
	/* Specific ad ids to look out for (that don't have the generic ad class). */
	adIds			: [
		'underGameLB',	/* leaderboard under game */
		'BAP-holder'	/* Ad Choices image */
	],

	/* Class names that determine how the plugin handles an overlay. */
	flags : {
		display 	: 'overlay_display',	/* Display an overlay on page load. */
		hide		: 'overlay_hide',		/* Hide an overlay on the close event. */
		remove		: 'overlay_remove',		/* Remove an overlay from the page on the close event. */
		refresh		: 'overlay_refresh',	/* Remove an overlay then refresh page on the close event. */
		keep_ads	: 'overlay_keep_ads'	/* Don't hide the ads when this overlay is opened. */
	},
	
	overlayElements : null,
	
	/* The id of the overlay that is currently open. */
	currentOpen : null,

	/* The list of auto-initialized overlays to display. */
	toDisplay : [],
	
	/* Array of overlay ids that are covering ads */
	overAdIds : [],
	
	/* Array of overlay ids that were opened during a preroll/interstitial and need to be checked for ads they cover again. */
	toRecheckForAds : [],

	/* On page load, initialize all hidden overlays and display the overlays that are flagged for auto-display. */
	initAll : function(){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.initAll();

		var overlays = SLINGO.getElementsByClass(this.overlayClass,document.body,'div');
		if(overlays != 'false'){                                  
			if(overlays.length > 0){
				for(var i = 0; i < overlays.length; i++){
					this.init(overlays[i].id, false);

					if(overlays[i].className.search(this.flags.display) > -1)
						this.toDisplay.push({'id':overlays[i].id, 'type':overlays[i].type});
				}

				var first = this.toDisplay.shift();
				if(first) this.display(first.id);
			}
		}
	},

	/* Initialize and display an overlay. Call this before display() if you've created the overlay *after* initial page load. */
	init : function(id){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.init(id);
		
		if(!SLINGO.$(id))
			return SLINGO.triggerError('Overlay.init: overlay does not exist: '+id);
		
		this.addCloseEvents(id);
	},

	/* Display a single hidden overlay. Params: id = overlay id, cid = "container" id to display the overlay over, vcenter = whether or not to vertically center the overlay, pos = specific top/left values for the overlay */
	display : function(id, cid, vcenter, hideBg, pos){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.display(id, cid, vcenter, hideBg, pos);

		if(!SLINGO.$(id))
			return SLINGO.triggerError('Overlay.display: overlay does not exist: '+id);
		
		if(hideBg !== true) this.createOverlayBg();
			
		SLINGO.$(id).style.display = 'block';

		var oh = SLINGO.DHTML.getObjectHeight(id);
		var ow = SLINGO.DHTML.getObjectWidth(id);			

		var dpo = document.documentElement.pageYOffset;
		var wpo = window.pageYOffset;
		var dst = document.documentElement.scrollTop;

		var t = 0;
		if(dpo) 		t = dpo;
		else if(wpo) 	t = wpo;
		else if(dst)	t = dst;

		if(cid && SLINGO.$(cid)){
			
			if(vcenter === true) t = 0;
			var epos = SLINGO.DHTML.getElementPosition(cid);
			/* vn -adding ability to pass the top position even if an obj already exists,
			 * noticed that if the obj is smaller than the overlay, the calculation for top is off
			 */
			SLINGO.$(id).style.top = (pos && pos.top) ? pos.top + 'px' : (t + (epos.top + ((SLINGO.DHTML.getObjectHeight(cid) - oh) / (vcenter === true ? 2 : 4)))) + 'px';
			SLINGO.$(id).style.left = (pos && pos.left) ? pos.left + 'px' : epos.left + ((SLINGO.DHTML.getObjectWidth(cid) - ow) / 2) + 'px';				

		} else {				
			SLINGO.$(id).style.top 	= ((pos && pos.top) 	? pos.top 	: (t + ((document.documentElement.clientHeight - oh) / (vcenter === true ? 2 : 4)))) + 'px';
			SLINGO.$(id).style.left = ((pos && pos.left) 	? pos.left 	: ((document.documentElement.clientWidth - ow) / 2)) + 'px';
			if(pos && pos.right) SLINGO.$(id).style.right = pos.right + 'px';
		}
	},
	
	/**
	 * Build a new overlay and display it on the page.
	 *
	 * @param string id - the id of the overlay
	 * @param object opts - config options for the overlay. possible values include:
	 *							- cid		the id of the element the overlay should appear above
	 *							- hide	 	if true, add the hide event to the overlay close button
	 *							- remove	if true, add the remove event to the overlay close button
	 *							- refresh	if true, add the refresh event to the overlay close button
	 *							- vcenter	if true, vertically center the overlay over the container element
	 *							- hideBg	if true, don't display the overlay background
	 *							- keepAds	if true, don't hide the ads on display overlay
	 *							- content	the content for the overlay
	 *							- type 		which content layout to create ('slingo', 'generic', 'flyout_left', 'flyout_top', 'content_only')
	 *							- solo		if true, this overlay needs to be displayed by itself (and not under/over another overlay)
	 *							- hideCloseButton if true, hide the red X
	 */
	createOverlay : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay(id, opts);
		/* If this overlay already exists, just display it. */
		if(SLINGO.$(id)){
			if(!opts.hideBg) this.createOverlayBg();	/* Added 2011.06.14 -NV */
			SLINGO.$(id).style.display = 'block';
			this.manageAds(id, true);
			return;
		}
		
		if(!opts) opts = {};
		
		switch(opts.type){
			case 'slingo':
				this.createOverlay_SLINGO(id, opts);
				break;
			case 'flyout_left':
				this.createOverlay_FLYOUT_LEFT(id, opts);
				break;
			case 'flyout_top':
				this.createOverlay_FLYOUT_TOP(id, opts);
				break;
			case 'flyout_top_right':
				this.createOverlay_FLYOUT_TOP_RIGHT(id, opts);
				break;
			case 'flyout_top_center':
				this.createOverlay_FLYOUT_TOP_CENTER(id, opts);
				break;
			case 'flyout_none':
				this.createOverlay_FLYOUT_NONE(id,opts);
				break;
			case 'content_only':
				this.createOverlay_CONTENT_ONLY(id, opts);
				break;
			case 'generic':
			default:
				this.createOverlay_GENERIC(id, opts);
				break;
		}
		
		document.getElementsByTagName('body')[0].appendChild(this.overlayElements);		
		this.init(id);
		
		if(opts.topPx || opts.leftPx || opts.rightPx){
			var pos = {};
			if(opts.topPx) 	pos.top = opts.topPx;
			if(opts.rightPx){
				pos.right = opts.rightPx;
			} else if(opts.leftPx){
				pos.left = opts.leftPx;
			}
		}
		
		/* Added 2011.06.16 to allow for chaining overlays. -NV */
		if(this.currentOpen && opts.solo){
			this.toDisplay.push({'id':id, 'type':opts.type});
			return;
		}
		
		this.currentOpen = id;
		this.display(
			id,
			opts.cid 		? opts.cid 		: false,
			opts.vcenter 	? opts.vcenter 	: false,
			opts.hideBg 	? opts.hideBg 	: false,
			pos				? pos			: false
		);
		
		this.adjustDivs(id, opts.type);
		this.manageAds(id, true);
	},
	
	adjustDivs : function(id, type){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.adjustDivs(id, type);
		
		if(!SLINGO.$(id))
			return SLINGO.triggerError('Overlay.adjustDivs: overlay does not exist: '+ id);
		var oh = SLINGO.DHTML.getObjectHeight(id);
		var ow = SLINGO.DHTML.getObjectWidth(id);
		switch(type){
			case 'slingo':
				SLINGO.$('overlayTopSide').style.width = ow - 18 + 'px';
				SLINGO.$('overlayWrapper').style.width = ow - 14 + 'px';
				SLINGO.$('overlayBottomSide').style.width = ow - 18 + 'px';
				
				
				var contentId = 'overlayContent_' + id;
				
				if(!SLINGO.$(contentId))
					return SLINGO.triggerError('overlay content id does not exist: '+ contentId);
				
				var overlayHeight = SLINGO.DHTML.getObjectHeight(contentId);
				oh = overlayHeight;
				SLINGO.$('overlayLeftSide').style.height = oh  + 'px';
				SLINGO.$('overlayWrapper').style.height = oh  + 'px';
				SLINGO.$('overlayRightSide').style.height = oh  + 'px';
				if(SLINGO.$('overlayImgHeader'))
					SLINGO.$('overlayImgHeader').style.marginRight =  80 + 'px';
				outerDiv = 'overlayOuter_' + id;

				break;
			default:
				//nothing..
		}
		
	},
	
	manageAds : function(id, hide){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.manageAds(id, hide);
		
		/* The ads shouldn't be affected by this overlay, so stop right here. */
		if(SLINGO.$(id) && (SLINGO.$(id).className.indexOf(this.flags.keep_ads) !== -1))
			return;
		
		/* Get all the ads that are eligible for hiding. */
		var ads = SLINGO.getElementsByClass(this.adClass);
		if(!ads) ads = [];
		
		for(var i in this.adIds)
			if(SLINGO.$(this.adIds[i]))
				ads.push(SLINGO.$(this.adIds[i]));
		
		if(ads){
			if(!hide){
				/* If this overlay was covering an ad, remove it from the list.. we'll only show the ads again if this list is empty. */
				var show = false;
				if(SLINGO.inArray(id, this.overAdIds)){
					var ids = [];
					for(var i = 0; i < this.overAdIds.length; i++)
						if(this.overAdIds[i] != id)
							ids.push(this.overAdIds[i]);
					this.overAdIds = ids;
					if(this.overAdIds.length <= 0){
						show = true;
						if(SLINGO.isPluginLoaded('SGS') && SLINGO.SGS.startStaticAdRefresh) SLINGO.SGS.startStaticAdRefresh();
						if(SLINGO.isPluginLoaded('GamePage') && SLINGO.GamePage.startStaticAdRefresh) SLINGO.GamePage.startStaticAdRefresh();
					}
				}
			}
			
			for(var a in ads){
				if(!ads[a] || !ads[a].style) continue;
				
				/* Hide each ad that is being covered by an overlay. */
				if(hide){
					ads[a].style.display = 'none';
					/* If we're on the game page, set the ad state for the static ad refresh. -NV 2011.07.28 */
					if(SLINGO.isPluginLoaded('GamePage') && ads[a].OBJ_INST_REF)
						ads[a].OBJ_INST_REF.setState('COVER');

				/* If all of the covering overlays are gone, show the ad. */
				} else if(show){
					ads[a].style.display = 'block';
					/* If we're on the game page, set the ad state for the static ad refresh. -NV 2011.07.28 */
					if(SLINGO.isPluginLoaded('GamePage') && ads[a].OBJ_INST_REF)
						ads[a].OBJ_INST_REF.setState('UNCOVER');
				}
			}
			
			/* Add the overlay to the list and let the game page know (if we're on one). */
			if(hide){
				this.overAdIds.push(id);
				if(SLINGO.isPluginLoaded('SGS') && SLINGO.SGS.stopStaticAdRefresh) SLINGO.SGS.stopStaticAdRefresh();
				if(SLINGO.isPluginLoaded('GamePage') && SLINGO.GamePage.stopStaticAdRefresh) SLINGO.GamePage.stopStaticAdRefresh();
			}
		}
	},
	
	valueInRange : function(value, min, max){
		return (value <= max) && (value >= min);
	},
	
	/**
	 * Build a new overlay (as per 2011 redesign) and display it on the page.
	 *
	 * @param string id - the id of the overlay
	 * @param object opts - config options for the overlay. possible values include:
	 *							- cid		the id of the element the overlay should appear above
	 *							- hide	 	if true, add the hide event to the overlay close button
	 *							- remove	if true, add the remove event to the overlay close button
	 *							- refresh	if true, add the refresh event to the overlay close button
	 *							- vcenter	if true, vertically center the overlay over the container element
	 *							- hideBg	if true, don't display the overlay background
	 *							- content	the content for the overlay
	 *							- logoImgName the name of the image for the logo on the right corner
	 *							- logoEnabled whether to show the logo or not.
	 */
	createOverlay_SLINGO : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_SLINGO(id, opts);
		var overlay = document.createElement('div');
		overlay.id = id;
		overlay.className = 'overlayItem'+(opts.keepAds ? ' '+this.flags.keep_ads : '');
		
			var overlayOuter = document.createElement('div');
			overlayOuter.id = 'overlayOuter_' + id;
			overlayOuter.className = 'overlayOuter';
			overlay.appendChild(overlayOuter);
			
				/** note the strict typing, 'null' or 'undefined' defaults the behavior to true **/
				if(opts.logoEnabled !== false) {
					var imgHeader = document.createElement('img');
					imgHeader.id = 'overlayImgHeader';
					imgHeader.className = 'overlayImgHeader';
					var logoImgName = (opts.logoImgName) ? opts.logoImgName : 'newOverlayTpr_Img.png';
					imgHeader.src = '/images/common/' + logoImgName;
					
					overlayOuter.appendChild(imgHeader);
				}
				var overlayUpperLeft = document.createElement('div');
				overlayUpperLeft.id = 'overlayUpperLeft';
				overlayUpperLeft.className = 'overlayBg';
				overlayOuter.appendChild(overlayUpperLeft);
				
				var overlayTopSide = document.createElement('div');
				overlayTopSide.id = 'overlayTopSide';
				overlayTopSide.className = 'overlayBg';
				overlayOuter.appendChild(overlayTopSide);
				
				var overlayUpperRight = document.createElement('div');
				overlayUpperRight.id = 'overlayUpperRight';
				overlayUpperRight.className = 'overlayBg';
				overlayOuter.appendChild(overlayUpperRight);
				
				var overlayLeftSide = document.createElement('div');
				overlayLeftSide.id = 'overlayLeftSide';
				overlayLeftSide.className = 'overlayBg';
				overlayOuter.appendChild(overlayLeftSide);
				
				var overlayWrapper = document.createElement('div');
				overlayWrapper.id = 'overlayWrapper';
				overlayWrapper.className = 'overlayWrapper';
				overlayOuter.appendChild(overlayWrapper);
				
					var overlayContent = document.createElement('div');
					overlayContent.id = 'overlayContent_' + id;
					overlayContent.className = 'overlayContent';
					overlayContent.innerHTML = opts.content;
					overlayWrapper.appendChild(overlayContent);
				
				var overlayRightSide = document.createElement('div');
				overlayRightSide.id = 'overlayRightSide';
				overlayRightSide.className = 'overlayBg';
				overlayOuter.appendChild(overlayRightSide);
				
				var overlayLowerLeft = document.createElement('div');
				overlayLowerLeft.id = 'overlayLowerLeft';
				overlayLowerLeft.className = 'overlayBg';
				overlayOuter.appendChild(overlayLowerLeft);
				
				var overlayBottomSide = document.createElement('div');
				overlayBottomSide.id = 'overlayBottomSide';
				overlayBottomSide.className = 'overlayBg';
				overlayOuter.appendChild(overlayBottomSide);
				
				var overlayLowerRight = document.createElement('div');
				overlayLowerRight.id = 'overlayLowerRight';
				overlayLowerRight.className = 'overlayBg';
				overlayOuter.appendChild(overlayLowerRight);
				
				var overlayClearDiv = document.createElement('div');
				overlayClearDiv.id = 'clear';
				overlayOuter.appendChild(overlayClearDiv);
				
				var closeBtn2 = document.createElement('div');
				closeBtn2.id = 'overlayCloseTop_'+id;
				closeBtn2.className = 'overlayCloseTop';
				if(opts.hide) closeBtn2.className += ' '+this.flags.hide;
				else if(opts.remove) closeBtn2.className += ' '+this.flags.remove;
				else if(opts.refresh) closeBtn2.className += ' '+this.flags.refresh;
					
				overlayOuter.appendChild(closeBtn2);
				
				var closeBtn = document.createElement('div');
				closeBtn.id = 'overlayClose_'+id;
				closeBtn.className = 'overlayClose'; 
				closeBtn.innerHTML = 'Close Window';
				if(opts.hide) closeBtn.className += ' '+this.flags.hide;
				else if(opts.remove) closeBtn.className += ' '+this.flags.remove;
				else if(opts.refresh) closeBtn.className += ' '+this.flags.refresh;
				overlayOuter.appendChild(closeBtn);
				
		SLINGO.Overlay.overlayElements = overlay;
	},
	
	/**
	 * Build a new overlay just with  content given
	 *
	 * @param string id - the id of the overlay
	 * @param object opts - config options for the overlay. possible values include:
	 *							- cid		the id of the element the overlay should appear above
	 *							- hide	 	if true, add the hide event to the overlay close button
	 *							- overlayClassName  - class for overlay
	 *							- remove	if true, add the remove event to the overlay close button
	 *							- vcenter	if true, vertically center the overlay over the container element
	 *							- hideBg	if true, don't display the overlay background
	 *							- content	the content for the overlay
	 */
	createOverlay_CONTENT_ONLY : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_SLINGO(id, opts);
		
		var overlay = document.createElement('div');
		overlay.id = id;
		overlay.className = 'overlayItem'+(opts.keepAds ? ' '+this.flags.keep_ads : '');
		if(opts.overlayClassName) overlay.className += " " + opts.overlayClassName;
		overlay.innerHTML = opts.content;	
				
		SLINGO.Overlay.overlayElements = overlay;
	},
	
	createOverlay_GENERIC : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_GENERIC(id, opts);
		
		var overlay = document.createElement('div');
		overlay.id = id;
		overlay.className = 'overlayItem genericOverlay'+(opts.keepAds ? ' '+this.flags.keep_ads : '');
		
			var overlayTL = document.createElement('div');
			overlayTL.className = 'genericBoxTL';
			overlay.appendChild(overlayTL);
			
			var overlayTM = document.createElement('div');
			overlayTM.className = 'genericBoxTM';
			overlay.appendChild(overlayTM);
			
			var overlayTR = document.createElement('div');
			overlayTR.className = 'genericBoxTR';
			overlay.appendChild(overlayTR);
			
			var innerBox = document.createElement('div');
			innerBox.className = 'genericBox';
			overlay.appendChild(innerBox);
			
				var innerBoxTL = document.createElement('div');
				innerBoxTL.className = 'genericContentTL';
				innerBox.appendChild(innerBoxTL);
				
				var innerBoxTM = document.createElement('div');
				innerBoxTM.className = 'genericContentTM';
				innerBox.appendChild(innerBoxTM);
				
					var closeBtn = document.createElement('div');
					closeBtn.id = 'genericClose_'+id;
					closeBtn.className = 'genericClose';
					if(opts.hide) closeBtn.className += ' '+this.flags.hide;
					else if(opts.remove) closeBtn.className += ' '+this.flags.remove;
					else if(opts.refresh) closeBtn.className += ' '+this.flags.refresh;
					innerBoxTM.appendChild(closeBtn);
				
				var innerBoxTR = document.createElement('div');
				innerBoxTR.className = 'genericContentTR';
				innerBox.appendChild(innerBoxTR);
				
				var content = document.createElement('div');
				content.className = 'genericContent';
				content.id = 'genericContent';
				content.innerHTML = opts.content ? opts.content : '';
				innerBox.appendChild(content);
				
				var innerBoxBL = document.createElement('div');
				innerBoxBL.className = 'genericContentBL';
				innerBox.appendChild(innerBoxBL);
				
				var innerBoxBM = document.createElement('div');
				innerBoxBM.className = 'genericContentBM';
				innerBox.appendChild(innerBoxBM);
				
				var innerBoxBR = document.createElement('div');
				innerBoxBR.className = 'genericContentBR';
				innerBox.appendChild(innerBoxBR);
					
			var overlayBL = document.createElement('div');
			overlayBL.className = 'genericBoxBL';
			overlay.appendChild(overlayBL);
			
			var overlayBM = document.createElement('div');
			overlayBM.className = 'genericBoxBM';
			overlay.appendChild(overlayBM);
			
			var overlayBR = document.createElement('div');
			overlayBR.className = 'genericBoxBR';
			overlay.appendChild(overlayBR);
			
			SLINGO.Overlay.overlayElements = overlay;
	},
	
	createOverlay_FLYOUT_LEFT : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT_LEFT(id, opts);
		return this.createOverlay_FLYOUT(id, opts, 'left');
	},
	
	createOverlay_FLYOUT_TOP : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT_TOP(id, opts);
		return this.createOverlay_FLYOUT(id, opts, 'top');
	},
	
	createOverlay_FLYOUT_TOP_RIGHT : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT_TOP_RIGHT(id, opts);
		return this.createOverlay_FLYOUT(id, opts, 'top_right');
	},
	createOverlay_FLYOUT_TOP_CENTER : function(id, opts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT_TOP_CENTER(id, opts);
		return this.createOverlay_FLYOUT(id, opts, 'top_center');
	},
	createOverlay_FLYOUT_NONE : function(id, opts) {
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT_NONE(id, opts);
		return this.createOverlay_FLYOUT(id, opts, 'none');	
	},
	createOverlay_FLYOUT : function(id, opts, type){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlay_FLYOUT(id, opts, type);
		
		if(!type || !SLINGO.inArray(type, ['top','left','top_right','top_center','none']))
			type = 'top';
		
		var overlay = document.createElement('div');
		overlay.id = id;
		overlay.className = 'flyout'+(type === 'left' ? ' flyoutHoriz' : '')+(opts.keepAds ? ' '+this.flags.keep_ads : '');
		overlay.innerHTML = opts.content ? opts.content : '';
		
		if(type !== 'none') {
			var pointer = document.createElement('div');
			pointer.className = 'pointer pointer_'+type;
			if (opts.pointerLeft){
				pointer.style.left = opts.pointerLeft + 'px';
			}
			overlay.appendChild(pointer);
		}
		if (typeof opts.hideCloseButton == 'undefined'){
			var closeBtn = document.createElement('div');
			closeBtn.id = 'flyout-close-'+id;
			closeBtn.className = 'flyout-close flyout_hide';
				var closeBtnLink = document.createElement('a');
				closeBtnLink.href = 'javascript:void(0)';
				closeBtn.appendChild(closeBtnLink);
				SLINGO.Event.addEvent(closeBtn, 'onclick', SLINGO.Overlay.close, { 'id': id, 'type': 'hide'});
			overlay.appendChild(closeBtn);
		}
		this.overlayElements = overlay;
	},

	/* Display an overlay over the entire page. */
	createOverlayBg : function(){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.createOverlayBg();

		/* Create a new overlay background and display it over the page. */
		if(!SLINGO.$(this.overlayBgId)){
			var bg = document.createElement('div');
			bg.id = this.overlayBgId;
			//bg.style.height = document.documentElement.scrollHeight + 'px';
			//bg.style.width = document.documentElement.scrollWidth + 'px';
			
			/** Clamped for IE bug fix "white" bg" **/
			sh = Math.min(document.documentElement.scrollHeight, 3000);
			sw = Math.min(document.documentElement.scrollWidth, 5000);
			
			bg.style.height = sh + "px";
			bg.style.width = sw + "px";
			bg.style.position = 'fixed';
			
			document.getElementsByTagName('body')[0].appendChild(bg);

			// If this is IE(6) and there are select boxes on the page, we have to put an iframe behind the overlay
			// so the select boxes don't appear over the overlay.
			if(document.all){
				var sb = document.getElementsByTagName('select');
				if(sb){
					var i = document.createElement('iframe');
					i.id = 'overlayIframe';
					i.src = '/images/admin/blank.gif';
					i.style.width = bg.style.width;
					i.style.height = bg.style.height;
					i.style.filter = 'alpha(opacity=20)';
					SLINGO.$(this.overlayBgId).appendChild(i);
				}
			}
		}
	},

	/* If there are no more overlays to display, remove the overlay background. */
	removeOverlayBg : function(){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.removeOverlayBg();

		if((this.toDisplay.length <= 0) && SLINGO.$(this.overlayBgId))
			SLINGO.$(this.overlayBgId).parentNode.removeChild(SLINGO.$(this.overlayBgId));
	},

	/* Find all of the elements inside of the overlay that should close it. */
	addCloseEvents : function(id){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.addCloseEvents(id);

		if(!SLINGO.$(id))
			return SLINGO.triggerError('Overlay.addCloseEvents: invalid overlay id: '+id);

		var hide = SLINGO.getElementsByClass(this.flags.hide, SLINGO.$(id));
		if(hide.length > 0)
			for(var i = 0; i < hide.length; i++)
				this.addCloseEvent(id, hide[i]/*.id*/, 'hide');		/* Removed id 2011.06.16 -NV */

		var remove = SLINGO.getElementsByClass(this.flags.remove, SLINGO.$(id));
		if(remove.length > 0)
			for(var i = 0; i < remove.length; i++)
				this.addCloseEvent(id, remove[i]/*.id*/, 'remove');	/* Removed id 2011.06.16 -NV */
		
		/* vnarvaez 8/15/2012 - added refresh */
		var refresh = SLINGO.getElementsByClass(this.flags.refresh, SLINGO.$(id));
		if(refresh.length > 0)
			for(var i = 0; i < refresh.length; i++)
				this.addCloseEvent(id, refresh[i]/*.id*/, 'refresh'); /* Removed id 2011.06.16 -NV */
	},
	
	/* Add an onclick event to an element on the page that will close the overlay. */
	addCloseEvent : function(id, btnId, type){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.addCloseEvent(id, btnId, type);

		var insts = {
			id		: id,
			type	: type
		};
		
		if(SLINGO.$(id) && SLINGO.$(btnId))
			if(!SLINGO.Event.addEvent(SLINGO.$(btnId), 'onclick', SLINGO.Overlay.close, insts))
				SLINGO.triggerError('Cannot add event to overlay close button');
	},

	/* Hide an overlay, remove it or refresh page. */
	close : function(insts){
		if(this !== SLINGO.Overlay) return SLINGO.Overlay.close(insts);

		if(!insts.id && !insts.type)
			return SLINGO.triggerError('Overlay.close: invalid instructions');

		var id = insts.id;
		var type = insts.type; 
		var next = this.toDisplay.shift();
		if(SLINGO.$(id)){
			switch(type){
				case 'hide':
					SLINGO.$(id).style.display = 'none';
					break;
				case 'remove':
					SLINGO.$(id).parentNode.removeChild(SLINGO.$(id));
					break;
				case 'refresh': //refresh page
					
					if(next){
						//if we have more overlays to show, close this one, show the next..
						SLINGO.$(id).parentNode.removeChild(SLINGO.$(id));	
						
					}else{
						//if no other overlays to show...just refresh page and exit
						//remove overlay
						SLINGO.$(id).parentNode.removeChild(SLINGO.$(id));
						var contentId = 'overlayContent_' + id;
						var contentOverlay = "<div id='overlayLoading'> <img width='100' height='100' alt='Loading' src='/images/common/loading100x100.gif'> One moment please..... </div>";
						var overlayOpt  = { cid: 'theBody', content: contentOverlay, type : 'slingo', remove : true};
						this.createOverlay(id, overlayOpt);						
						window.location.reload();
						return false;					
					}
					
					break;
				default:
					SLINGO.triggerError('Overlay.close: invalid close type: '+type);
			}
		}
		
		if(this.currentOpen) this.currentOpen = null;		
		if(next){
			this.display(next.id);
			this.adjustDivs(next.id, next.type);
			this.manageAds(next.id, false);
		}else {
			this.removeOverlayBg();
			this.manageAds(id, false);
		}
	},
	
	getScrollTop : function(){
		var dpo = document.documentElement.pageYOffset;
		var wpo = window.pageYOffset;
		var dst = document.documentElement.scrollTop;

		var t = 0;
		if(dpo) 		t = dpo;
		else if(wpo) 	t = wpo;
		else if(dst)	t = dst;
		return t;
	}

};

SLINGO.Flyout = {
	
	/* The id of the overlay that is currently open. */
	currentId	: null,
	
	/* The class given to a flyout element. */
	flyoutClass	: 'flyout',
	
	/* Acceptable directions for the pointer to point. */
	pointerDirs : ['top','left','top_right','top_center','none'],
	
	/* Class names that determine how the plugin handles an overlay. */
	flags : {
		hide 	: 'flyout_hide'		/* Hide a flyout on the close event. */
	},

	/**
		Build a new flyout and display it on the page.
		1. Close any overlay that's currently open.
		2. Determine the top/left position.
		3. Pass off to SLINGO.Overlay.createOverlay()
		4. Add close flyout events.
		
		@param string id - the id of the overlay
		@param object flyoutOpts - config options for the flyout. possible options:
										- pid			the node to base the flyout position's on. the flyout pointer should point at it.
										- direction		the direction the pointer should point (currently accepts "top" or "left")
	 	@param object overlayOpts - config options for the overlay. see SLINGO.Overlay.createOverlay() for list.
	*/
	createFlyout : function(id, fopts, oopts){
		if(this !== SLINGO.Flyout) return SLINGO.Flyout.createFlyout(id, fopts, oopts);
		if(this.currentId) this.close();
		this.currentId = id;
		
		/* If this flyout already exists, just display it. */
		if(SLINGO.$(id)){
			SLINGO.$(id).style.display = 'block';
			SLINGO.Overlay.manageAds(id, true);
			return;
		}		
		if(!fopts.direction || !SLINGO.inArray(fopts.direction, this.pointerDirs))
			fopts.direction = 'top';
			
		// Figure out the element's top and left positions so we can determine the flyout's position.
		if(fopts.pid && SLINGO.$(fopts.pid)){
			var pos = SLINGO.DHTML.getElementPosition(fopts.pid);
			if(fopts.direction === 'top'){
				var h = SLINGO.DHTML.getObjectHeight(fopts.pid);
				oopts.topPx = pos.top + h + 17;
				oopts.leftPx = pos.left + 5;
			} else if(fopts.direction === 'top_right'){
				var h = SLINGO.DHTML.getObjectHeight(fopts.pid);
				oopts.topPx = pos.top + h + 17;
				oopts.rightPx = pos.left;
			} else if(fopts.direction === 'top_center'){
				// duplicate of top
				var h = SLINGO.DHTML.getObjectHeight(fopts.pid);
				var w = SLINGO.DHTML.getObjectWidth(fopts.pid);
				oopts.topPx = pos.top + h + 24;
				oopts.leftPx = pos.left - Math.floor(w/2);
				oopts.pointerLeft = Math.floor(w/2) + 15; // pointer width 30 px
			} else {
				var w = SLINGO.DHTML.getObjectWidth(fopts.pid);
				oopts.topPx = pos.top;
				oopts.leftPx = pos.left + w + 15;
			}
		}			
		if(!oopts) oopts = {};
		oopts.hideBg	= true;
		oopts.type		= 'flyout_'+fopts.direction;
			
		SLINGO.Overlay.createOverlay(id, oopts);
		this.addCloseEvents(id);
	},	
	/* Find all of the elements inside of the overlay that should close it. */
	addCloseEvents : function(id){
		if(this !== SLINGO.Flyout) return SLINGO.Flyout.addCloseEvents(id);
		if(!SLINGO.$(id))
			return SLINGO.triggerError('Flyout.addCloseEvents: invalid flyout id: '+id,SLINGO.EWARN);

		var hide = SLINGO.getElementsByClass(this.flags.hide, SLINGO.$(id));
		if(hide.length > 0)
			for(var i = 0; i < hide.length; i++)
				this.addCloseEvent(id, hide[i].id, 'hide');
	},	
	/* Add an onclick event to an element on the page that will close the flyout. */
	addCloseEvent : function(id, btnId){
		if(this !== SLINGO.Flyout) return SLINGO.Flyout.addCloseEvent(id, btnId);
		
		if(SLINGO.$(id) && SLINGO.$(btnId))
			if(!SLINGO.Event.addEvent(SLINGO.$(btnId), 'onclick', SLINGO.Flyout.close))
				SLINGO.triggerError('Flyout.addCloseEvent: Cannot add event to flyout close button',SLINGO.EWARN);
	},
	/* Hide an overlay or remove it from the page. */
	close : function(){
		if(this !== SLINGO.Flyout) return SLINGO.Flyout.close();		
		if(!this.currentId)				return SLINGO.triggerError('Flyout.close: no current id',SLINGO.EWARN);
		if(!SLINGO.$(this.currentId))	return SLINGO.triggerError('Flyout.close: current node does not exist: '+this.currentId,SLINGO.EWARN);

		SLINGO.Overlay.close({id : this.currentId, type : 'hide'});
		this.currentId = null;
	}	
};
/* Onload functions */
SLINGO.addLoadEvent(SLINGO.Overlay.initAll);
/* OK, Overlay plugin finished. */
SLINGO.transitLoadComplete = true;