/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.pagination.js */
/* @Author:  Nicole Violette */
/* @Contact: nicole at slingo dot com */
/* @Created: Tue Mar 08 10:36:25 EST 2011 */
/* @Notes:    */

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.Pagination)throw new Error('SLINGO.Pagination already defined');

/************************************************************************
/************************************************************************
/* Pagination Object Constructor
/************************************************************************
/* @PROTO:  RETURN SLINGO.Pagination(PARAMS)
/* @DESC:   FUNCTION_DESCRIPTION
/* @RETURN: RETURN_VALUE_DESCRIPTION
/* @PARAM:  PARAM_VALUE_DESCRIPTION
/************************************************************************
/************************************************************************/
SLINGO.Pagination = {
	
	/* results is an object in the form {unique_id : 'first item', unique_id_2 : 'second item'} */
	Mgr : function(results){
		if(typeof results != 'object')
			return SLINGO.triggerError('Pagination.Mgr: invalid result object',SLINGO.EWARN);
		
		/* In case results is an object with numerical keys, pull the keys into an array so we can use the nice built-in JS array functions later on. */
		/* This is because JS won't always properly recognize numerical properties of an object (biggest example is with length). */
		items = new Array();
		for(var i in results)
                    if(i !== 'xfind')
			items.push(i);
			
		this.results = results;
		this.resultKeys = items;
		this.numRows = this.resultKeys.length;
		
		if(!this.numRows)
			return SLINGO.triggerError('Pagination.Mgr: empty result set',SLINGO.EWARN);
			
		return true;
	}
	
};

SLINGO.Pagination.Mgr.prototype = {
	
	/* full results object in the format {unique_id_1 : 'first item', unique_id_2 : 'second item} */
	results		: null,
	
	/* an array of the unique identifiers for each results item */
	resultKeys	: null,
	
	/* the number of items in the result set */
	numRows 	: null,
	
	/* how many items to show per page */
	perPage		: 10,
	
	/* the page we're currently displaying */
	currPage	: 1,
	
	/* the default number of page links to show before/after the current page */
	linksRange	: 5,
	
	/* set the number of results to show per page */
	setPerPage : function(perPage){
		perPage = parseInt(perPage);
		if(!perPage)
			return false;
			
		this.perPage = perPage;
	},
	
	/* get the specified page of results */
	getPage : function(pg){
		pg = parseInt(pg);
		if(!pg) return false;
		
		if(pg <= 0) pg = 1;
		else {
			total = this.getTotalPages();
			if(pg > total) pg = total;
		}
			
		this.setCurrPage(pg);
		return this._getResults();
	},
	
	/* get the current page of results */
	getCurrent : function(){
		return this._getResults();
	},
	
	/* get the next page of results */
	getNext : function(){
		curr = this.getCurrPage();
		total = this.getTotalPages();

		if((curr + 1) > total)
			return this.getLast();
			
		this.setCurrPage(curr + 1);
		return this._getResults();
	},
	
	/* get the previous page of results */
	getPrev : function(){
		curr = this.getCurrPage();
		if((curr - 1) <= 0)
			prev = 1;
		else prev = curr - 1;
		
		this.setCurrPage(prev);
		return this._getResults();
	},
	
	/* get the first page of results */
	getFirst : function(){
		this.setCurrPage(1);
		return this._getResults();
	},
	
	/* get the last page of results */
	getLast : function(){
		this.setCurrPage(this.getTotalPages());
		return this._getResults();
	},
	
	/* get the total number of results */
	getTotalRows : function(){
		return this.numRows;
	},
	
	/* finds the total number of result pages */
	getTotalPages : function(){
		rows = this.getTotalRows();
		pages = rows / this.perPage;
		if(rows % this.perPage != 0)
			pages = Math.ceil(pages);
		return pages;
	},
	
	/* get the current page number */
	getCurrPage : function(){
		return this.currPage;
	},
	
	/* gets the number of the result to start at for the current page */
	getStartNum : function(logical){
		start = (this.currPage * this.perPage) - this.perPage;
		return (logical === true) ? (start + 1) : start;
	},
	
	/* Gets the number of the last result for the current page */
	getEndNum : function(){
		end = this.getStartNum() + this.perPage;
		total = this.getTotalRows();
		return end > total
			? total
			: end;
	},
	
	/* Get the correct page numbers for the first/last/prev/nxt/number links in an object for link processing. */
	getPagesForLinks : function(range){
		range = parseInt(range);
		if(range) this.linksRange = range;
		
		var pages = {};
		
		last = this.getTotalPages();
		if(last == 1){
			pages.first = false;
			pages.last = false;
		} else {
			pages.first = 1;
			pages.last = last;
		}
		
		/* "next" isn't working.. maybe a reserved word? */
		next = this.currPage + 1;
		if(next > pages.last) pages.nxt = false;
		else pages.nxt = next;
		
		prev = this.currPage - 1;
		if(prev < pages.first) pages.prev = false;
		else pages.prev = prev;
		
		start = this.currPage - this.linksRange;
		if(start < pages.first) pages.start = pages.first;
		else pages.start = start;
		
		stop = this.currPage + this.linksRange;
		if(stop > pages.last) pages.stop = pages.last;
		else pages.stop = stop;
		
		return pages;
	},
	
	/* sets the current page of results to retrieve */
	setCurrPage : function(pg){
		if(!parseInt(pg)) return false;
		
		this.currPage = parseInt(pg);
		return true;
	},
	
	/* gets the current page of results */
	_getResults : function(){
		start = this.getStartNum();
		end = this.getStartNum() + this.perPage;
		
		var keys = this.resultKeys.slice(start, end);
		var res = {};
		for(var i = 0; i < keys.length; i++)
			res[keys[i]] = this.results[keys[i]];
		return res;
	}
	
};

/************************************************************************
/************************************************************************
/* Definitions of Pagination class constants
/************************************************************************
/************************************************************************/





/************************************************************************
/************************************************************************
/* Definitions of Pagination class methods
/************************************************************************
/************************************************************************/



/* OK, Pagination plugin finished. */
SLINGO.transitLoadComplete = true;