/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.bookmarks.js */
/* @Author:  Nicole Violette */
/* @Contact: nicole at slingo dot com */
/* @Created: Mon Feb 28 15:44:44 EST 2011 */
/* @Notes:   Functionality for the bookmarks user interface */

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.Bookmarks)throw new Error('SLINGO.Bookmarks already defined');

/************************************************************************
/************************************************************************
/* Bookmarks Object Constructor
/************************************************************************
/* @PROTO:  RETURN SLINGO.Bookmarks(PARAMS)
/* @DESC:   FUNCTION_DESCRIPTION
/* @RETURN: RETURN_VALUE_DESCRIPTION
/* @PARAM:  PARAM_VALUE_DESCRIPTION
/************************************************************************
/************************************************************************/
SLINGO.Bookmarks = {
	
	/* whether or not the bookmark interface has been initialized */
	initialized : false,
	
	/* the object containing information for the user's list of saved bookmarks */
	info : [],
	
	/* all of the user's saved bookmark ids */
	idList : [],
	
	/* the pagination manager object */
	pmgr : null,
	
	/* the number of bookmarks to display per page */
	resultsPerPage : 10,
	
	/* the current page of results */
	currentPage : 1,
	
	/* the default image to use if a bookmark doesn't have one defined */
	defaultImage : 'community',
	
	/* the name of the cookie holding the list of saved bookmark ids */
	listCookieName : 'slbkmrks',
	
	/* the name of the cookie holding the information for the first page of saved bookmarks */
	firstPageCookieName : 'slbkmrks_pg1',
	
	/* display msg when no bookmarks */		
	noBookMarksMsg : '<p class="noBookmarksMsg">You currently have no saved Quick Links. Click the "Quick Link This Page" button above to start!</p>',

	/* open the bookmarks interface and load the first page of bookmarks */
	init : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.init();
		if(this.initialized) return true;
		this.initialized = true;
		
		this.listCookieName = this.listCookieName + SLINGOGLOBAL.userId;
		this.firstPageCookieName = this.firstPageCookieName + SLINGOGLOBAL.userId;

		if(SLINGO.$('addBookmarkBtn'))
			SLINGO.Event.addEvent('addBookmarkBtn', 'onclick', this.add);
		
		this._loadBookmarkInfo();  
		
		if(window.isIE){
			var ua = navigator.userAgent;
			var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
			if (re.exec(ua) != null)
			   rv = parseFloat( RegExp.$1 );

			if(rv < 10){
				var ieNodes = SLINGO.getElementsByClass('flyoutHeader');

				for(var i=0; i<ieNodes.length; i++){
					var h1Node = ieNodes[i].children[0];
					h1Node.className = 'ieBkMk';
				}
			}			
		}
                
		if(this.idList.length > 0){
			this.setPgMgr();
        }		
	},
                
        setPgMgr : function(){
            if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.setPgMgr();
            
            this.pmgr = new SLINGO.Pagination.Mgr(this.idList);
            this.pmgr.setPerPage(this.resultsPerPage);
            
            return this.showPage(1);
        },
	
	/* get the specified page of bookmarks */
	showPage : function(pg){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.showPage(pg);
		
		pg = parseInt(pg);
		if(!pg) return SLINGO.triggerError('Bookmarks.showPage: Invalid page number',SLINGO.EWARN);
		this.currentPage = pg;
		this.pmgr.setCurrPage(this.currentPage);
		
		/* We already retrieved this page, so no need to request it again. */
                if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = '';
		if(this.info[pg] && (typeof this.info[pg] == 'object'))
			return this._displayPage();
		
		return this._getInfo();
	},
	
	/* load the bookmark information from the cookies */
	_loadBookmarkInfo : function(){
            if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._loadBookmarkInfo();

            /* Load the list of saved bookmark ids from the cookie. */
            var c = this._getListCookie();
            if(c === ''){
				 if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = 'Loading page, please wait...';
               //empty cookie, check server
                var d = new Date();
                if(!SLINGO.XHR.createAndSendRequest('loadBkMks' + d.getTime(), SLINGO.Bookmarks._catchLoadBkMks, '/ajax/bookmarks/load_bookmark_list/'+ d.getTime(), 'GET', 'JSON', true))
                    return SLINGO.triggerError('Failed to create the loadBkMks XHR request', SLINGO.ETRIGGER);	

            }else if(c && (c != 'none')){
				if(typeof c === 'string'){
					c = SLINGO.JSON.parse(c);
					c = c.v.split(',');
				}
				
                if(c && (typeof c == 'object'))
                    this.idList = c;

                this._loadFirstPage();
            }
            return true;
	},
       
       /* load first page of bookmarks */
       _loadFirstPage : function(){
            if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._loadFirstPage();
            
           /* Load the first page of saved bookmarks from the cookie. */
            this.info[1] = [];
            var c = this._getFirstPageCookie();
            if(c && (c != 'none')){
                var c = c.split('|');
                if(c && (typeof c == 'object')){
                    /*for(var item in c){*/
                    for(var item = 0; item < c.length; item++){
                        var add = {};
                        var info = c[item].split('&');
                        if(info && (typeof info == 'object')){
                            /*for(var i in info){*/
                            for(var i = 0; i < info.length; i++){
                                var n = info[i].split('=');
                                add[n[0]] = n[1];
                            }
                            this.info[1].push(add);
                        }
                    }
                }
            }
       },
	
	/* refresh the list of bookmarks and re-initialize the pagination manager after a bookmark is added or removed */
	_setPaginationMgr : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._setPaginationMgr();
		
		if(this.pmgr) delete this.pmgr;
		this.pmgr = new SLINGO.Pagination.Mgr(this.idList);
		this.pmgr.setPerPage(this.resultsPerPage);
		
		if(!this.pmgr)
			return SLINGO.triggerError('Bookmarks._setPaginationMgr: could not initialize pagination mgr',SLINGO.EWARN);
			
		return true;
	},
	
	/* retrieve bookmark information for the page we're trying to display */
	_getInfo : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._getInfo();
		
		var results = this.pmgr.getCurrent();
        if(results){
			var bids = [];
			for(var i in results)
				bids.push(results[i]);
			results = bids.join(',');
		}

		if(!results)
			return this._displayPage();
			
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = 'Loading page, please wait...';
			
		var post = {};
		post.bookmark_ids = results;
		
		var d = new Date;
		var req = 'getBookmarks_'+d.getTime();
		SLINGO.XHR.createRequest(req, '/ajax/bookmarks/get_info/'+d.getTime(), 'POST', 'JSON', true, post);
		SLINGO.XHR.activateRequest(req, this._catchInfo);
		return true;
	},
	
	/* catch the results from the get info request */
	_catchInfo : function(retv){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._catchInfo(retv);
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = '';
		
		if(retv.error)
			SLINGO.triggerError('Bookmarks._catchInfo: '+retv.error,SLINGO.EWARN);
		else this.info[this.currentPage] = retv;
			
		return this._displayPage();
	},
	
	/* build the markup for this page */
	_displayPage : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._displayPage();
		
		if(this.info[this.currentPage])
			var results = this.info[this.currentPage];
			
		var content = '';
		if(results){
			var treg = new RegExp('\\+', 'g');
			
			for(var r in results){
				if(!results[r].u || !results[r].t || !results[r].b) continue;
				
				if(!results[r].i) results[r].i = this.defaultImage;
				
				var img = this.getImage(results[r].b, results[r].i);
				/*if(img) content += img;*/
				var bmarkClass = (r == (results.length -1) ) ? 'class=lastBookMark' : false;					
				content += '<li ';
				content += (bmarkClass) ? bmarkClass : '';
				//content +=  '>' +(img ? img : '')+'<a href="/bookmarks'+results[r].u+'">'+results[r].t.replace(treg,' ')+'</a><span class="deleteBkmrkBtn" onclick="javascript:SLINGO.Bookmarks.remove(\''+results[r].b+'\')"></span></li>';
				content +=  '>' +(img ? img : '')+'<a href="/bookmarks'+results[r].u+'">'+results[r].t.replace(treg,' ')+'</a><span id="emailToolTip" tiptext="Remove Quick Link" class="deleteBkmrkBtn toolTipAnchor" onclick="javascript:SLINGO.Bookmarks.remove(\''+results[r].b+'\')"></span></li>';
			}
			
			if(content)
				content = '<ul id="listNavigation">'+content+'</ul>';
		}
		
		if(!content)
			content = this.noBookMarksMsg;
		
		if(!SLINGO.$('bookmarksBody'))
			return SLINGO.triggerError('Bookmark.Mgr.showPage: bookmarks node not found',SLINGO.EWARN);
			
		SLINGO.$('bookmarksBody').innerHTML = content;
		
		if(SLINGO.isPluginLoaded('ToolTip')) SLINGO.ToolTip.init();
		
		if(SLINGO.$('bkMarkContent'))
			bkColHt = SLINGO.$('bkMarkContent').offsetHeight;

		if(SLINGO.$('bkSuggestionsContent'))
			sgtColHt = SLINGO.$('bkSuggestionsContent').offsetHeight;

		if(bkColHt > sgtColHt){
			SLINGO.$('bkSuggestionsContent').style.height = bkColHt + 'px';
		}else if(bkColHt < sgtColHt){
			SLINGO.$('bkMarkContent').style.height = sgtColHt + 'px';
		}
		
		this._showPageLinks();
		return true;
	},
	
	/* show the page links that correspond to the current page of results */
	_showPageLinks : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._showPageLinks();
		
		var links = this.pmgr.getPagesForLinks();
		if(!links) return;
		
		var content = [];
		if(links.first && (links.first != this.currentPage)) 	content.push('<a class="nonNumber" href="javascript:void(0);" onclick="javascript:SLINGO.Bookmarks.showPage('+links.first+')">First</a> <');
		if(links.prev && (links.prev != this.currentPage)) 		content.push('<a class="nonNumber" href="javascript:void(0);" onclick="javascript:SLINGO.Bookmarks.showPage('+links.prev+')">Prev</a>');
		
		var hasNext = links.nxt && (links.nxt != this.currentPage);
		var hasLast = links.last && (links.last != this.currentPage);
		
		if(links.start && links.stop)
			for(var i = links.start; i <= links.stop; i++){
				if(this.currentPage != i)
					content.push('<a href="javascript:void(0);" onclick="javascript:SLINGO.Bookmarks.showPage('+i+')">'+i+'</a>');
				else content.push('<a href="javascript:void(0);" class="active_tnt_link '+(((i == links.stop) && !hasNext && !hasLast) ? 'last' : '')+'">'+i+'</a>');
			}
				
		if(hasNext) content.push('<a href="javascript:void(0);" onclick="javascript:SLINGO.Bookmarks.showPage('+links.nxt+')" class="nonNumber">Next</a> >');
		if(hasLast) content.push('<a href="javascript:void(0);" onclick="javascript:SLINGO.Bookmarks.showPage('+links.last+')" class="last nonNumber">Last</a>');
		
		if(!SLINGO.$('flyoutPageLinks'))
			return SLINGO.triggerError('Bookmarks.showPage: links node not found',SLINGO.EWARN);
			
		SLINGO.$('flyoutPageLinks').innerHTML = /*(content.length ? '<span>Go to Page: </span>' : '')+*/content.join('');
		return true;
	},
	
	/* add bookmark for the current page */
	add : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.add();
		
		var post = {};
		post.url = document.URL;
		if(!post.url) return SLINGO.triggerError('Boomarks.add: no url available',SLINGO.EWARN);
		
		post.title = document.title;
		if(!post.title) return SLINGO.triggerError('Bookmarks.add: no title available',SLINGO.EWARN);
		
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = 'Adding Quick Link, please wait...';
		
		d = new Date;
		req = 'addBookmark_'+d.getTime();
		SLINGO.XHR.createRequest(req, '/ajax/bookmarks/add/'+d.getTime(), 'POST', 'JSON', true, post);
		SLINGO.XHR.activateRequest(req, this._catchAdd);
		return true;
	},
	
	/* remove the specified bookmark from the user's saved bookmarks list */
	remove : function(id){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.remove(id);
		
		var ok = confirm('Are you sure you want to remove this Quick Link?');
		if(!ok) return;
		
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = 'Removing Quick Link, please wait...';
		
		/* get the items currently on the page */
		var c = this.pmgr.getCurrent();
		var current = [];
		for(var i in c)
			current.push(c[i]);
		
		/* remove the id from the list */
		for(var i in this.idList)
			if(this.idList[i] == id){
				delete this.idList[i];
				break;
			}
			
		/* remove the info from the current page */
		if(this.info[this.currentPage])
			for(var i in this.info[this.currentPage])
				if(this.info[this.currentPage][i].b == id)
					delete this.info[this.currentPage][i];
			
		/* delete all of the saved pages starting with the current page */
		for(var i in this.info)
			if(i > this.currentPage)
				delete this.info[i];
			
		/* reset the pagination object with the updated list of bookmark ids */	
		this._setPaginationMgr();
		
		/* build the new list of bookmarks that should appear on this page */
		var results = this.pmgr.getPage(this.currentPage);
		var usePrev = !results && (this.currentPage > 1);
		if(usePrev){
			/* the current page is empty, so let's get the previous page */
			this.currentPage--;
			
			/* if we don't already have the previous page saved, we have to retrieve the info for it */
			if(!this.info[this.currentPage])
				results = this.pmgr.getPage(this.currentPage);
		}
		
		if(results){
			var bids = [];
			for(var i in results)
				bids.push(results[i]);
				
			/* get all the items on the page */
			if(usePrev)
				results = bids.join(',');	
				
			/* otherwise just get the new last item on the page */
			else if(this.info[this.currentPage]){
				results = bids.pop();
				if(SLINGO.inArray(results, current))
					results = false;
				
			} else results = false;
		}
		
		/* build the form and send the ajax request */
		var post = {};
		post.bookmark_id = id;
		if(results) post.bookmark_ids = results;
	
		d = new Date;
		req = 'removeBookmark_'+d.getTime();
		SLINGO.XHR.createRequest(req, '/ajax/bookmarks/remove/'+d.getTime(), 'POST', 'JSON', true, post);
		SLINGO.XHR.activateRequest(req, this._catchRemove);
		return true;
	},
	
	/* handle the results of the "add bookmark" ajax request */
	_catchAdd : function(retv){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._catchAdd(retv);
		
		if(retv.error){
			if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = retv.error;
			else alert(retv.error);
			
			return SLINGO.triggerError('Bookmark.catchAdd: '+retv.error,SLINGO.EWARN);
		}
			
		/* If the bookmarks interface is open.. */
		if(SLINGO.$('bookmarkMsg')){
			SLINGO.$('bookmarkMsg').innerHTML = '';
			if(retv.b){
				if(SLINGO.inArray(retv.b, this.idList))
					var msg = 'You already have a Quick Link for this page.';
					
				else {
					/* add the new bookmark id to the id list */
					this.idList.push(retv.b);
					
					/* reset the pagination object with the updated list of bookmark ids */	
					this._setPaginationMgr();
					this.pmgr.setCurrPage(this.currentPage);
	
					/* add the new bookmark info to the last page if we already have it saved */
					last = this.pmgr.getTotalPages();
					if(last && this.info[last])
						this.info[last].push(retv);
					else if(last == 1){
						this.info[1] = [];
						this.info[1].push(retv);
					}
					
					var msg = 'Quick Link successfully added!';
				}
				
				SLINGO.$('bookmarkMsg').innerHTML = msg;
			}
			
			return this.showPage(this.currentPage);
			
		} else {
			var ok = confirm('Quick Link successfully added! Do you want to view your saved Quick Links?');
			if(ok) window.location.href = 'http://'+window.location.hostname+'/bookmarks';
			return true;
		}
	},
	
	/* handle the results of the "remove bookmark" ajax request */
	_catchRemove : function(retv){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._catchRemove(retv);
		
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = '';
		
		if(retv.error){
			if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = retv.error;
			return SLINGO.triggerError('Bookmark.catchRemove: '+retv.error,SLINGO.EWARN);
			
		} else if(retv.info){
			if(!this.info[this.currentPage]) this.info[this.currentPage] = [];
			this.info[this.currentPage].push(retv.info[0]);
		}
			
		if(SLINGO.$('bookmarkMsg')) SLINGO.$('bookmarkMsg').innerHTML = 'Quick Link successfully removed.';
		return this.showPage(this.currentPage);
	},
        /* handle the results of the "load bookmark ajax request */        
    _catchLoadBkMks : function(retv){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._catchLoadBkMks(retv);

		if(retv.error)
		   return SLINGO.triggerError('Bookmark._catchLoadBkMks: '+retv.error,SLINGO.EWARN);

		else if(!retv.bookmarks){
			if(SLINGO.$('bookmarkMsg'))
				SLINGO.$('bookmarkMsg').innerHTML = '';
			if(SLINGO.$('bookmarksBody'))
				SLINGO.$('bookmarksBody').innerHTML =  this.noBookMarksMsg;
		}else{
			 this.idList = retv.bookmarks;
			 this._loadFirstPage();
			 this.setPgMgr();
		}		
   },
	
	_getListCookie : function(){
            if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._getListCookie();
            return SLINGO.getCookie(this.listCookieName);
	},
	
	_getFirstPageCookie : function(){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks._getFirstPageCookie();
		return SLINGO.getCookie(this.firstPageCookieName);
	},
	
	/* get the image markup for a bookmark */
	getImage : function(id, img){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.getImage(id, img);
		
		if(!id)		return SLINGO.triggerError('Bookmarks.getImage: no id provided');
		if(!img)	return SLINGO.triggerError('Bookmarks.getImage: no img provided for: '+id);
		
		/* if this is a user pic */
		if(img.indexOf('/user_pics/') > -1){
			id = 'bookmarkImg_'+id;
			return '<img id="'+id+'" src="'+img+'" style="width:26px;height:23px;" onerror="javascript:SLINGO.Bookmarks.replaceUserPic(\''+id+'\')">';
		}
		switch(img){
			case 'games':
				return '<span class="bookmarkImg games"></span>';
			case 'community':
				return '<span class="bookmarkImg community"></span>';
			default:
				return '<span class="bookmarkImg"></span>';
		}
		
		return false;
	},
	
	/* replace a nonexistant user pic with the default user pic */
	replaceUserPic : function(imgId){
		if(this != SLINGO.Bookmarks) return SLINGO.Bookmarks.replaceUserPic(img);
		if(!SLINGO.$(imgId) || !SLINGO.$(imgId).src)
			return false;
		
		SLINGO.$(imgId).src = '/images/common/comm_noimage3.jpg';
		return true;
	}
	
}

/************************************************************************
/************************************************************************
/* Definitions of Bookmarks class constants
/************************************************************************
/************************************************************************/





/************************************************************************
/************************************************************************
/* Definitions of Bookmarks class methods
/************************************************************************
/************************************************************************/



/* OK, Bookmarks plugin finished. */
SLINGO.transitLoadComplete = true;