/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.gamepagedev.js */
/* @Author:  Nicole Violette */
/* @Contact: nicole at slingo dot com */
/* @Created: Mon Jul 11 14:08:16 EDT 2011 */
/* @Notes:   */
/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.GamePage)throw new Error('SLINGO.GamePage already defined');
/************************************************************************
/************************************************************************
/* GamePage Object Constructor
/************************************************************************
/* @PROTO:  RETURN SLINGO.GamePage(PARAMS)
/* @DESC:   FUNCTION_DESCRIPTION
/* @RETURN: RETURN_VALUE_DESCRIPTION
/* @PARAM:  PARAM_VALUE_DESCRIPTION
/************************************************************************
/************************************************************************/

/**
 * bind a function to an object
 * @param {obj} scope
 * @param {func} fn
 */
function bind(scope, fn) {
	return function(){
		fn.apply(scope, arguments);
	};
};

function bindWithArgs(scope, fn, args) {
	var argsToPass = args && args.length ? args : arguments;
	return function(){
		fn.apply(scope, argsToPass);
	};
};

function getStyle(el, cssprop){
    if (el.currentStyle) //IE
     return el.currentStyle[cssprop];
    else if (typeof getComputedStyle == 'function') //Firefox
     return getComputedStyle(el,null).getPropertyValue(cssprop);//document.defaultView.getComputedStyle(el, "")[cssprop];
    else if(el.style) //try and get inline style
     return el.style[cssprop];
	else {
		SLINGO.triggerError('getStyle:  could not get style');
		return false;
	}
}

/* The GamePage object starts the whole process off, and manages instances for the rest of the objects. */
SLINGO.GamePage = {

	_game_cycle_type: null,
	_game_cycle_mgr : null,
	_game_comm		: null,

	playerInfo		: null,
	gameConfig		: null,
	requestInfo		: null,
	intermissionStarted: false,
        testValue: null,
        
        
	/* Start the whole process off. */
	init : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.init();
		SLINGO.triggerError('GamePage.init :(dev plugin)',SLINGO.EWARN);
		if(this.initComplete) return;
		
		/* Load and process the game config information provided by the template. */
		this.loadConfig();
		
		/* Preload Stage test -NV 2012.07.11 */
		/*SLINGO.Game.isPreloadTest = (SLINGO.Game.gameId >= 161) ? true : false; */
		SLINGO.Game.isPreloadTest = true;	/* Enabled for testing for all games. -NV 2012.09.08 */
		SLINGO.triggerError('GamePage.init: isPreloadTest: '+SLINGO.Game.isPreloadTest,SLINGO.EWARN);

		/* Set up communication with the Flash game. */
		this.setGameComm();
		this.getGameComm().init(this._game_cycle_type);

		// Enable for all wrapperless games	
		// And Swedish Bingo
		//TESTING BEACON ON WRAPPER GAMES
        if(SLINGO.Game.isBeaconDisabled)
            SLINGO.Game.adBeaconWatchEnabled = false;
        else
            SLINGO.Game.adBeaconWatchEnabled = !this.hasWrapper() || SLINGO.Game.isSocketless;
        //SLINGO.Game.adBeaconWatchEnabled = !SLINGO.Game.isMulti || SLINGO.Game.isSocketless;
		/*
							SLINGO.Game.gameId == 41 ||	// Swedish Bingo 
							SLINGO.Game.gameId == 40 || 	// Mega Jackpot Bingo
							SLINGO.Game.gameId == 151 ||	// Ricochet
						
							SLINGO.Game.gameId == 31 ||	// Majong Tiles
							SLINGO.Game.gameId == 32 ||	// Slingo Mahjong
							SLINGO.Game.gameId == 46 ||	// Super Keno
							SLINGO.Game.gameId == 47 ||	// Triple Slingo Jackpot
							SLINGO.Game.gameId == 140 	// TriPeak Solitaire
							;
		*/
		/* If this isn't a game that uses a wrapper, we don't need to wait for the game communication to be
			initialized before we start the preroll. */
		/* if(!this.hasWrapper()) */
		this.startGameCycle();
        
        if(!SLINGO.Game.isSample){ 
            d = new Date();
            //here set the memcached variable that user is playing this game...
            if(!SLINGO.XHR.createAndSendRequest('setLPG'+d.getTime(), bind(this, this.catchSetLPG), '/ajax/sgs/setLPG/' + SLINGO.Game.gameId + '/' + SLINGO.Game.roomId + '/' + d.getTime(), 'GET', 'JSON', true))
                return SLINGO.triggerError('GameStage.initGameComm: unable to create a valid XHR request for setLPG', SLINGO.EWARN);
        }

		/* catch a backspace click and disable it */
		document.onKeyUp = document.onKeyDown = function(e){
			e = SLINGO.Event.getEvent(e);
			if(e.keyCode == 13)return false;
		};
		this.initComplete = true;

		var cookieVal = SLINGO.getCookie('slggaerrs');
		if(cookieVal.length){
			SLINGO.triggerError('SGS ERROR COOKIE: ' + cookieVal, SLINGO.EWARN);
			this.gaErrVars = cookieVal.split(',');
		}
		var warnings = [];
		//flash is not enabled or not most recent version
		if(!SLINGO.Client.Info.flashEnabled || SLINGO.Client.FlashDetect.major < 12){
			switch(SLINGO.Client.Info.browser){
				case "GoogleChrome":
					var warning = {};
					warning.item = 'Flash';
					warning.url = 'http://helpx.adobe.com/flash-player/kb/flash-player-google-chrome.html';
					warning.text = 'Update Chrome to get the latest version of Flash Player!';
					warnings.push(warning);
					break;
				case "Safari":
					//do not show warnings for safari
					break;
				default:
					var warning = {};
					warning.item = 'Flash';
					warning.url = 'http://get.adobe.com/flashplayer/';
					warning.text = false;
					warnings.push(warning);					
			}
		}		

		/* IE OUT OF DATE BROWSER DETECTION  */
		//SLINGO.Client.Info.browserVersion = '7';
		if(SLINGO.Client.Info.browser == 'IE' && parseInt(SLINGO.Client.Info.browserVersion) != '11'){
			//just showing warnings on slingo.com
			if(SLINGO.Game.site == 'www'){
				var warning = {};
				var isXP = (SLINGO.Client.Info.platform == 'Windows XP') ? 1 : 0;
				if(!isXP){
					warning.url = 'http://windows.microsoft.com/en-us/internet-explorer/download-ie';
				}else{
					warning.text = 'Get the latest browser by downloading ';
					//xp users, show urls to download ff or chrome
					warning.url = {'Chrome' : 'https://www.google.com/chrome/browser/#eula', 'Firefox' : 'https://www.mozilla.org/en-US/firefox/new/'};
				}		
				if(warning.url){
					warning.item = 'Internet Explorer';
					warnings.push(warning);
				}
			}			
		}
		this.showWarnings(warnings);
		/* TODO - google analytic events */
	},

	/* Load config information provided by the game page template. */
	loadConfig : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.loadConfig();

		if(this.playerInfo)
			for(var i in this.playerInfo)
				SLINGO.Player[i] = this.playerInfo[i];

		if(this.gameConfig)
			for(var i in this.gameConfig)
				SLINGO.Game[i] = this.gameConfig[i];

		/* Information about the request that we can pass along in the event of an error. */
		if(this.requestInfo)
			for(var i in this.requestInfo)
				SLINGO.GameRequest[i] = this.requestInfo[i];
		
		/* Load elements to handle in game stuff */
		/* @@@
		if (this.inGameEndScreen)
			for (var i in this.inGameEndScreen)
				SLINGO.inGameEndScreen[i] = this.inGameEndScreen[i];
		
		if (this.inGameLeaderboard)
			for (var i in this.inGameLeaderboard)
				SLINGO.inGameLeaderboard[i] = this.inGameLeaderboard[i];
		*/

		/* Determine what game cycle type we should adhere to.
			TODO - this needs to be handled better, but for now... */
		if(this.gameConfig.isBlast)				this._game_cycle_type = 'BLAST';
		else if(this.gameConfig.isAdvantage)	this._game_cycle_type = 'ADVANTAGE';
		else if(this.gameConfig.isSolitaireDC)	this._game_cycle_type = 'SOLITAIREDC';
		else if(this.gameConfig.isSDC)			this._game_cycle_type = 'SDC';
		else if(this.gameConfig.gameId == 162)  this._game_cycle_type = 'FIVECARDDC';
		else if(this.gameConfig.gameId == 161)  this._game_cycle_type = 'TWIKIDC';
		else if(this.gameConfig.gameId == 168)  this._game_cycle_type = 'ALLSTARSDC';
		else if(this.gameConfig.gameId == 169)  this._game_cycle_type = 'ISLANDDC';
		else if(this.gameConfig.gameId == 170)  this._game_cycle_type = 'CHIPFLIPDC';
		else if(this.gameConfig.gameId == 180)	this._game_cycle_type = 'ZENMATCHDC';
		else if(this.gameConfig.gameId == 181)	this._game_cycle_type = 'DEEPSEADC';
		else if(this.gameConfig.gameId == 190)	this._game_cycle_type = 'PACHINKODC';
		else if(this.gameConfig.gameId == 191)	this._game_cycle_type = 'SDCONFB';
		//else if(this.gameConfig.gameId == 184)	this._game_cycle_type = 'GENDC';
		else if(this.gameConfig.gameId == 174)	this._game_cycle_type = 'MATCHMAKERDC';
		else if(this.gameConfig.gameId == 197)	this._game_cycle_type = 'SPELLBUGSDC';
		else if(this.gameConfig.gameId == 201)	this._game_cycle_type = 'WORDROUNDUPDC';
		else if(this.gameConfig.gameId == 202)	this._game_cycle_type = 'WORDFRENZYDC';
		else if(this.gameConfig.gameId == 203)	this._game_cycle_type = 'PICTURIFFICDC';
		/*else if(SLINGO.inArray(this.gameConfig.gameId, [167]))			this._game_cycle_type = 'BLITZDC';*/
		else if(SLINGO.inArray(this.gameConfig.gameId, [163,164,165])) 	this._game_cycle_type = 'ALTGENDC';
		else if(this.gameConfig.isDailyChallenge)	this._game_cycle_type = 'GENDC';
		else if(this.gameConfig.isWrapperless)	this._game_cycle_type = 'WRAPPERLESS';
		else if(this.gameConfig.isSocketless)	this._game_cycle_type = 'SOCKETLESS';
		else 									this._game_cycle_type = 'SGS';
	},

	/* This function will be called once the game swf has been embedded on the page (BUT NOT NECESSARILY finished loading).
		Now we can start trying to communicate with the game and make sure all the functions we need are accessible. */
	initGameSwf : function(e){
		if(this != SLINGO.GamePage) return SLINGO.GamePage.initGameSwf(e);
		SLINGO.triggerError('GamePage.initGameSwf',SLINGO.EWARN);
		
		if(SLINGO.Game.isPreloadTest)
			return SLINGO.triggerError('GamePage.initGameSwf: bypassing initGameSwf for dev',SLINGO.EWARN);

		/*
		if(!e.success)
			return SLINGO.triggerError('GamePage.initGameSwf: game swf was not loaded',SLINGO.EWARN);

		if(this.hasWrapper())
			this.getGameComm().setInitdCallback(SLINGO.GamePage.startGameCycle);

		var result = this.getGameComm()._initRelayComms();
		return result;
		*/ 
	},
	
	/* This function will be called once the wrapper/Platform Adapter has signalled that it is 
		ready for page-game communication. */
	initGameComm : function(){
		if(this != SLINGO.GamePage) return SLINGO.GamePage.initGameComm();
		SLINGO.triggerError('GamePage.initGameComm',SLINGO.EWARN);

		var result = this.getGameComm()._initRelayComms();

		if(SLINGO.GamePage.intermissionStarted)
			SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
        
    	return result;
	},

	/* Determine if this game is using the Flash wrapper. */
	hasWrapper : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.hasWrapper();
		return SLINGO.inArray(this._game_cycle_type, ['SGS','SOCKETLESS']);
	},

	/* Set up the game communication object instance. */
	setGameComm : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.setGameComm();
		this._game_comm = SLINGO.GameComm;
	},

	/* Get the game communication object instance. */
	getGameComm : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.getGameComm();
		return this._game_comm;
	},

	/* Create an instance of the game cycle manager. This is the object that runs the show. */
	setGameCycleMgr : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.setGameCycleMgr();
		this._game_cycle_mgr = new SLINGO.GameCycleMgr();
		this._game_cycle_mgr.setCycleType(this._game_cycle_type);
	},

	/* Get the game cycle manager instance. */
	getGameCycleMgr : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.getGameCycleMgr();
		return (this._game_cycle_mgr && (this._game_cycle_mgr instanceof SLINGO.GameCycleMgr))
			? this._game_cycle_mgr
			: false;
	},

	/* Returns the game cycle type. Possible values are: SGS, SOCKETLESS, WRAPPERLESS, SDC, ADVANTAGE, BLAST */
	getGameCycleType : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.getGameCycleType();
		return this._game_cycle_type;
	},

	/* This is used to move the game cycle along from the global scope. This should only be used if the game cycle
		needs to be moved to the next state based on something happening from an outside source. For example, in socketed games
		the cycle is controlled by the SGS. */
	setGameCycleState : function(state){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.setGameCycleState();
		var mgr = this.getGameCycleMgr();
		return mgr
			? mgr.setState(state)
			: SLINGO.triggerError('GamePage.setGameCycleState: could not get game cycle manager',SLINGO.EWARN);
	},

	/* Start the game cycle.. wheeeee. */
	startGameCycle : function(){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.startGameCycle();
		SLINGO.triggerError('GamePage.startGameCycle',SLINGO.EWARN);
		this.setGameCycleMgr();
		this.setGameCycleState('PREROLL_START');
	}
	,	
	showFlashWarning: function() {
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.showFlashWarning();
		root = SLINGO.$('gameContainer');
		if(!root)
			return SLINGO.triggerError('SLINGO.GamePage.showFlashWarning: Unable to show warning, root element not found!',SLINGO.EWARN);
		div = document.createElement('div');
		div.id = 'flashVersionWarning';
		div.className = 'versionWarning';
		div.innerHTML = "Oops! It looks like you don't have the latest version of Flash installed. <a target='_blank' href='http://get.adobe.com/flashplayer/'>Click Here to Install!</a>";
		root.parentNode.insertBefore(div,root);
		//root.appendChild(div);
	},
			
	showWarnings : function(warnings){
		if(this !== SLINGO.GamePage) return SLINGO.GamePage.showWarnings(warnings);
		
		if(warnings.length > 0){
			root = SLINGO.$('gameContainer');
			if(!root)
				return SLINGO.triggerError('SLINGO.GamePage.showOutOfDateBrowser: Unable to show warning, root element not found!',SLINGO.EWARN);
			
			var div = document.createElement('div');
			div.id = 'outOfDateWarning';
			div.className = 'versionWarning';
			var curSite = (SLINGO.Game.site == 'gamehouse') ? '' : 'Slingo ';
			div.innerHTML = "Oops! It looks like you can get an even better " +  curSite + "experience by downloading the following:";
			for(var warning in warnings){
				if(warning != 'xfind'){
					//create paragraph
					var p = document.createElement('p');
					if(warnings[warning].text){
						p.innerHTML = warnings[warning].text;
						if(typeof warnings[warning].url === 'object'){
							var urls = warnings[warning].url;
							var urlTags = [];
							for(var i in urls){
								urlTags.push(" <a target='_blank' href='" + urls[i] + "' >" + i + "</a> ");
								//p.innerHTML += " <a target='_blank' href='" + urls[i] + "' >" + i + "</a> " ;
							}
							p.innerHTML += urlTags.join(" or ");
						}else
							p.innerHTML += " <a target='_blank' href='" + warnings[warning].url + "' >Click here for more info</a>" ;
					}else{
						p.innerHTML = "Get the latest version of " + warnings[warning].item +" <a target='_blank' href='"+ warnings[warning].url +"'>By clicking Here!</a>";
					}					
					div.appendChild(p);
				}				
			}
			root.parentNode.insertBefore(div,root);
		}
		
	},
    
    catchSetLPG : function(retv){
       if(retv.error)
           SLINGO.triggerError('SLINGO.GamePageDev.catchSetLPG error setting LPG var', SLINGO.EWARNING);
	},
	
	getDefaultChallengePrefix : function() {
		if(typeof SLINGO.Game.dailyChallenges != "undefined") {
			var dc = SLINGO.Game.dailyChallenges.challenges;
			for(var key in dc) {
				if(dc[key].isDefaultChallenge)
					return key;
			}
		}
		return false;
	}
};

/* Manages information about the player. */
SLINGO.Player = {

	gamesPlayed : 0,
	lastScore : null,

	getPlayerId : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getPlayerId();
		return this.playerId;
	},

	getPlayerToken : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getPlayerToken();
		return this.playerToken;
	},

	getPlayerName : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getPlayerName();
		return this.playerName;
	},

	getPlayerBux : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getPlayerBux();
		return this.playerBux;
	},
	
	setPlayerBux : function(bux){
		if(this != SLINGO.Player) return SLINGO.Player.setPlayerBux(bux);
		if(!bux) return SLINGO.triggerError('Player.setPlayerBux: bux not provided',SLINGO.EWARN);
		this.playerBux = bux;
		if(SLINGO.isPluginLoaded('UserTools')) SLINGO.UserTools.updateBuxView(bux.toString());
	},

	getGamesPlayed : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getGamesPlayed();
		return this.gamesPlayed;
	},

	incrGamesPlayed : function(){
		if(this != SLINGO.Player) return SLINGO.Player.incrGamesPlayed();
		this.gamesPlayed++;
	},

	getLastScore : function(){
		if(this != SLINGO.Player) return SLINGO.Player.getLastScore();
		return this.lastScore;
	},

	setLastScore : function(score){
		if(this != SLINGO.Player) return SLINGO.Player.setLastScore(score);
		this.lastScore = score;
	},

	isInit : function(){
		return !this.playerId ? false : true;
	}

};

/* Manages information about the game. Currently this all comes from the game page template. */
SLINGO.Game = {

};

/* Manages information about the page request. */
SLINGO.GameRequest = {

	/* The timestamp of the last start game request. */
	startGameReqTime : false,

	/* Get the timestamp of when the page was first loaded. */
	getPageReqTimestamp : function(){
		if(this !== SLINGO.GameRequest) return SLINGO.GameRequest.getPageReqTimestamp();
		return this.pageReqTime;
	},

	/* Determine what time it is on the server. */
	getCurrServerTimestamp : function(){
		if(this !== SLINGO.GameRequest) return SLINGO.GameRequest.getCurrServerTimestamp();

		var serverOffset = this.timezoneOffset;
		if(!serverOffset) return SLINGO.triggerError('GameRequest.getCurrServerTimestamp: no offset provided',SLINGO.EWARN);

		var d = new Date;
		var clientOffset = -d.getTimezoneOffset()/60;

		var diff = clientOffset - serverOffset;
		d.setHours(d.getHours()-diff);
		return d.getTime();
	}
	
};

/* Manages communication with the Flash game. */
SLINGO.GameComm = {

	/* properties */
	commInitCnt	: false,

	/* flags */
	objInit			: false,
	initComplete	: false,
	commsOpen		: false,

	/* page and SGS state flags */
	pgState 		: 'PREROLL',
	prevPgState 	: 'PREROLL',
	sgsState 		: 'PREROLL',

	/* The type of game communcation to use; based on the cycle type */
	commType		: null,

	/* Determine if the game is ready for communication */
	gameReady		: false,
	gameReadyCount	: 0,
	maxReadyCount	: 450,	/* Changed from 60 to 450 (15 min) on 2012.09.08 -NV */

	/* Function to call once game communction is initialized */
	initdCallback 	: null,

	/* poll to check for the start game acknowledgement from the Platform Adapter */
	startGamePoll : null,

	/* true if the Platform Adapter has acknowledged that the game has started */
	gameStarted : false,

	/* the number of times we've checked to see if the Platform Adapter has acknowledged start game */
	gameStartChecks : 0,

	/* the max number of times to check if the Platform Adapter has acknowledged start game */
	maxGameStartChecks : 10,

	/* an object containing arrays of messages sent from the Platform Adapter */
	platformMsgs : {},

	/* node to store progress text */
	_progTxt	: '',
			
	_adsDoneInt : null,
	_setEndGameOptionInt : null,
	_enableEndGameOptionInt : null,
	endGameOptionEnabled : false,
	score : false,
	
	/* initialize this comms object */
	init : function(type){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.init(type);
		if(!type)
			return SLINGO.triggerError('SLINGO.GameComm.init: no game comm type specified',SLINGO.EWARN);

		this.commType = type;
		this.objInit = true;
		this.initComplete = true;
		//this._progText = SLINGO.$('progTxtTarg');
		// If a canvas tag id #gameElement exists, we're working with an HTML5 game
		var gameElementNode = SLINGO.$('gameElement');
		if(gameElementNode && gameElementNode.tagName.toUpperCase() == 'CANVAS') {
			this.initCanvasInterface(gameElementNode);
		}
		
		return true;
	},

	/* Set the function to call once communication with the game has been successful. */
	setInitdCallback : function(fn){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.setInitdCallback(fn);
		if(typeof fn === 'function')
			this.initdCallback = fn;
	},

	/* Determine if the game is ready to be started. */
	isGameReady : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.isGameReady();
		SLINGO.triggerError('GameComm.isGameReady: attempt #'+this.gameReadyCount, SLINGO.EWARN);
		
		switch(this.commType){
			case 'SGS':
				return this.gameReady;

			default:
				var rdy = this.getGameVar('game_ready');
				return SLINGO.Game.isPreloadTest
					? (this.gameReady && ((rdy == '1') || (rdy == 'true')))		/* Make sure game communications have been established as well. Added 2012.05.16 -NV */
					: ((rdy == '1') || (rdy == 'true'));
		}
	},

	/* Determine if the game has announced that it's over. */
	isGameOver : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.isGameOver();
		var ovr = this.getGameVar('game_over');
		return ((ovr == '1') || (ovr == 'true'));
	},

	/* Start checking every x seconds to determine if the game has announced that it's over. */
	startGameOverPoll : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.startGameOverPoll();
		if(this.gameOverPoll){
			clearInterval(this.gameOverPoll);
			this.gameOverPoll = null;
		}

		/*this.gameOverPoll = setInterval(bind(this, this.pollGameOver), 2000);*/
		this.gameOverPoll = setTimeout(bind(this, this.pollGameOver), 2000);	/* Changed 2012.08.27 -NV */
	},

	/* This function is called during an interval that checks every x seconds to see if the game is over. Once the
		game is over, clear the interval and set the game cycle state accordingly so we can move along. */
	pollGameOver : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.pollGameOver();
		if(this.gameOverPoll){
			clearTimeout(this.gameOverPoll);
			this.gameOverPoll = null;
		}

		if(SLINGO.GameComm.isGameOver())
			return SLINGO.GamePage.setGameCycleState('GAME_OVER');
		else this.gameOverPoll = setTimeout(bind(this, this.pollGameOver), 2000);
	},

	/* THIS SHOULD BE USED FOR DEBUGGING PURPOSES ONLY!!! */
	endGameEarly : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.endGameEarly();
		clearInterval(this.gameOverPoll);
		this.gameOverPoll = null;
		return SLINGO.GamePage.setGameCycleState('GAME_OVER');
	},

	/* For games using the Platform Adapter, retrieve a game variable. */
	getGameVar : function(nm){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.getGameVar(nm);
		if(!this.getFlashMovie())
			return SLINGO.triggerError('GameComm.getGameVar: flash movie not yet defined',SLINGO.EWARN);

		if(this.gameReady) {			
			if('slingoGetVar' in this.getFlashMovie()) {
				value = this.getFlashMovie().slingoGetVar(nm);
				return value;
			}
			if('GetVariable' in this.getFlashMovie()) {
				value = this.getFlashMovie().GetVariable(nm);
				return value;
			}
			
			return SLINGO.triggerError('GameComm.getGameVar: No getGameVar function in game obj, trying to retrieve: '+nm,SLINGO.EWARN);
		} else
			return SLINGO.triggerError('GameComm.getGameVar: Game not ready!');
		
			
	},

	/* For games using the Platform Adapter, set a game variable. */
	setGameVar : function(nm, val){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.setGameVar(nm, val);
		
		SLINGO.triggerError('GameComm.setGameVar: name: '+nm+', val: '+val,SLINGO.EWARN);

		if('slingoSetVar' in this.getFlashMovie()){
			var retval = this.getFlashMovie().slingoSetVar(nm, val);
			return true;
		}

		if('SetVariable' in this.getFlashMovie())
			return this.getFlashMovie().SetVariable(nm, val);

		return SLINGO.triggerError('No setGameVar function in game obj',SLINGO.EWARN);
	},

	/* Keep checking for an acknowledgement from the game that it successfully started. If we didn't get the acknowledgement,
		try starting it again. */
	startGame : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.startGame();
		if(this.startGamePoll) clearTimeout(this.startGamePoll);
		if(!this.gameStarted){
			if(this.gameStartChecks > this.maxGameStartChecks){
				SLINGO.triggerError('GameComm.startGame: max game start checks exceeded, cannot start game:\ngame_ready: '+SLINGO.GameComm.getGameVar('game_ready')+'\nstart_game: '+SLINGO.GameComm.getGameVar('start_game')+'\ngame_over: '+SLINGO.GameComm.getGameVar('game_over')+'\nreset_game: '+SLINGO.GameComm.getGameVar('reset_game'),SLINGO.EWARN);
				/*alert('There was an error connecting to the game. Please click "OK" to refresh the page and try again.');
				return window.location.reload();*/
			} else {
				SLINGO.triggerError('GameComm.startGame: attempting to start game: #'+this.gameStartChecks,SLINGO.EWARN);
				this.setGameVar('start_game', '1');
				this.startGamePoll = setTimeout(this.startGame, 2000);
				this.gameStartChecks++;
			}
		} else {
			SLINGO.triggerError('GameComm.startGame: game started acknowledgement received',SLINGO.EWARN);
			this.gameStarted = false;
			this.gameStartChecks = 0;
		}
	},

	/* Tell the game to turn the music back on or off (depending on if it's currently off or on). */
	toggleGameMusic : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.toggleGameMusic();
		return this.setGameVar('music_toggle','1');
	},

	/* Tell the game to turn the sound effects on or off (depending on if they're currently off or on). */
	toggleGameSfx : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.toggleGameSfx();
		return this.setGameVar('sfx_toggle','1');
	},
	
	/* Allow the game to trigger intermission if the Platform Adapter API version is 2+ */
	playAgain : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.playAgain();
		SLINGO.triggerError('GameComm.playAgain called',SLINGO.EWARN);
		var slingoApiVer = this.getApiVersion();
		if(slingoApiVer < 2) return SLINGO.triggerError('GameComm.playAgain: invalid api version: '+slingoApiVer,SLINGO.EWARN);;
		return SLINGO.GamePage.setGameCycleState('GAME_PLAY_AGAIN'); // used to be GAME_END
	},
	
	/* Get the API version of the Platform Adapter. */
	getApiVersion : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.getApiVersion();
		var ver = this.getGameVar('slingo_api_ver');
		return parseInt(ver) ? ver : 1;
	},

	/* Catch updates from the socketed wrapper in order to move the game cycle along. */
	relay_updateState : function(state){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.relay_updateState(state);
		state = state.toUpperCase();
		this.sgsState = state;
		var runAction = this.commType == 'SGS';
		switch(this.pgState){
			case 'PREROLL':
				switch(state){
					case 'PREROLL':
						SLINGO.triggerError('state = PREROLL',SLINGO.EWARN);
					case 'INTERMISSION': break;
					case 'INPROGRESS':
						this.pgState = state;
						if(runAction) {
							SLINGO.GamePage.getGameCycleMgr().addWatch('PRELOAD_END', function(){SLINGO.GamePage.setGameCycleState('GAME_START')});
							SLINGO.GamePage.setGameCycleState('PREROLL_END');
						}
						break;
					case 'INLOBBY':
						this.pgState = state;
						SLINGO.GamePage.getGameCycleMgr().addWatch('PRELOAD_END', function(){SLINGO.GamePage.setGameCycleState('LOBBY_START')});
						SLINGO.GamePage.setGameCycleState('PREROLL_END');
						break;
				}
				break;
			case 'INTERMISSION':
				switch(state){
					case 'PREROLL':
					case 'INLOBBY':
					case 'INTERMISSION':
						this.pgState = state;
						SLINGO.triggerError('GameComm.relay_updateState: illegal state passed as argument: '+state+', currState set as: '+this.pgState,SLINGO.EWARN);
						break;
					case 'INPROGRESS':
						this.pgState = state;
						if(runAction)
							SLINGO.GamePage.setGameCycleState('INTERMISSION_END');
						break;
				}
				break;
			case 'INPROGRESS':
				switch(state){
					case 'PREROLL':
					case 'INPROGRESS':
					case 'INLOBBY':
						this.pgState = state;
						SLINGO.triggerError('GameComm.relay_updateState: illegal state passed as argument:'+state+', currState set as: '+this.pgState,SLINGO.EWARN);
						break;
					case 'INTERMISSION':
						this.pgState = state;
						if(runAction){
								SLINGO.GamePage.setGameCycleState('GAME_END');	
								SLINGO.GamePage.getGameCycleMgr().getStageMgr().getChild('GAME')._hideEndGameOverlay(true);
						}
					break;
					case 'INACTIVE':
						SLINGO.GamePage.setGameCycleState('INACTIVE');
						break;
				}
				break;
			case 'INLOBBY':
				switch(state){
					case 'PREROLL':
					case 'INTERMISSION':
					case 'INLOBBY':
						this.pgState = state;
						SLINGO.triggerError('GameComm.relay_updateState: illegal state passed as argument: '+state+', currState set as: '+this.pgState,SLINGO.EWARN);
						break;
					case 'INPROGRESS':
						this.pgState = state;
						if(runAction)
							SLINGO.GamePage.setGameCycleState('LOBBY_END');
						break;
				}
				break;
			default : SLINGO.triggerError('Gamecomm.relay_updateState: illegal state passed as argument: '+state+', currState set as: '+this.pgState,SLINGO.EWARN);
		}
		return true;
	},

	/* Catch updates from the socketed wrapper on the game loading progress so we can update the preroll/intermission progress bar. */
	relay_updateProgress : function(prog){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.relay_updateProgress(prog);
		if(!prog || this.commType != 'SGS') return;
		//SLINGO.$('progTxtTarg').innerHTML = prog;
		//this._progTxt.innerHTML = prog;

		if(prog != parseInt(prog)) {
			SLINGO.$('progTxtTarg').innerHTML = prog;
			this._progTxt = prog;
		}
		
		var mgr = SLINGO.GamePage.getGameCycleMgr();
		if('updatePBar' in mgr) return mgr.updatePBar(prog);
		return true;
	},	
	/* Catch an error from the socketed wrapper (or anywhere else, really) so we can create a ticket and redirect the user. */
	relay_reportError : function(msg){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.relay_reportError(msg);

		/* using analytics to track the profiles of different errors */
		eType = msg.match(/SGS-GENERATED TICKET <TYPE=(\w+)>/);

		if(eType && eType[1].length){
			this._trackGAError(eType[1]);
		}else SLINGO.triggerError('COULD NOT GET SGS ERROR TYPE',SLINGO.EWARN);

		var f = document.createElement('form');
		f.method = 'post';
		f.action = '/support/sgs_error';

		var pvals = {
			/*objInitd		: this.objInitd,
			stageInitd		: this.stageInitd,
			prollInitd		: this.prollInitd,
			interInitd		: this.interInitd,
			wraprInitd		: this.wraprInitd,*/
			commsOpen		: this.commsOpen,
			pgState			: this.pgState,
			prevPgState		: this.prevPgState,
			sgsState		: this.sgsState,
			plyrId			: SLINGO.Player.playerId,
			isMulti			: SLINGO.Game.isMulti,
			isWide			: SLINGO.Game.isWide,
			isDaily			: false, // This doesn't exist anywhere - RS
			isSample		: SLINGO.Game.isSample,
			isTorny			: SLINGO.Game.isTourn,
			roomHref		: location.href,
			roomId			: SLINGO.Game.roomId,
			gameId			: SLINGO.Game.gameId,
			token			: SLINGO.Player.playerToken,
			usrName			: SLINGO.Player.playerName,
			isSocketless	: SLINGO.Game.isSocketless,
			isWrapperless	: SLINGO.Game.isWrapperless,
			isDailyChal		: SLINGO.Game.isDailyChallenge,
			pageReqTime		: SLINGO.GameRequest.pageReqTime,
			startGameReqTime: SLINGO.GameRequest.startGameReqTime,
			gamesPlayed		: SLINGO.Player.gamesPlayed,
			commType		: this.commType,
			commInitCnt		: this.commInitCnt,
			gameReady		: this.gameReady,
			gameReadyCount	: this.gameReadyCount
		};
		
		if(SLINGO.Game.isPreloadTest) pvals.dlProgress = this.getFlashPercentLoaded();

		var arr = [];
		for(p in pvals) arr.push(p+'='+pvals[p]);

		var i = document.createElement('input');
		i.type = 'hidden';
		i.name = i.id = 'pageValues';
		i.value = arr.join('&');
		f.appendChild(i);

		var i = document.createElement('input');
		i.type = 'hidden';
		i.name = i.id = 'gameId';
		i.value = this.isDailyChal ? 'sdc' : SLINGO.Game.gameId;
		f.appendChild(i);

		var i = document.createElement('input');
		i.type = 'hidden';
		i.name = i.id = 'message';
		i.value = msg;
		f.appendChild(i);

		/* add in the reqported errors from the slingo object of any are available */

		if(SLINGO.isPluginLoaded('Client')){
			var d = document.createElement('div');
			d.appendChild(SLINGO.Client.renderInfo());
			document.getElementsByTagName('body')[0].appendChild(d);

			var i = document.createElement('input');
			i.type = 'hidden';
			i.name = i.id = 'clientInfo';
			i.value = d.innerHTML;
			f.appendChild(i);
		}

		document.getElementsByTagName('body')[0].appendChild(f);
		f.submit();
	},
	/* this is what swf calls to update the leaderboard */
	relay_updateLeaderboard : function(reqObj){
		SLINGO.triggerError('SLINGO.GameComm.relay_updateLeaderboard',SLINGO.EWARN);
        if(SLINGO.Game.isFBCanvas)
            SLINGO.InGameLeaderboardElement.getFBLeaderboard(reqObj);
        else
            SLINGO.InGameLeaderboardElement.getLeaderboard(reqObj);
	},

    /* this is what swf calls to send app request */
	relay_sendAppReq : function(){
		SLINGO.triggerError('SLINGO.GameComm.relay_sendAppReq',SLINGO.EWARN);
		if(typeof SLINGO.Facebook == 'object' && SLINGO.Game.isFBCanvas){
            switch(SLINGO.Game.gameId){
                case 191:
                    var title = "Play Slingo Daily Challenge";
                    var msg = "Check out this great game and see if you can beat the daily challenge!!";
                    break;
                default:
                    SLINGO.triggerError('SLINGO.GameComm.relay_sendAppReq no title/msg sent for game app req',SLINGO.EWARN);
                    return false;
                    
            }
        }else {
            SLINGO.triggerError('SLINGO.GameComm.relay_sendAppReq game is not fb canvas or facebook obj not found',SLINGO.EWARN);
            return false;
        } 
         
        SLINGO.Facebook.fbSendAppRequest(title, msg);
        
	},

	/* Catch messages from the Platform Adapter. */
	handlePlatformMsg : function(key, val){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.handlePlatformMsg(key, val);
		SLINGO.triggerError('GameComm.handlePlatformMsg: key: '+key+', val: '+val.toString(),SLINGO.EWARN);

		var isDebug = (key.toUpperCase() == 'PLATFORM_DEBUG');
		if(!isDebug) this.addPlatformMsg(key, val);
		switch(key.toUpperCase()){
			case 'ACK_SET':
				if(val.toLowerCase() == 'start_game'){
					this.gameStarted = true;
					SLINGO.triggerError('SGSSocketless.handlePlatformMsg: START_GAME_ACK',SLINGO.EWARN);
				}
				break;

			case 'PLATFORM_DEBUG':
				SLINGO.GameMsg.send(key + val);
				break;
		}
	},

	/* Get an array of the messages sent out by the Platform Adapter. */
	getPlatformMsgs : function(typ){
		if(this != SLINGO.GameComm) return SLINGO.GameComm.getPlatformMessages(typ);
		if(!this.platformMsgs) return false;
		if(typ && !(typ in this.platformMsgs)) return false;
		return typ ? this.platformMsgs[typ] : this.platformMsgs;
	},

	/* Store messages sent out by the Platform Adapter in an array. */
	addPlatformMsg : function(key, val){
		if(!this.platformMsgs) this.platformMsgs = {};
		if(!(key in this.platformMsgs)) this.platformMsgs[key] = [];
		this.platformMsgs[key].push(val);
		return true;
	},
	
	/* END PUBLIC INTERFACE */

	/* Make sure we can communicate with the game, and that the game has the necessary functions for us to access. */
	_initRelayComms : function(){
		if(this != SLINGO.GameComm) return SLINGO.GameComm._initRelayComms();
		SLINGO.triggerError('GameComm._initRelayComms',SLINGO.EWARN);

		/*var swf = this.getFlashMovie();
		if(!swf) return SLINGO.triggerError('SLINGO.GameComm._initRelayComms: failed to register relay functions. Game Info Object not found',SLINGO.EWARN);*/

		var regFn = false;
		var callFn = false;
		switch(this.commType){
			case 'SGS':
			case 'SOCKETLESS':
				regFn = 'sgs_RegisterJavascriptMethod';
				callFn = 'sgs_CallActionscriptMethod';
				
				break;

			default:
				regFn = 'pa_RegisterJavascriptMethod';
				callFn = false;
				break;
		}
		if(!regFn)
			return SLINGO.triggerError('SLINGO.GameComm._initRelayComms: no register fn available for comm type: '+this.commType,SLINGO.EWARN);

		var swf = this.getFlashMovie();
		if(!swf || !(regFn in swf)){
			SLINGO.triggerError('SLINGO.GameComm._initRelayComms: register method not available from game object',SLINGO.EWARN);
			if(!this.commInitCnt)
				this.commInitCnt = 1;
			if(this.commInitCnt <= 60){
				SLINGO.GameMsg.send(regFn+' not available: check #' + this.commInitCnt);
				++this.commInitCnt;
				setTimeout('SLINGO.GameComm._initRelayComms()', 1000);
				return;
			} else {
				SLINGO.GameMsg.send('Relay Bug Method: '+regFn+' not available to the page.. Processing cannot continue.');
				/*alert('Could not connect to the game. Please click "OK" to refresh the page and try again.');
				return window.location.reload();*/
			}
		} else {
			SLINGO.triggerError('SLINGO.GameComm._initRelayComms: register method is now available',SLINGO.EWARN);
			swf[regFn]("SLINGO.GameComm.relay_sgsAck", "js_SGSCallAck");
			swf[regFn]("SLINGO.GameComm._sendDbgMsg","js_alert");
			swf[regFn]("SLINGO.GameComm.relay_updateState", "js_UpdateGameState");
			swf[regFn]("SLINGO.GameComm.relay_updateProgress","js_UpdateGameProgress");
			swf[regFn]("SLINGO.GameComm.relay_reactivateWrapper", "js_ReActivateWrapper");
			swf[regFn]("SLINGO.GameComm.relay_reportError", "js_ReportError");
			swf[regFn]("SLINGO.GameComm.handlePlatformMsg", "sendMsgToPlatform");
			swf[regFn]("SLINGO.GameComm.playAgain", "js_PlayAgain");
			swf[regFn]("SLINGO.GameComm.relay_updateLeaderboard", "js_updateLeaderboard");
            swf[regFn]("SLINGO.GameComm.relay_updateGameState", "js_SendScore");
			swf[regFn]("SLINGO.GameComm.relay_sendAppReq", "js_SendAppReq");
			// In-game leaderboard set up
			var isLBEnabled = (SLINGO.Game.isSample) ? (SLINGO.Game.isFBCanvas) ? '1' : '0' : '1';
			SLINGO.GameComm.setGameVar('leaderboard_enable', isLBEnabled);

			if(!(callFn in swf)){
				SLINGO.triggerError('SLINGO.GameComm._initRelayComms: call method not available from game',SLINGO.EWARN);
			} else {
				if(SLINGO.Player.isInit()){
					swf[callFn]("setPlayerId", SLINGO.Player.getPlayerId());
					swf[callFn]("setPlayerToken", SLINGO.Player.getPlayerToken());
				} else {
					swf[callFn]("setPlayerId", '000000');
					swf[callFn]("setPlayerToken", 'XXXXXXXXXXXX');					
				}
				
				var cycleType = SLINGO.GamePage.getGameCycleType();
				if(cycleType === 'SGS'){
					if(!SLINGO.GamePage.getGameComm().endGameOptionEnabled){
						this.setInterVars('_enableEndGameOptionInt', swf, swf[callFn], ["enableEndGameOption", "Y"]);
					}
						
				}		
			}
			this.commsOpen = true;
			this.gameReady = true;
			if(typeof this.initdCallback === 'function')
				this.initdCallback();
		}
	},
			
			
	relay_updateGameState : function(score){
		if(SLINGO.GamePage.getGameCycleType() == 'SGS'){
			this.score = (score);
			SLINGO.GamePage.setGameCycleState('GAME_OVER');
		}
	},
			
	setInterVars : function(intValName, obj, func, args){
		if(obj){
			if(this[intValName] === null){
				if(intValName === '_enableEndGameOptionInt'){
					this[intValName] = func.apply(obj, args);
					this.endGameOptionEnabled = true;
				}else{
					this[intValName] = setInterval(bindWithArgs(obj, func, args), 2000);
					SLINGO.triggerError('SLINGO.GameComm.setInterVars: setting interval '+ intValName + 'property is now: ' + this[intValName],SLINGO.EWARN);
				}
			}else
				SLINGO.triggerError('SLINGO.GameComm.setInterVars: intValName '+ intValName + 'has already been set',SLINGO.EWARN);
		}else
			SLINGO.triggerError('SLINGO.GameComm.setInterVars: object '+ obj + 'is not defined',SLINGO.EWARN);
	},
			
	clearInterVars : function(intValName){
		switch(intValName){
			case '_enableEndGameOptionInt':
				break;
			case '_adsDoneInt':
				clearInterval(this[intValName]); break;
			case '_setEndGameOptionInt':
				clearInterval(this[intValName]); 
			default: //
				
		}
		this[intValName] = null;		
	},			
	
	relay_sgsAck : function(ack){
		SLINGO.triggerError('SLINGO.GameComm.relay_sgsAck: '+ ack,SLINGO.EWARN);
		switch(ack){
			case 'setAdsDone':
				this.clearInterVars('_adsDoneInt');
				break;
			case 'setEndGameOption':
				this.clearInterVars('_setEndGameOptionInt');
				break;				
			case 'enableEndGameOption':
				this.clearInterVars('_enableEndGameOptionInt');
				break;
			default:
				//nothing..			
		}		
		return ack;
	},
		
	_sendDbgMsg : function(str){
		var d = new Date();
		var ds = d.getHours() + ':' + d.getMinutes() + ':' + d.getSeconds() + ':' + d.getMilliseconds();
		var s = '> PAGE: [' + ds + '] ' + str;
		if('console' in window || (!window.all && typeof console != 'undefined'))
			console.log(s);
		/*else{
			if(SLINGO.$('tmpConsole') != null){
				if(SLINGO.$('tmpConsole').style.backgroundColor.toUpperCase() != '#FFFFFF'){
					t = SLINGO.$('tmpConsole');
					t.style.position = 'fixed';
					t.style.top='0';
					t.style.left='0';
					t.style.zIndex='100';
					t.style.height = '100px';
					t.style.width = '450px';
					t.style.backgroundColor = '#FFFFFF';
					t.style.overflow = 'auto';
				}
				t = SLINGO.$('tmpConsole');
				t.appendChild(document.createTextNode(s));
				t.appendChild(document.createElement('BR'));
				while(t.childNodes.length > 8){
					t.removeChild(t.firstChild);
				}
			}
			//itms[0].appendChild(d);
			return false;
		}*/
	},
			
	initCanvasInterface : function(canvas) {
		canvas.GetVariable = function(key)		{ return SLINGO.PlatformAdapter.getData(key); }
		canvas.SetVariable = function(key,val) 	{ SLINGO.PlatformAdapter.setData(key,val)}
		canvas.PercentLoaded = function()		{ return SLINGO.PlatformAdapter.PercentLoaded(); }
		canvas.pa_RegisterJavascriptMethod = function() { }
	},
	getFlashMovie : function(){
		var gameElementNode = SLINGO.$('gameElement');
		if(gameElementNode && gameElementNode.tagName.toUpperCase() == 'CANVAS') {
			return gameElementNode;
		} else {			
			return (document.all)?window['gameElement']:document['gameElement'];
		}
	},
	
	/* Determine the percent loaded of the game swf. */
	getFlashPercentLoaded : function(){
		var swf = this.getFlashMovie();
		if(!swf || !('PercentLoaded' in swf))
			return false;
			
		return swf.PercentLoaded();
	}
};

/* Extends Composite display to add in constraints and required elements
	It also introduces the show and hide handler required by any Stage Display instance
*/
SLINGO.StageDisplay = SLINGO.CompositeDisplay.extend({
	_obj_class_name : 'StageDisplay',
	_elm_keys : [],
	_allElementsExist : function(){
		for(var i = 0; i < this._elm_keys; ++i){
			if(!this.isElement(this._elm_keys[i])) return false;
		}
		return true;
	},
	_setState_SHOW : function(){
		c = this.getContainer();
		if(!c) throw 'Class:'+this.getObjClassName()+' method:StageDisplay.setState_SHOW : no stage container element found';
		c.style.display = 'block';
		return true;
	},
	_setState_HIDE : function(){
		c = this.getContainer();
		if(!c) throw 'Class:'+this.getObjClassName()+' method:StageDisplay.setState_SHOW : no stage container element found';
		c.style.display = 'none';
		return true;
	}
});

/* manages multiple stage display elements
	!!! it will only show one stage at a time !!!
*/
SLINGO.StageMgr = SLINGO.CompositeDisplay.extend({
	_obj_class_name : 'StageMgr',
	/* wrapper for addChild for convenience of class users */
	addStage : function(refId,stage,override /* default false */){
		if(this.addChild(refId,stage,override)){
			s = this.getChild(refId);
			s.addWatch('ANY',this.catchStageStateUpdate);
			s.STAGEMGR_REF = this;
			s.STAGEMGR_REFID = refId;
		}else return false;

		/* set up references for getting around */


	},
	/* extended to make sure any element being added was a StageDisplay object */
	addChild : function(refId,element,override /* default false */){
		if(!(element instanceof SLINGO.StageDisplay)) throw 'Class:'+this.getObjClassName()+' method:StageMgr.addChild : invalid StageDisplay element passed as argument';
		ret = this._super(refId,element,override);
		/* set up a 'fake' state handler for stages */
		if(ret === true){
			this[this.set_hdlr_pfx+refId.toUpperCase()] = function(){return true;}
		}
		return ret;
	},
	/* shows a given stage, hides all the others */
	showStage : function(refId){
		if(!this.isChild(refId)) throw 'Class:'+this.getObjClassName()+' method:StageMgr.showStage : invalid stage reference id passed as argument';
		else{
			stgs = this.getAllChildren();
			if(stgs){
				for(var i in stgs){					
					if(i != refId){
						this.setChildState(i, 'HIDE');
					} else {
						this.setChildState(i, 'SHOW');
						this.setState(refId.toUpperCase());
					}
				}
			}
		}
	},
	/* hides a stage without having to show another one */
	hideStage : function(refId){
		if(!this.isChild(refId)) throw 'Class:'+this.getObjClassName()+' method:StageMgr.hideStage : invalid stage reference id passed as argument: '+refId;
		else{
			this.setChildState(refId, 'HIDE');
		}
	},
	/* catches stage state updates that are a result of calls outside this objects scope */
	catchStageStateUpdate : function(obj, curr, prev, refId){
		if(typeof refId == 'undefined'){
			if('STAGEMGR_REF' in obj){
				/* re-scope */
				return obj.STAGEMGR_REF.catchStageStateUpdate(obj, curr, prev, obj.STAGEMGR_REFID);
			}else {
				throw 'Class:'+this.getObjClassName()+' method:StageMgr.catchStageStateUpdate : invalid stage object passed as argument';
			}
		}else{
			//SLINGO.triggerError('Class:'+this.getObjClassName()+' method:StageMgr.catchStageStateUpdate : obj class:'+obj.getObjClassName()+' curr:'+curr+' prev:'+prev+' refId: '+refId,SLINGO.EWARN);
		}
	}
});
/* extension to create a preroll stage */
SLINGO.PrerollStage_BEACON = SLINGO.StageDisplay.extend({
	_obj_class_name : 'PrerollStage_BEACON',
	
	/* once the stage is initialized, create instances of the elements on the stage */
	_onInit : function(){
		var inter = new SLINGO.IntermissionElement_BEACON(
			'prerollAdElm',
			{
				'pbar_elm_id' 	: 'prerollAdPBar',
				'video_elm_id'	: 'prerollIframeAd',
				'video_src'	: SLINGO.Game.prollSrc,
                                'loader_elm_id' : 'prerollAdLoader'
			}
		);
		this.addChild('INTER', inter);
	},
	
	/* start the preroll by starting the progress bar and the video ad */
	_setState_SHOW : function(){
		var state = this.getState();
		if(state !== 'INIT' && state !== 'END')
			return SLINGO.triggerError('PrerollStage_BEACON._setState_SHOW: invalid state: '+state,SLINGO.EWARN);

		this._super();

		this.getChild('INTER').addWatch('END', bind(this, this.catchInterEnd));
		this.setChildState('INTER', 'SHOW');

		return true;
	},

	/* the preroll is over, do some cleanup */
	_setState_HIDE : function(){
		this._super();
		this.setChildState('INTER', 'HIDE');
		var elm = this.getElement();
		elm.parentNode.removeChild(elm);
		return true;
	},
	
	_setState_END : function(){
		var state = this.getState();
		return (state !== 'SHOW')
			? SLINGO.triggerError('PrerollStage_BEACON._setState_END: invalid state: '+state,SLINGO.EWARN)
			: true;
	},
	
	catchInterEnd : function(){
		this.setState('END');
	}
});
/* extension to create a preroll stage */
SLINGO.PrerollStage = SLINGO.StageDisplay.extend({
	_obj_class_name : 'PrerollStage',
	_elm_keys : ['PBARCONTAINER','VIDEOAD'],
	_pbar_obj : null,
	_pbar_txt : null,
	_pbar_duration : 90, // 40
	_pbar_ready : false,

	/* once the stage is initialized, create instances of the elements on the stage */
	_onInit : function(){
		this.addChild('PBARCONTAINER', new SLINGO.DisplayElement('prerollAdPBar'));
		this.addChild('VIDEOAD', new SLINGO.VideoAdElement('prerollIframeAd'));
	},

	addChild : function(refId,element,override /* default false */){
		this._super(refId,element,override);
		var r = refId.toUpperCase();
		switch(r){
			case 'PBARCONTAINER' :
				var e = element.getElement();
				this.initPBar(e);
				break;
			default: break;
		}
	},

	/* Initialze the progress bar based on the game cycle type. */
	initPBar : function(e){
		
		this._pbar_obj = new SLINGO.PBar.ProgressBar(e);
		this._pbar_obj.STAGEOBJ_REF 		= this;
		this._pbar_obj.LOAD_TEXT 			= 'Intermission:';
		this._pbar_obj.progressBarClass		= 'pBarFill';
		this._pbar_obj.TOTAL_SECONDS		= SLINGO.Game.isDevMode === true ? 10 : this._pbar_duration;
		
		var cycleType = SLINGO.GamePage.getGameCycleType();
		if(!cycleType) return SLINGO.triggerError('PrerollStage.initPBar: no game cycle type found',SLINGO.EWARN);

		switch(cycleType){
			case 'SGS':
				if(SLINGO.GamePage.getGameComm().gameReady) {
					this._pbar_obj.TIME_BASED_UPDATES	= false;
					SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
				} else 
					SLINGO.GamePage.intermissionStarted = true;
				break;

			case 'SOCKETLESS':				
				if(SLINGO.GamePage.getGameComm().gameReady) {
					SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
				} else 
					SLINGO.GamePage.intermissionStarted = true;
			default:
				this._pbar_obj.TIME_BASED_UPDATES	= true;
				this._pbar_obj.endCallFunction		= this.catchPBarEnd;
				break;
		}

		this._pbar_txt = SLINGO.$('progTxtTarg');
		this._pbar_obj.init();
		this._pbar_ready = true;
	},

	/* Set the percentage filled for the progress bar. */
	updatePBar : function(prog){
		var cycleType = SLINGO.GamePage.getGameCycleType();
		if(cycleType != 'SGS') return;
		var currPct = parseInt(prog);
		if(currPct) this._pbar_obj.setFillPct(currPct);
		else this._pbar_txt.innerHTML = prog;
	},

	/* the preroll is over, do some cleanup */
	_setState_HIDE : function(){
		this._super();
		var elm = this.getElement();
		elm.parentNode.removeChild(elm);
		this.setChildState('VIDEOAD', 'END');
		return true;
	},

	/* start the preroll by starting the progress bar and the video ad */
	_setState_START : function(){
		var state = this.getState();
		if(state !== 'SHOW')
			return SLINGO.triggerError('PrerollStage._setState_START: invalid state: '+state,SLINGO.EWARN);

		this._pbar_obj.launch();

		this.getChild('VIDEOAD').setSrc(SLINGO.Game.prollSrc);
		this.setChildState('VIDEOAD', 'START');

		return true;
	},

	/* This is set when the preroll is over. */
	_setState_END : function(){
		if('STAGEMGR_REF' in this){
			//SLINGO.triggerError('PrerollStage._setState_END - YAY!!!! stage manager reference found',SLINGO.EWARN);
		}else{
			throw 'PrerollStage._setState_END - no stage manager reference found';
		}
		return true;
	},

	/* This function is called when the progress bar has reached 100%. */
	catchPBarEnd : function(){
		if('STAGEOBJ_REF' in this) return this.STAGEOBJ_REF.catchPBarEnd();
		else this.setState('END');
	},

	/* This function is called everything this object's state is changed. */
	_onUpdState : function(cs){
		if(cs.toUpperCase() == 'SHOW'){
			this.setState('START');
		}
	}
});

/* Stage to show when the preroll is skipped/finished, but the game is not done loading. */
SLINGO.PreloadStage = SLINGO.StageDisplay.extend({
	_obj_class_name : 'PreloadStage',
	_elm_keys : ['PBARCONTAINER'],
	_num_progress_checks : 0,
	_max_progress_checks : 900,	/* Changed from 360 to 900 (15 min) on 2012.09.08 -NV */
	_pbar : {
		obj 		: null,
		text_node 	: null,
		ready 		: false,
		load_text	: 'Loading Game:',
		timeout		: false
	},
	
	_onInit : function(){
		SLINGO.triggerError('PreloadStage._onInit',SLINGO.EWARN);
		this.addChild('PBARCONTAINER', new SLINGO.DisplayElement('preloadPBar'));
	},

	/* Initialze and start the progress bar based on the game cycle type. */
	startPBar : function(){
		SLINGO.triggerError('PreloadStage.startPBar: '+this._pbar.ready,SLINGO.EWARN);
		if(!this._pbar.ready){
			SLINGO.triggerError('PreloadStage.startPBar: starting pbar',SLINGO.EWARN);
			this._pbar.obj 						= new SLINGO.PBar.ProgressBar(this.getChild('PBARCONTAINER').getElement());
			this._pbar.obj.STAGEOBJ_REF 		= this;
			this._pbar.obj.LOAD_TEXT 			= this._pbar.load_text;
			this._pbar.obj.progressBarClass		= 'pBarFill';
			this._pbar.obj.TIME_BASED_UPDATES	= false;
			this._pbar.obj.endCallFunction		= bind(this, this.catchPBarEnd);
			this._pbar.obj.text_node 			= SLINGO.$('progTxtTarg');
			this._pbar.obj.targetTextNodeId		= 'preloadbar_innerText'
				
			this._pbar.obj.init();
			this._pbar.ready = true;
			this._pbar.obj.launch();
		}
		
		return true;
	},

	/* Set the percentage filled for the progress bar. */
	updatePBar : function(prog){
		var currPct = parseInt(prog);
		if(currPct) this._pbar.obj.setFillPct(currPct);
		else this._pbar.obj.text_node.innerHTML = prog;
	},
	
	/* This function is called when the progress bar reaches 100% */
	catchPBarEnd : function(){	
		this.setState('END');
	},
	
	/* This function runs on a 1 second timeout and checks the progress of the game download */
	watchProgress : function(){		
		var progress = this.getProgress();
		var text = 'Initializing Game... Please Wait...';
		SLINGO.triggerError('PreloadStage.watchProgress: '+progress+'%, num checks: '+this._num_progress_checks,SLINGO.EWARN);
		
		if(this._num_progress_checks > this._max_progress_checks)
			return this.setState('TIMEOUT');
		
		if(this._pbar.timeout) this._pbar.timeout = false;
			
		if(progress < 100){
			this.updatePBar(progress);
			++this._num_progress_checks;
			this._pbar.timeout = setTimeout(bind(this, this.watchProgress), 1000);
		} else {
			this.updatePBar(100);
			this.updatePBar(text);
			if(SLINGO.$('preloadText')) SLINGO.$('preloadText').innerHTML = text;
		}
		
		return true;
	},
	
	/* For SDC Relaunch: The main swf will account for half of the progress bar, and any additional game assets will count
		for the other half. If the game does not provide an "asset_dl_progress" key, the progress bar will hang
		at 50% until the game sets "game_ready" to 1. 
	  For all other games: The main swf will account for all of the progress bar. The progress bar will hang at 100% until
	  	the game sets "game_ready" to 1. */
	getProgress : function(){
		/*if(SLINGO.Game.roomId != 1062)*/
			return SLINGO.GameComm.getFlashPercentLoaded();
			
		/*var progress = 0;
		var mainSwfProg = SLINGO.GameComm.getFlashPercentLoaded();
		if((parseInt(mainSwfProg) == mainSwfProg) && (mainSwfProg > 0)){
			progress += Math.floor(mainSwfProg / 2);
			if(mainSwfProg == 100){
				var assetsProg = SLINGO.GameComm.getGameVar('asset_dl_progress');
				if(parseInt(assetsProg)) progress += Math.floor(assetsProg / 2);
			}
			if(SLINGO.GameComm.isGameReady()) progress = 100;
		}
		return progress;*/
	},
	
	_setState_START : function(){
		SLINGO.triggerError('PreloadStage._setState_START',SLINGO.EWARN);
		
		var st = this.getState();
		if(st !== 'SHOW')
			return SLINGO.triggerError('PreloadStage._setState_START: invalid state: '+st,SLINGO.EWARN);
	
		this.startPBar();
		this.watchProgress();
		return true;
	},
	
	_setState_END : function(){
		SLINGO.triggerError('PreloadStage._setState_END',SLINGO.EWARN);
		
		var st = this.getState();
		if(st !== 'START')
			return SLINGO.triggerError('PreloadStage._setState_END: invalid state: '+st,SLINGO.EWARN);
			
		return true;
	},
	
	_setState_TIMEOUT : function(){
		SLINGO.triggerError('PreloadStage._setState_TIMEOUT',SLINGO.EWARN);
		return true;
	}
	
});

/* creates the game stage */
SLINGO.GameStage = SLINGO.StageDisplay.extend({
	_obj_class_name : 'GameStage',
	_elm_keys : ['HEADING','GAME','INTER','LOBBY','COMPANION'],
    _elm_post_vars : ['score', 'token', 'room_id', 'game_id', 'post_time'],

	/* StageMgr for the game area (possible stages include actual game container, title screen, other game screens, lobby, interstitial) */
	_game_area_mgr : null,

	/* the current game area stage being displayed */
	currGameAreaStage : 'GAME',

	/* post variables to pass along in score submit AJAX request */
	postScoreArgs : {},

	/* URL to submit a game score */
	postScoreURL : '/ajax/sgs/gp_post_score/',

	_onInit : function(){
		/* Set up stage manager for game area (which covers actual game, intermission, lobby, and any extra game screens (title screen, tutorial, etc) */
		this._game_area_mgr = new SLINGO.StageMgr('stageRightCol');
		this._game_area_mgr.addStage('GAME'	, new SLINGO.GameElement('gameElm'));
		var inter = SLINGO.Game.adBeaconWatchEnabled
		? new SLINGO.IntermissionElement_BEACON(
			'videoAdElm', 
			{
				'pbar_elm_id' : 'stageAdPBar',
				'video_elm_id' : 'videoAd',
				'video_src'	: SLINGO.Game.interSrc,
                                'loader_elm_id' : 'videoAdLoader'
			}
		 )
		: new SLINGO.IntermissionElement('videoAdElm');
		this._game_area_mgr.addStage('INTER', inter);
		this._game_area_mgr.addStage('LOBBY', new SLINGO.LobbyElement('lobbyElm'));

		/* Child elements that are independent of whether or not the game is being displayed. */
		this.addChild('HEADING'		, new SLINGO.StageElement('gameTitle'));
		if(SLINGO.$('sdcOverGame')){
			this.addChild('TOP_COMPANION' , new SLINGO.StageElement('sdcOverGame'));
			this.topCompanion = this.getChild('TOP_COMPANION');
		}else this.topCompanion = false;
		
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			var adElement = new SLINGO.GPTAdElement(SLINGO.GamePage.Ads[i].containerId);
			this.addChild('AD_'+i, adElement);
			//VJ adElement.setIframe(SLINGO.$(SLINGO.GamePage.Ads[i].iframeId),SLINGO.GamePage.Ads[i].adSource);
			adElement.setGPTData(SLINGO.GamePage.Ads[i].gptData.url, SLINGO.GamePage.Ads[i].gptData.sz, SLINGO.GamePage.Ads[i].iframeId);
			adElement.addWatch('UNCOVER', bind(this, this.uncoverBannerAd,SLINGO.GamePage.Ads[i]));
			SLINGO.GamePage.Ads[i].elementRef = adElement;
			SLINGO.GamePage.Ads[i].bannerInterval = null;						
		}
		//this.addChild('SKYSCRAPER'	, new SLINGO.IframeAdElement('bannerElm'));

		if(SLINGO.$('houseBannerElm'))	this.addChild('HOUSEBANNER', new SLINGO.AdElement('houseBannerElm'));
		if(SLINGO.$('ugBannerElm'))     this.addChild('UGBANNER', new SLINGO.GPTAdElement('ugBannerElm'));

		this.stage 				= this.getElement();
		this.leftCol			= SLINGO.$('stageCont');
		this.gameTitle			= this.getChild('HEADING');
		this.houseBanr			= this.getChild('HOUSEBANNER');
		this.ugBanr                             = this.getChild('UGBANNER');
//		this.lobbyAd 			= SLINGO.$('lobbyAd');				//LobbyAd extends AdElement
//		this.lobbyAd.src 		= '';								//LobbyAd property
//		this.ibarNode 			= SLINGO.$('stageAdPBar');			//LobbyElement/IntermissionElement property

		/* Add a watcher to the skyscraper ad so we know when an overlay/flyout is no longer covering them. */
		//this.banr.setIframe(SLINGO.$('skyscraper'), SLINGO.Game.bannerSrc);
		//this.banr.addWatch('UNCOVER', bind(this, this.uncoverBannerAd));

                /* Add a watcher to the under game ad so we know when an overlay/flyout is no longer covering them. */
                if(this.ugBanr){
                        //VJ this.ugBanr.setIframe(SLINGO.$('underGameLB'), SLINGO.Game.ugBannerSrc);
			this.ugBanr.setGPTData(SLINGO.Game.ugBannerGPTData.url, SLINGO.Game.ugBannerGPTData.sz, 'adContentArea_LB');
                        this.ugBanr.addWatch('UNCOVER', bind(this, this.uncoverUGBannerAd));
                }

		// extra GPT init
		googletag.cmd.push(function() {
			console.log("*VJ* GPT enable");
			googletag.pubads().enableSingleRequest();
			googletag.enableServices();
		});


		/* Set up chat or leaderboard. */
		this._setGameCompanion();
	},

	/* Determine what is going to appear to the left of the game, and initialize it. */
	_setGameCompanion : function(){
		switch(SLINGO.Game.gameCompanionType){
			case 'FLASHCHAT'	: var gc = 'FlashChatElement';			break;
			case 'WEBCHAT'		: var gc = 'WebChatElement';			break;
			case 'LEADERBOARD'	: var gc = (SLINGO.Game.isDailyChallenge) ? 'GenDailyChallengeScoreLeaderboardElement' : 'ScoreLeaderboardElement'; break;
			case 'NONE'         : var gc = false; break; // no companion
			case 'SAMPLE'		:
			default				: var gc = 'GameCompanionElement';		break;
		}
		if (gc){
			this.addChild('COMPANION', new SLINGO[gc]('chatContainer'));
			this.gameCompanion = this.getChild('COMPANION');			
			if("setPrefix" in this.gameCompanion)
				this.gameCompanion.setPrefix(SLINGO.GamePage.getDefaultChallengePrefix());
		}else{
			this.gameCompanion = false;
		}
	},

	/* Set the width of the blue bucket containing the game area. */
	_setStageWidth : function(px){
		if(this.stage && this.stage.style)
			this.stage.style.width = px + 'px';
		return true;
	},

	/* Get the width of the blue bucket containing the game area. */
	_getStageWidth : function(){
		if(!SLINGO.isPluginLoaded('DHTML')) return SLINGO.triggerError('GameStage._getStageWidth:DHTML plugin not loaded', SLINGO.EWARN);
		var p=0;
		try {
			var p = SLINGO.DHTML.getObjectWidth(this.stage);
		} catch(e){}
		return (p)?p:0;
	},

	/* Get the width of the Flash game. */
	_getGameWidth : function(){
		return SLINGO.Game.gameWidth;
	},

	/* Get the height of the Flash game. */
	_getGameHeight : function(){
		return SLINGO.Game.gameHeight;
	},

	_setCurrGameAreaStage : function(refId){
		if(!this._game_area_mgr.isChild(refId))
			return SLINGO.triggerError('GameStage._setDefaultGameAreaStage: invalid stage id: '+refId);

		this.currGameAreaStage = refId;
	},

	_setState_SHOW : function(){
		this.gameTitle.setState('SHOW');
		if (this.gameCompanion){
			this.gameCompanion.setState('SHOW');
		}
		if(!SLINGO.hasClassName(this.stage, 'stageOn')) SLINGO.addClassName(this.stage, 'stageOn', false);
		return true;
	},

	/* This state handler MUST be here so things don't break when the preroll stage is shown. */
	_setState_HIDE : function(){
		return true;
	},
	_setStage_PREROLL_END : function () {
		return true;
	},
	_setState_GAME_START : function(){
		
		/* Shrink page down if it doesn't have a wide banner ad. */
		var hasWideBanner = false;
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			for(var j = 0; j < SLINGO.GamePage.Ads[i].sizes.length; j++) {
				hasWideBanner = hasWideBanner || SLINGO.GamePage.Ads[i].sizes[j].width >= 300;
			}
		}
		
		if(!hasWideBanner && SLINGO.hasClassName(SLINGO.$('theBody'), 'bodyTypeWide'))
			SLINGO.removeClassName(SLINGO.$('theBody'), 'bodyTypeWide');
				
		var gameCompanionWidth = 0;
		if (this.gameCompanion){		
			gameCompanionWidth = parseInt(this.gameCompanion.getElement().style.width);
					
			if(this.gameCompanion.getState() === 'HIDE'){	/* Added 2012.11.16 for larger video ads -NV */
				this.gameCompanion.setState('SHOW');
			}
		}
		
		var gameElements = SLINGO.getElementsByClass('gameStageElement');
		if(gameElements) {
			for(var i = 0; i < gameElements.length; i++) {
				SLINGO.removeClassName(gameElements[i],'hide');
				SLINGO.addClassName(gameElements[i],'show');
			}
		}
		var stageWidth = this._getGameWidth();
		if(!SLINGO.Game.isWide) stageWidth += gameCompanionWidth;
		this._setStageWidth(stageWidth);
		
		this.leftCol.style.width = this._getStageWidth() + 'px';
		this._game_area_mgr.getElement().style.width = this._getGameWidth() + 'px';

		this._game_area_mgr.showStage(this.currGameAreaStage);
		
		if(SLINGO.Game.isSample){
			if (this.gameCompanion && this.gameCompanion.getElement().className === ''){	
				var gameCompanionH= SLINGO.DHTML.getObjectHeight(this.gameCompanion.getElement().id);
				//var stageId = SLINGO.GamePage.getGameCycleMgr().getStageMgr().getElement().id;
				var stageH = SLINGO.DHTML.getObjectHeight(this.stage.id);
				if(gameCompanionH < stageH){
					this.gameCompanion.getElement().style.height = stageH + 'px';
				}
			}		
		}
		
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			SLINGO.GamePage.Ads[i].elementRef.setState('SHOW');
		}

		if(this.houseBanr) this.houseBanr.setState('SHOW');
		
		this._setAllBannerAdIntervals();

		if(!SLINGO.Game.isWide || (SLINGO.Game.isWide && (this._getGameWidth() > 734)) || (SLINGO.Game.isWide && !this.gameCompanion)){
			if(this.ugBanr) this.ugBanr.setState('SHOW');
			//if(this.stage.offsetWidth < 734)			/* Not really sure if this is necessary or not.. */
			//	this.leftCol.style.width = '734px';
			/* TODO - fire Google Analytic event */
		}
		
		this._onGameStart();
		
		return true;
	},

	_setState_GAME_OVER : function(){
		var score = SLINGO.GameComm.getGameVar('game_score');		
		var playAgain = "0";
		if(SLINGO.Game.isWrapperless)
			playAgain = SLINGO.GameComm.getGameVar('play_again_enable');
		if(this._hasEndGameOverlay()  && playAgain !== "1"){			
			this._showEndGameOverlay(score);
		}
		
		if(!SLINGO.Game.isDailyChallenge){
			SLINGO.GameComm.setGameVar('game_awards', this.getCoinAward(score)+','+this.getRankAward(score));
		}
		
		this._handleScore(score);		
		if('gamesPlayedLife' in SLINGO.Game)
			SLINGO.Game.gamesPlayedLife++;

		// handle the user stats
		this.handleUserStats();
		
		//if leaderboard is enabled, clear properties
		var isLBEnabled = (SLINGO.Game.isSample) ? (SLINGO.Game.isFBCanvas) ? '1' : '0' : '1';
		if(isLBEnabled){
			SLINGO.InGameLeaderboardElement._lbList = false;
			SLINGO.InGameLeaderboardElement._currUser = false;
			SLINGO.InGameLeaderboardElement._friendNumPage = false;
		}
		
		return true;
	},

	
	_setState_GAME_PLAY_AGAIN : function(){
		return true;
	},

	_setState_GAME_END : function(){
		return true;
	},

	_setState_INTERMISSION_START : function(){
		this._clearAllBannerIntervals();
       if(SLINGO.$('sdcGH_endGameMsg')) SLINGO.$('sdcGH_endGameMsg').style.display = 'none';
		this._game_area_mgr.hideStage(this.currGameAreaStage);

		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			SLINGO.GamePage.Ads[i].elementRef.setState('HIDE')
		}
		
		if(this.houseBanr)	this.houseBanr.setState('HIDE');
		if(this.ugBanr)         this.ugBanr.setState('HIDE');

		/* Added 2012.11.21 for larger video ads on games w/o rectangle ads -NV */
		var isWideVideo = SLINGO.hasClassName(this._game_area_mgr.getChild('INTER').getElement(), 'wideVideoFormat');
		if(isWideVideo && !SLINGO.hasClassName(SLINGO.$('theBody'), 'bodyTypeWide')){
			if(!SLINGO.hasClassName(SLINGO.$('theBody'), 'bodyTypeWide'))
				SLINGO.addClassName(SLINGO.$('theBody'), 'bodyTypeWide');
		}
		
		var gameContainerWidth = SLINGO.DHTML.getObjectWidth('gameContainer')-25;
		var gameAreaWidth = gameContainerWidth;
		if(this.topCompanion){
			this.topCompanion.getElement().style.display = 'none';
		}
		if(this.gameCompanion){
			var gameCompanionWidth = parseInt(this.gameCompanion.getElement().style.width);
			if(isWideVideo){
				this.gameCompanion.setState('HIDE');
			} else if(!SLINGO.Game.isWide)
				gameAreaWidth -= gameCompanionWidth;
		}
		
		var gameElements = SLINGO.getElementsByClass('gameStageElement');
		if(gameElements) {
			for(var i = 0; i < gameElements.length; i++) {
				SLINGO.removeClassName(gameElements[i],'show');
				SLINGO.addClassName(gameElements[i],'hide');
			}
		}
		
		this.leftCol.style.width = gameContainerWidth+'px';
		this._setStageWidth(gameContainerWidth);
		this._game_area_mgr.getElement().style.width = gameAreaWidth+'px';
		
		this._game_area_mgr.getChild('INTER').addWatch('END', SLINGO.bind(this,this.setState,[ 'INTERMISSION_END' ]));
		
		this._game_area_mgr.showStage('INTER');
		return true;
	},

	_setState_INTERMISSION_END : function(){
		return true;
	},

	_hasEndGameOverlay : function(){
		if (SLINGO.GamePage.gameConfig.hasNoEndGameScreen){
			return false;
		}else if(SLINGO.Game.isSample){
			return true;
		}else{
			return (!SLINGO.Game.isMulti);
		}
	},
			
	_showEndGameOverlay : function(score){
		if(SLINGO.$('overlayScore')) SLINGO.$('overlayScore').innerHTML = SLINGO.fmtNumber(Math.round(score));
		if(SLINGO.$('overlayCoins'))	SLINGO.$('overlayCoins').innerHTML = SLINGO.fmtNumber(this.getCoinAward(score));
        if(SLINGO.$('overlayRank'))		SLINGO.$('overlayRank').innerHTML = SLINGO.fmtNumber(this.getRankAward(score));
		
		
		/*if(SLINGO.$('overlayCoins')){
			var coins = SLINGO.Game.coinConv ? (score * SLINGO.Game.coinConv) : score;
			SLINGO.$('overlayCoins').innerHTML = SLINGO.fmtNumber(coins);
		}
		*/
		/* Added 2011.11.21 -NV */
		/*if(SLINGO.$('overlayRank')){
			SLINGO.$('overlayRank').innerHTML = SLINGO.fmtNumber(10 * SLINGO.Game.rankMultiplier);
		}*/
        
        if (SLINGO.GamePage.gameConfig.isDirect){
        	SLINGO.GameComm.getFlashMovie().style.height = '2px';
    		SLINGO.GameComm.getFlashMovie().style.width = '2px';
        }
        
		this.displayOverlay('sgs_endGameMsg', 'sgs_PlayAgain', this, this._hideEndGameOverlay, [false]);
		
	},
			
	_hideEndGameOverlay : function(hideOnly){		
    	//var screenID = 'sgs_endGameMsg';
		var overlayElm = this._getOverlayObj();
		if(overlayElm){
			var screenH = SLINGO.DHTML.getObjectHeight(overlayElm.id);
			overlayElm.style.height = screenH + 'px';
		if(!hideOnly){
			var cycleType = SLINGO.GamePage.getGameCycleType();
			if(cycleType === 'SGS'){
                var obj = SLINGO.GamePage.getGameComm().getFlashMovie();
				var func = obj['sgs_CallActionscriptMethod'];
				var args = ['setEndGameOption','1'];
				SLINGO.GamePage.getGameComm().setInterVars('_setEndGameOptionInt', obj, func, args);
				this.displayHideDiv(SLINGO.$('playBtnCont'), 'hide');
				this.displayHideDiv(SLINGO.$('playBtnCont_loading'), 'display');
				//MAKE SURE HEIGHT WAS NOT AFFECTED..
				overlayElm.style.height = screenH + 'px';
                return;
			}else{
                SLINGO.GamePage.setGameCycleState('GAME_END');
                this.displayHideDiv(overlayElm, 'hide'); return true; 
            } 
				
		}else
           this.displayHideDiv(overlayElm, 'hide'); return true;	 
        	
		}else{
			//no overlay
			return SLINGO.triggerError('GameStage._hideEndGameOverlay: no overlay to hide',SLINGO.EWARN);
		}
	},
	
	_getOverlayObj : function(){
		var overlayDiv = SLINGO.getElementsByClass('endGameMsg');
		if(overlayDiv.length > 0){
			return  overlayDiv[0];
		}else return false;
	},

	_handleScore : function(score){
		if(!score)
			return SLINGO.triggerError('GameStage._handleScore: no score found',SLINGO.EWARN);

		if(!parseInt(score) || (parseInt(score) <= 0))
			return SLINGO.triggerError('GameStage._handleScore: invalid score: '+score,SLINGO.EWARN);

		SLINGO.Player.setLastScore(score);
		SLINGO.Player.incrGamesPlayed();

		if('_postScore' in this)
			this._postScore(score);
		else SLINGO.triggerError('GameStage._handleScore: no postScore function defined',SLINGO.EWARN);
	},

	/* 	This is for SGS, Socketless, Wrapperless.
		Override this.postScoreURL and add extra this.postScoreArgs for SDC.
		Child extension should call this._super() at the very *END*
	*/
	_postScore : function(score){	
		if(!score)
			return SLINGO.triggerError('GameStage._postScore: no score found',SLINGO.EWARN);

		if(!parseInt(score) || (parseInt(score) <= 0))
			return SLINGO.triggerError('GameStage._postScore: invalid score: '+score,SLINGO.EWARN);

		var d = new Date();
        this.postScoreArgs['score'] = score;
		if(!SLINGO.Game.isDailyChallenge) this.postScoreArgs['token'] = SLINGO.Player.getPlayerToken();
		this.postScoreArgs['game_id'] = SLINGO.Game.gameId ? SLINGO.Game.gameId : false;
		this.postScoreArgs['room_id'] = SLINGO.Game.roomId ? SLINGO.Game.roomId : false;
        this.postScoreArgs['post_time'] = d.getTime();
        
		//if it's sample game, save the info in a cookie in case user wants to join		
		if(!this.postScoreURL)
			return SLINGO.triggerError('GameStage._postScore: no post url found',SLINGO.EWARN);
				
		if('xfind' in this.postScoreArgs)
			delete this.postScoreArgs['xfind'];

		if(SLINGO.Game.isSample){
			//fb sample games	
			if(typeof SLINGO.Facebook === 'object' && SLINGO.Game.isFBCanvas){				
				if(!SLINGO.Facebook.me)
					SLINGO.Facebook.promptFBLogin('fbConnectSDC');
				if(SLINGO.Facebook.me){
					this.postScoreArgs['sample_uid'] = SLINGO.Facebook.me.fid;
					this.postScoreArgs['uid_source'] = 'FB_' + SLINGO.Facebook._getSlingoAppId();
					this.postSampleFBScore(this.postScoreArgs);
					return;
				}
			}else{
				//all sample games..
				var pLink = this.getPostString();
				var scoreArgsObj = this.postScoreArgs;
				scoreArgsObj['url'] = pLink;
				var joinPost = SLINGO.JSON.stringify(scoreArgsObj);
				SLINGO.setCookie('sljpcn', joinPost, .0035);
			}			
		}
		
		if(!pLink) pLink = this.getPostString();

		if(!SLINGO.XHR.createAndSendRequest('postScore'+d.getTime(), bind(this, this._catchPostScore), pLink, 'GET', 'JSON', true))
			return SLINGO.triggerError('GameStage._postScore: unable to create a valid XHR request.', SLINGO.EWARN);
				
		// Reset for the next time.
		this.postScoreArgs = {};		
	},
			
	postSampleFBScore	: function(post){
		this.postScoreArgs = post;		
		var d = new Date();
		var url = (this.postScoreFBCanvasURL) ? this.postScoreFBCanvasURL : null;		
		var pLink = this.getPostString(url);
		
		//this.postScoreArgs['url'] = pLink;
		
		if(!SLINGO.XHR.createAndSendRequest('postSampleFBScore'+d.getTime(), bind(this, this._catchPostSampleFBScore), pLink, 'GET', 'JSON', true))
			return SLINGO.triggerError('GameStage.postSampleFBScore: unable to create a valid XHR request.', SLINGO.EWARN);
		
	},
			
	_catchPostSampleFBScore : function(retv){
		if(retv.msg === 'FB_CANVAS_GAME'){
			//get score and check if it's higher if it is add it to fb
			SLINGO.Facebook.fbUpdateUserScore(this.postScoreArgs['score']);
			return true;
        }else {
			SLINGO.triggerError('GameStage._catchPostSampleScore: message: '+ retv.msg);
		}
		
	},
			
	/*	Expects retv.msg (on error), retv.coins, retv.token
		Child extension should call this._super() at the very *END*
	*/
	_catchPostScore : function(retv){
		if(retv.msg){
			if(retv.logged_out && !SLINGO.Game.isSample){
				alert('You are not logged in. You will now be redirected to the login page.');
				location.href='/gen/login';
				return;
			}

			if(retv.diff_msg){
				alert('Another game session is already in progress. Please be sure that you only have one game window open at a time.');
				return this._sendToLobby();
			}

			/* GP Edit - added token values to report. */
			if(!retv.no_report){
				var emsg = 'GEN:JS-GENERATEDTICKET,TYPE:POSTSCORE,ERROR:'+retv.msg;
				if(retv.ptok) emsg += ',pgtok:'+retv.ptok;
				if(retv.stok) emsg += ',shtok:'+retv.stok;
				if(retv.ssid) emsg += ',ssid:'+retv.ssid;
				if(retv.misc) emsg += ','+retv.misc;

				return SLINGO.GameComm.relay_reportError(emsg);
			}
			
			SLINGO.triggerError('GameStage._catchPostScore: '+retv.msg);
		}

		if(retv.coins){
			if(SLINGO.isPluginLoaded('UserTools')) SLINGO.UserTools.updateCoinView(retv.coins);
			else SLINGO.triggerError('GameStage._catchPostScore: UserTools plugin not loaded', SLINGO.EWARN);
		}

		if(retv.conv && (retv.conv != SLINGO.Game.coinConv)){
			SLINGO.Game.coinConv = retv.conv;
			SLINGO.GameComm.setGameVar('coin_conversion', SLINGO.Game.coinConv);
		}
		
		/* Added 2011.11.21 -NV */
		if(retv.rank_mult && (retv.rank_mult != SLINGO.Game.rankMultiplier))
			SLINGO.Game.rankMultiplier = retv.rank_mult;
	},
	
	
	getCoinAward : function(score){	
		if(!score){
			var score = SLINGO.GameComm.getGameVar('game_score');
		}
		var score = parseInt(score);
		if(SLINGO.Game.coinConv){
			coins = score * SLINGO.Game.coinConv;
			coins = Math.ceil(coins);
		}
		return coins;
	},
	
	getRankAward : function(){
		if(SLINGO.Game.rankMultiplier){
			rank = parseInt(SLINGO.Game.baseRankAward) * SLINGO.Game.rankMultiplier;
			rank = Math.ceil(rank);
		}
		return rank;
	},	
	

	_sendToLobby : function(){
		setTimeout(function(){window.location.href = 'http://' + window.location.hostname + '/games/detail/' + SLINGO.Game.gameId + '/online';},10000);
	},

	_setAllBannerAdIntervals : function() {
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {			
			this._setBannerAdInterval(SLINGO.GamePage.Ads[i]);
		}
	},
	_setBannerAdInterval : function(ad){
		var element = ad.elementRef;
		
		if(ad.bannerInterval === null && ad.refreshMS > 0 && element.isDisplayed() && !element.isCovered()) {
			ad.bannerInterval = setInterval(bindWithArgs(this, this.refreshBannerAd, [ad]), ad.refreshMS);
		}
		if(SLINGO.$(ad.containerId)) SLINGO.$(ad.containerId).style.display = 'block';
		if(SLINGO.$(ad.iframeId)) SLINGO.$(ad.iframeId).src = ad.adSource;
	},

	_clearAllBannerIntervals : function() {
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			this._clearBannerAdInterval(SLINGO.GamePage.Ads[i]);
		}
	},
	_clearBannerAdInterval : function(ad){
		if(ad.bannerInterval !== null){
			clearInterval(ad.bannerInterval);
			ad.bannerInterval = null;
		}
	},

	refreshBannerAd : function(ad){
		if(ad.elementRef && !ad.elementRef.isDisplayed()) {
			ad.elementRef.setState('CLEAR');
			this._clearBannerAdInterval(ad);
		} else {
			ad.elementRef.setState('REFRESH');
		}
	},

	uncoverBannerAd : function(ad){
		if(SLINGO.inArray(this.getState(), ['GAME_START','GAME_END'])){
			ad.elementRef.setState('SHOW');
			return this._setBannerAdInterval(ad);
		}
		return false;
	},

	uncoverUGBannerAd : function(){
		if(SLINGO.inArray(this.getState(), ['GAME_START','GAME_END']))
			return this.ugBanr.setState('SHOW');
		return false;
	},
	
	getPostString : function(postScoreURL){
		var st = xt = sURL = xURL = pURL = '';
		for(var i in this.postScoreArgs){
			var isPVar = this.checkPVar(i);
			if(isPVar){
				st += this.postScoreArgs[i];
				sURL += this.postScoreArgs[i] + '/';
			}else{
				var newVar = this.postScoreArgs[i].replace(/,/g, '.');
				xt += i  + newVar ;
				xURL +=  i + '/' + newVar + '/';
			}              
		}		
		st += xt;	
		//remove the last / 
		xURL = xURL.substring(0, xURL.length - 1);
		var str = CryptoJS.MD5(st) + '/';
		var postURL = (!postScoreURL) ? this.postScoreURL : postScoreURL;	
		return postURL + str + sURL + xURL;	
	},
	
	checkPVar : function(pVar){		
		for(var i=0; i <= this._elm_post_vars.length; i++){
			if(this._elm_post_vars[i] === pVar)
				return true;
		}
		return false;
	},
	
	updatePBar : function(prog){
		var cycleType = SLINGO.GamePage.getGameCycleType();
		if(cycleType != 'SGS') return;
		var stage = this._game_area_mgr.getChild('INTER');
		var currPct = parseInt(prog);
		if(currPct) stage._pbar_obj.setFillPct(currPct);
		else stage._pbar_txt.innerHTML = prog;
	},
			
	displayOverlay : function(screenID, playAgainButton, obj, func, argsStr){
		if(SLINGO.$(screenID)){
			SLINGO.Game.resetGameH = false;
			SLINGO.$(screenID).style.display = 'block';
			var oldScreenH = SLINGO.DHTML.getObjectHeight(screenID);
			var oldScreenW = SLINGO.DHTML.getObjectWidth(screenID);
            var gameElem = this._game_area_mgr.getChild('GAME').getElement();			
			var gameElemH = SLINGO.DHTML.getObjectHeight(gameElem.id);
            var isHorzGameComp = (this.gameCompanion && this.gameCompanion.getElement().className !== '') ? true : false;
			if(!SLINGO.$('sdcGH_endGameMsg')){
                var borderH = (isHorzGameComp && !SLINGO.Game.isSample) ? parseInt(getStyle(SLINGO.$(this.stage.id), 'border-top-width')) : parseInt(getStyle(SLINGO.$(this.stage.id), 'border-top-width')) + parseInt(getStyle(SLINGO.$(this.stage.id), 'border-bottom-width'));
				var stageH = (borderH) ? SLINGO.DHTML.getObjectHeight(this.stage.id) - (borderH) : SLINGO.DHTML.getObjectHeight(this.stage.id) ;
                var gameAreaH = (isHorzGameComp && !SLINGO.Game.isSample) ? gameElemH : stageH;
                if(oldScreenH){ var screenH = (oldScreenH < gameAreaH) ? gameAreaH : oldScreenH; } else{ var screenH = gameAreaH; }
				if(SLINGO.Game.isSample || SLINGO.Game.isWide){
					//var borderW = parseInt(getComputedStyle(SLINGO.$(this.stage.id),null).getPropertyValue('border-left-width')) + parseInt(getComputedStyle(SLINGO.$(this.stage.id),null).getPropertyValue('border-right-width'));
                    var borderW = parseInt(getStyle(SLINGO.$(this.stage.id), 'border-left-width')) + parseInt(getStyle(SLINGO.$(this.stage.id), 'border-right-width'));
					var screenW = (borderW) ? SLINGO.DHTML.getObjectWidth(this.stage.id) - (borderW) : SLINGO.DHTML.getObjectWidth(this.stage.id);
					
				}else screenW = oldScreenW;
				SLINGO.$(screenID).style.width = screenW + 'px';
				SLINGO.$(screenID).style.height = screenH + 'px';
			}		

			//make sure game elem is not smaller than screen			
			if(gameElemH > 2 && gameElemH < screenH){
				if(isHorzGameComp && SLINGO.Game.isSample){
					//do nothing, it's ok.
				}else{
					if(this.topCompanion && SLINGO.DHTML.getObjectHeight(this.topCompanion.getElement().id)){					
						var topCompanionH =  SLINGO.DHTML.getObjectHeight(this.topCompanion.getElement().id);
						if(gameElem + topCompanionH < screenH){ SLINGO.Game.resetGameH = gameElemH; gameElem.style.height = screenH + 'px'; }					
					}else { 
						SLINGO.Game.resetGameH = gameElemH;
						gameElem.style.height = screenH + 'px';
					}
				}				
			}		
			if(SLINGO.Game.isSample && !isHorzGameComp){	
				var stageElem = this._game_area_mgr.getElement();
				stageElem.style.position = 'static';
				this.stage.style.position = 'relative';
			}
          
			//adjust top margin             
			var nodeArr = SLINGO.getElementsByClass('endGameContent');
			if(nodeArr.length > 0){
				var endContentId = nodeArr[0].id;
				var endContentH = SLINGO.DHTML.getObjectHeight(endContentId);
				if(screenH > endContentH){
					var topMargin =  (screenH - endContentH)/2;
					//  because of IE7 buggy margin top 
					var parentId = SLINGO.$(nodeArr[0].id).parentNode.id;
					if(SLINGO.$(parentId)){
						SLINGO.$(parentId).style.paddingTop= topMargin + "px";
						SLINGO.$(nodeArr[0].id).style.margin = '0 auto';
					}
				}
			}           
		}

		//add onclick event to play button 
		if(playAgainButton){
			// make sure button is showing..
			if(SLINGO.$('playBtnCont'))
				this.displayHideDiv(SLINGO.$('playBtnCont'), 'display');
			if(SLINGO.$('playBtnCont_loading')){
				this.displayHideDiv(SLINGO.$('playBtnCont_loading'), 'hide');
				SLINGO.$('playBtnCont_loading').style.height= '45px';
			}
			if(SLINGO.$(playAgainButton)){
				if(!SLINGO.Event.addEvent(SLINGO.$(playAgainButton), 'onclick', bindWithArgs(obj, func, argsStr)))
					SLINGO.triggerError('GameStage.displayOverlay: Cannot add event to button for ' + playAgainButton);
			}
		}
	
	},
	
	displayHideDiv : function(obj, act){
		if(obj){
			obj.style.display = (act === 'display') ? 'block' : 'none'; 
		}else SLINGO.triggerError('GenDailyChallengeStage.displayHideDiv, null obj sent to func');	
	},
	
	_onGameStart : function(){
		// note that user data stuff is defined in some child classes to handle this stuff,
		// so it might be dangerous to change any of those classes to use _super and call this
		// code without modifications to all the classes in between for some degree of consistency
		this.prepareUserStatsForGame();		
		// only set user data if there at least one of its own property
		for (var prop in SLINGO.Game.userData){
			if (SLINGO.Game.userData.hasOwnProperty(prop)){
				var stats = SLINGO.JSON.stringify(SLINGO.Game.userData);
				SLINGO.GameComm.setGameVar('user_data', stats);
				break;
			}
		}
		// now we can start the game
		if ('startGame' in this){
			this.startGame();
		}
		this.initGameOverChecking();
	},
	
	initGameOverChecking : function(){
		// nothing
	},
	
	handleUserStats : function(stats){
		var stats = SLINGO.GameComm.getGameVar('user_data');
		if(stats && this.processUserStatsUpdates(stats)){
			this.sendUserStatsReq();
		}
	},
	
	/* Take updates from the game or the server into account. */
	processUserStatsUpdates : function(stats){		
		var s = stats;
		try{
			stats = SLINGO.JSON.parse(stats);
		}catch(e){
			// games that don't support the user_data key will pretty much stop here
			return SLINGO.triggerError('GameStage.processUserStatsUpdates: invalid stats: '+ s + ', error: ' + e,SLINGO.EWARN);
		}
		
		SLINGO.Game.userData = {};
		for (var k in stats){
			if (stats.hasOwnProperty(k)){
				SLINGO.Game.userData[k] = stats[k];
			}
		}
		
		return true;
	},
	
	prepareUserStatsForGame : function(){
		// nothing
	},
	
	/* Send an AJAX request to the server to update the user stats. */
	sendUserStatsReq : function(){
		// If there are no user stats defined, this game doesn't support them. Move along.
		var stats = SLINGO.Game.userData;
		if(stats === false){
			return;
		}
			
		// Set up the post variables to be logged.
		var post = {
			user_data				: SLINGO.JSON.stringify(stats),
			game_id					: SLINGO.Game.gameId
		};
		
		// Send the AJAX request.
		var d = new Date;
		if(!SLINGO.XHR.createAndSendRequest('sendUserStats_'+d.getTime(), bind(this, this.catchUserStatsReq), '/ajax/gameservice/user_data/'+d.getTime(), 'POST', 'JSON', true, post))
			return SLINGO.triggerError('GameStage.sendUserStatsReq: could not send XHR request',SLINGO.EWARN);
			
		return true;
	},
	
	/* Receive the AJAX response and process any updates to the user stats (if the user is playing in more than one window). */
	catchUserStatsReq : function(retv){
		if(retv.msg)
			SLINGO.triggerError('GameStage.catchUserStatsReq: '+retv.msg,SLINGO.EWARN);
			
		if(retv.stats) {
			this.processUserStatsUpdates(retv.stats);
		}
	}
});

SLINGO.SGSStage = SLINGO.GameStage.extend({
	_obj_class_name : 'SGSStage',
	_setState_GAME_START : function(){
		if(SLINGO.isPluginLoaded('UserTools') && !SLINGO.Game.isSample)
			setTimeout("SLINGO.UserTools.updateCoins();",2000);

		this._super();
		return true;
	},
	_setState_INTERMISSION_START : function(){
		this._super();

		if(SLINGO.isPluginLoaded('UserTools') && !SLINGO.Game.isSample)
			setTimeout("SLINGO.UserTools.updateCoins();",2000);

		return true;
	},
	_setState_LOBBY_START : function() {		
		/* Added 2012.11.21 to go back to the regular page body width after displaying a larger video ad if the ad next to the game isn't a wide one -NV */
		var hasWideBanner = false;
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			for(var j = 0; j < SLINGO.GamePage.Ads[i].sizes.length; j++) {
				hasWideBanner = hasWideBanner || SLINGO.GamePage.Ads[i].sizes[j].width >= 300;
			}
		}		
		if(!hasWideBanner && SLINGO.hasClassName(SLINGO.$('theBody'), 'bodyTypeWide'))
			SLINGO.removeClassName(SLINGO.$('theBody'), 'bodyTypeWide');
				
		var gameCompanionWidth = 0;
		if (this.gameCompanion){
			gameCompanionWidth = parseInt(this.gameCompanion.getElement().style.width);
			if(this.gameCompanion.getState() === 'HIDE'){	/* Added 2012.11.16 for larger video ads -NV */
				this.gameCompanion.setState('SHOW');
			}
		}
		
		var stageWidth = this._getGameWidth();
		if(!SLINGO.Game.isWide) stageWidth += gameCompanionWidth;
		this._setStageWidth(stageWidth);
		
		this.leftCol.style.width = this._getStageWidth() + 'px';
		this._game_area_mgr.getElement().style.width = this._getGameWidth() + 'px';
		
		this._game_area_mgr.showStage('LOBBY');
		return true;
	},
	_setState_LOBBY_END : function() {
		this._game_area_mgr.showStage('GAME');
		return true;
	},
	_setState_GAME_OVER : function() {
		if('gamesPlayedLife' in SLINGO.Game)
			SLINGO.Game.gamesPlayedLife++;
		
		return true;
	},
	_setState_INACTIVE : function() {
		this._sendToLobby();
	},
	_postScore : function(score){
		/* Over-write _postScore from GameStage, SGS handles the score submittion */
		return true;
	},
	
	initGameOverChecking : function(){
		// do nothing
	}
});

SLINGO.SocketlessStage = SLINGO.GameStage.extend({
	_obj_class_name : 'SocketlessStage',

	/* URL to retrieve info for a new socketless game */
	startGameURL : '/ajax/sgs/gp_start_game/',
	_setState_GAME_START : function(){
		var pname = SLINGO.Player.getPlayerName();
		if(pname) SLINGO.GameComm.setGameVar('player_name', pname);
		
		var puid = SLINGO.Player.getPlayerId();
		if(puid) SLINGO.GameComm.setGameVar('player_uid', puid);

		var conv = SLINGO.Game.coinConv;
		if(conv) SLINGO.GameComm.setGameVar('coin_conversion', conv);

		return this._super();
	},

	_setState_GAME_END : function(){
		this._super();
		SLINGO.GameComm.setGameVar('reset_game', '1');
		return true;
	},

	_setState_INTERMISSION_END : function(){
		if(SLINGO.Game.shouldSendStartGame){
			this.sendStartGameReq();
		} 
		
		return this._super();
	},

	startGame : function(){
		SLINGO.GameComm.setGameVar('start_game', '1');
	},

	sendStartGameReq : function(){
        if(!(this instanceof SLINGO.SocketlessStage))
            return SLINGO.SocketlessStage.sendStartGameReq();
        
		if(SLINGO.Game.isSample) return;

		if(!this.startGameURL)
			return SLINGO.triggerError('SocketlessStage.sendStartGameReq: no start game url',SLINGO.EWARN);

		var d = new Date();
		
		reqh = this.startGameURL +  d.getTime() +  "/" + SLINGO.Game.roomId ;
		
		if(!SLINGO.XHR.createAndSendRequest("sendStartGameReq", this.catchStartGameReq,  reqh, 'GET', 'JSON', true))
			return SLINGO.triggerError('SGSSocketless.sendStartGameReq: unable to create a valid XHR request.', SLINGO.EWARN);
	},

	catchStartGameReq : function(retv){
		SLINGO.GameRequest.startGameReqTime = retv.req_time ? retv.req_time : 'none';

		if(retv.logged_out && !SLINGO.Game.isSample){
			alert('You are not logged in. You will now be redirected to the login page.');
			location.href='/gen/login';
			return;
		}

		if(retv.fee){
			alert(retv.fee);
			return this._sendToLobby();
		}

		if(retv.msg && !retv.no_report)
			return SLINGO.GameComm.relay_reportError('GEN:JS-GENERATEDTICKET,TYPE=STARTGAME,ERROR='+retv.msg);

		if(retv.coins){
			if(SLINGO.isPluginLoaded('UserTools')) SLINGO.UserTools.updateCoinView(retv.coins);
			else SLINGO.triggerError('SocketlessStage.catchStartGameReq: UserTools plugin not loaded', SLINGO.EWARN);
		}

		if(retv.conv && (retv.conv != SLINGO.Game.coinConv)){
			SLINGO.Game.coinConv = retv.conv;
			SLINGO.GameComm.setGameVar('coin_conversion', SLINGO.Game.coinConv);
		}
	},
	
	initGameOverChecking : function(){
		SLINGO.GameComm.startGameOverPoll();
	}

});

SLINGO.WrapperlessStage = SLINGO.SocketlessStage.extend({
	_obj_class_name : 'WrapperlessStage',
	
	_onInit : function(){
		this._super();
		
		if(SLINGO.$('globalScoresLoading') && (this.gameCompanion) && ('refreshGlobalScores' in this.gameCompanion)){
			this.gameCompanion.refreshGlobalScores();
		}
		
		if(SLINGO.$('friendScoresLoading') && (this.gameCompanion) && ('refreshFriendScores' in this.gameCompanion)){
			this.gameCompanion.refreshFriendScores();
		}
	},

	/* Override SocketlessStage */
	startGame : function(){
		SLINGO.GameComm.startGame();
	},
	
	_onGameStart : function(){
		// calling super because this used to be the most parental function
		this._super();
		if('gamesPlayedLife' in SLINGO.Game)
			SLINGO.GameComm.setGameVar('games_played', SLINGO.Game.gamesPlayedLife);
	},
	_hasEndGameOverlay : function() {
		if (SLINGO.GamePage.gameConfig.hasNoEndGameScreen){
			return false;
		}else{
			return true;
		}
	}	
});

SLINGO.GenDailyChallengeStage = SLINGO.WrapperlessStage.extend({
	_obj_class_name : 'GenDailyChallengeStage',
	
	/* URL to submit a game score (override GameStage) */
	postScoreURL : '/ajax/genericdailychallenge/post_score/',
	
	postScoreFBCanvasURL : '/ajax/genericdailychallenge/post_score_fb_canvas/',
	imagePath : '/images/common/dailychallenge/',
	
	/* We don't need the start game request anymore. (override SocketlessStage) */
	startGameURL : false,
	currentChallengePrefix : false,	
	currentChallengeObject : false,	
	defaultPrefix : false,	
	firstPageLoad	: false,
	updatedForWin : false,
   	
	_onInit : function(){
		this._super();
		
		this._elm_post_vars.push('challenge_game_prefix');
		this._elm_post_vars.push('chal_token');
        
       if(SLINGO.Game.isSample){
			if(SLINGO.$('chatContainer'))
				SLINGO.$('chatContainer').style.backgroundColor ="#398CC0";
		}
		
		//Get the current game
		this.numChallenges = 0;
		var dc = SLINGO.Game.dailyChallenges.challenges;
		for(var key in dc) {
			if(dc[key].isCurrentChallenge) {
				this.currentChallengePrefix = key;
				this.currentChallengeObject = dc[key];
			}
			if(dc[key].isDefaultChallenge){
				this.defaultPrefix = key;
			}				
			this.numChallenges++;
		}
		if(SLINGO.Game.isDevMode == true){
			var srch = window.location.search;
			
			if(srch){
				srch = srch.split("?");	srch = srch[1]; srch = srch.split("&");
				var uriArray = [];
				for(i=0; i<srch.length; i++){ var urVar = srch[i].split("="); uriArray[urVar[0]] = urVar[1];}
				for(var i in uriArray){
					switch(i){
						case "games_played": SLINGO.Game.gamesPlayedLife = parseInt(uriArray[i]);	break;
						case "powerup_keys":
						case "powerup_aliases":	var powerups = uriArray[i]; if(powerups){ var powerups = powerups.split(','); if(powerups){	var inf = this.getCurrentChallengeInfo(this.getCurrentChallengePrefix()); if(i == "powerup_keys") inf.powerUpKeys = powerups; else inf.powerUpAliases = powerups;}} break;
						default: //do nothing..
					}
				}
			}
		}
	},
	
	/* overriding _setState_GAME_OVER on parent so I can hide game header */
	_setState_GAME_OVER : function(){
		if(SLINGO.$('sdcOverGame'))
			SLINGO.$('sdcOverGame').style.display = 'none';
			
		/*return */this._super();
		
		if('gamesPlayedLife' in SLINGO.Game)
			SLINGO.Game.gamesPlayedLife++;

		return true;
	},
	
	/* overriding _onGameStart on parent so I can show game header */
	_onGameStart : function(){
		/* MOVING COMMON TASKS TO TOP OF FUNCTION */
		// Set the user stats to be displayed on the end game screen. Added 2012.03.06 -NV
		/** @TODO IS THIS SHOWN FOR SAMPLE GAMES  **/
		if(SLINGO.Game.dailyChalUserStats !== false){
			this.prepareUserStatsForGame();
			var stats = SLINGO.JSON.stringify(SLINGO.Game.dailyChalUserStats);
			SLINGO.GameComm.setGameVar('user_data', stats);
		}
		
		if('gamesPlayedLife' in SLINGO.Game){
			SLINGO.GameComm.setGameVar('games_played', SLINGO.Game.gamesPlayedLife);
		}
		if(SLINGO.Game.isSample){
			//set chat height for sample games
			if (this.gameCompanion){
				this.gameCompanion.getElement().style.height= '573px';
			}
			
			//make sure end game overlay closes
			//if(SLINGO.$('sdc_endgame')) SLINGO.$('sdc_endgame').style.display = 'none';
			//if(SLINGO.$('sdcOverGame')) SLINGO.$('sdcOverGame').style.display = 'none';
			//get overlay obj
			var overlayElm = SLINGO.GamePage.getGameCycleMgr().getStageMgr().getChild('GAME')._getOverlayObj();
			if(overlayElm)
				overlayElm.style.display = 'none';
			//if(SLINGO.$('sdcGH_endGameMsg')) SLINGO.$('sdcGH_endGameMsg').style.display = 'none';
						
			//delete join post cookie
			SLINGO.setCookie('sljpcn', '');
			this.startChallenge();
		}else{
			if(this.hasSubChallenges()){
				//Check if diner menu should be shown before game start.
				if(!this.showDinerMenuAtGameStart())
					this.startChallenge();				
			} else this.startChallenge();
		}		
	},
	
	_postScore : function(score){
		this.postScoreArgs['chal_token'] = SLINGO.Game.dailyChalToken;
		this.postScoreArgs['challenge_game_prefix'] = this.getCurrentChallengePrefix();
		return this._super(score);
	},
	
	_catchPostScore : function(retv){
		if(retv.new_day){
			alert('A new Daily Challenge is available. Please click OK to play.');			
			this._sendToLobby();
			return;
		}
		
		this._super(retv);
		
		if(retv.global_top && SLINGO.$('sdcTopScore')) 	SLINGO.$('sdcTopScore').innerHTML = retv.global_top;
		if(retv.player_top && SLINGO.$('sdcHighScore'))	SLINGO.$('sdcHighScore').innerHTML = retv.player_top;

		/* adding the following two lines to update user high score for leaderboard */
		if(retv.player_top && SLINGO.$('userScoreGlobal'))		SLINGO.$('userScoreGlobal').innerHTML = retv.player_top;
		if(retv.player_top && SLINGO.$('userScoreFriends'))		SLINGO.$('userScoreFriends').innerHTML = retv.player_top;
		
		if(retv.challenge_score && (retv.challenge_score != SLINGO.Game.dailyChalGoal)){
			//@TODO is this used anywhere?
			SLINGO.Game.dailyChalGoal = parseInt(retv.challenge_score);
			var goal = SLINGO.fmtNumber(retv.challenge_score);
			if(SLINGO.$('sdcChallengeGoal')) 	SLINGO.$('sdcChallengeGoal').innerHTML = goal;
			if(SLINGO.$('overlayGoal')) 		SLINGO.$('overlayGoal').innerHTML = goal;
			/*if(this.isChalWin(retv.score)) 		SLINGO.Game.dailyChalWon++;*/
			
			//Overlay for current game has already been shown and score posted.
			//Update coins and rank award for NEXT game's overlay
			this.setDailyChallengeCoins(retv.coinsAward);
			this.setDailyChallengeRanks(retv.rankAward);
		}		
	},

	
	_showEndGameOverlay : function(score){
		SLINGO.triggerError('GenDailyChallengeStage::_showEndGameOverlay: score: '+score);
		var screenObj = SLINGO.GamePage.getGameCycleMgr().getStageMgr().getChild('GAME')._getOverlayObj();
		var playAgainButton = this.defaultPrefix + '_PlayAgain';		
		var isWin = this.isChalWin(parseInt(score));	
		if(isWin){ this.setWinScreen(score); }else{ this.setLoseScreen(score); }

		/* If this is a sample game, update the high score for just this page. */
		if(SLINGO.Game.isSample && SLINGO.$('sdcHighScore'))
			if(score > SLINGO.$('sdcHighScore').innerHTML.replace(',', ''))
				SLINGO.$('sdcHighScore').innerHTML = SLINGO.fmtNumber(score);
		
		if (SLINGO.GamePage.gameConfig.isDirect){
        	SLINGO.GameComm.getFlashMovie().style.height = '2px';
    		SLINGO.GameComm.getFlashMovie().style.width = '2px';
        }
		
		if(screenObj){
			this.displayOverlay(screenObj.id, playAgainButton, this, this._hideEndGameOverlay, [this.getCurrentChallengePrefix(), screenObj.id, isWin]);
		}else		
			SLINGO.triggerError('GenDailyChallengeStage::_showEndGameOverlay: no overlay to show');
		
	},
	
	 /* no need for game prefix and overlay to be passed along.. */
	_hideEndGameOverlay : function(gamePrefix, overlayName){
		if (SLINGO.GamePage.gameConfig.isDirect){
			var w = SLINGO.Game.gameWidth + 'px';
			var h = SLINGO.Game.gameHeight + 'px';
			SLINGO.GameComm.getFlashMovie().style.width = w;
			SLINGO.GameComm.getFlashMovie().style.height = h;
		}
		
		var screenObj = SLINGO.GamePage.getGameCycleMgr().getStageMgr().getChild('GAME')._getOverlayObj();
			if(screenObj){
				screenObj.style.display = "none";
			}else SLINGO.triggerError('GenDailyChallengeStage::_hideEndGameOverlay: no gamePrefix or overlayName sent. gamePrefix: '+gamePrefix+ ' overlayName' + overlayName);		

		SLINGO.triggerError('GenDailyChallengeStage::_hideEndGameOverlay: gamePrefix: '+gamePrefix+ ' overlayName' + overlayName);
                		
		if(SLINGO.Game.isSample){
			if(SLINGO.$('sdcOverGame')){SLINGO.$('sdcOverGame').style.display = "none";	}
			SLINGO.GamePage.setGameCycleState('GAME_END');
			return true;
		}
		if(!SLINGO.Game.isSample && gamePrefix) this.changeCurrentGame(gamePrefix);	
		if(this.firstPageLoad) 	this.startChallenge();
		else SLINGO.GamePage.setGameCycleState('GAME_END');
		
		return true;
	},
	
	startChallenge : function () {
		this.updatedForWin = false;
		this.setGameChallenge();
		this.setGamePowerUps();	
		
		this.setGameGoal();
		this.startGame();
		this.firstPageLoad = false;
		this.initGameOverChecking();

		if(SLINGO.$('sdcOverGame'))
			SLINGO.$('sdcOverGame').style.display = 'block';
		
		var currentPrefix = this.getCurrentChallengePrefix();        
		//Set game banner and challenge goal background		
		if(SLINGO.$('challengesHeader')){
			var dc = SLINGO.Game.dailyChallenges.challenges;
			var challengePrefixForGame = dc[currentPrefix].challengePrefixForGame;
			SLINGO.$('challengesHeader').className = challengePrefixForGame + 'Header headerMenu' + SLINGO.GamePage.gameConfig.gameId;
		}

		//Show correct goal
		var challengeGoalClass = SLINGO.getElementsByClass('aboveGameChallengeGoal');
		if(challengeGoalClass.length > 0){
			var challengeGoalId = challengeGoalClass[0].id;
			if(SLINGO.$(challengeGoalId)) SLINGO.$(challengeGoalId).innerHTML = SLINGO.fmtNumber(SLINGO.Game.dailyChallenges.challenges[currentPrefix].goal);
		}
		//set leaderboard prefix
		this.setLeaderBoardPrefix();

		var lbClassName = currentPrefix + 'Leaderboard';
		if(SLINGO.$('sideLBScoreListFriends')){
                    if(!SLINGO.hasClassName(SLINGO.$('sideLBScoreListFriends'), lbClassName)){
			if((this.gameCompanion) && ('refreshGlobalScores' in this.gameCompanion)) this.gameCompanion.refreshGlobalScores(); 
                        if('refreshFriendScores' in this.gameCompanion) this.gameCompanion.refreshFriendScores();
                    }
                }
	},
	
	showDinerMenuAtGameStart : function() {
		
		var dc = SLINGO.Game.dailyChallenges.challenges;
		// Go through array and check if there's a current challenge, if there isn't one, display diner menu
		var playersChoice = 1;
		for(var key in dc) {
			if(dc[key].isCurrentChallenge) {
				playersChoice = 0;
				break;
			}
		}

		// If this is the case, display the diner menu.
		if(playersChoice){			
			var overlayDiv = SLINGO.getElementsByClass('endGameMsg');
			if(overlayDiv.length > 0){
				var screenID = overlayDiv[0].id;
				this.setWelcomeScreen(dc);
				if (SLINGO.GamePage.gameConfig.isDirect){
		        	SLINGO.GameComm.getFlashMovie().style.height = '2px';
		    		SLINGO.GameComm.getFlashMovie().style.width = '2px';
		        }
				this.displayOverlay(screenID, null, null, null, null);
                if(SLINGO.Game.isWide){
					SLINGO.$(screenID).style.width = this._getGameWidth() + 'px';               
					SLINGO.$(screenID).style.display = 'block';
				}
				
				this.firstPageLoad = true;
				return true;
			}else {
				//no end game overlays found...
				return false;
			}			
		} else {
			this.firstPageLoad = false;
			return false;
		}
	},
			
	setWelcomeScreen : function(dc){
		this.setBtnEvents(dc);
        this.setChallengesGoal(dc);
		this.setClass('dmBannerImg', 'endGameButtons ' + this.getImgName('welcome'));
		this.showHideElement({'addHide': ['beatChallengeTitle', 'overlayWinnings', 'overlayScoreBox', 'overlayGoalBox'], 'removeHide' : ['tryNextChalTitle']});
		this.showBaseChallengeCont();
	},
			
	setWinScreen : function(score){
		this.setClass('dmBannerImg', 'endGameButtons ' + this.getImgName('win'));
        this.setEndScreenAmts(score, true);
		if(!this.updatedForWin){this.updateChallengeEnabledForGameWin(score);}
		this.showHideElement({'addHide': ['beatChallengeTitle'], 'removeHide' : ['tryNextChallenge', 'overlayScoreBox', 'overlayWinnings', 'overlayGoalBox'], 'hide' : ['lock']});
		if(this.hasSubChallenges()) this.showBaseChallengeCont();
	},
	
	setLoseScreen : function(score){
		this.setClass('dmBannerImg', 'endGameButtons ' + this.getImgName('lose'));
        this.setEndScreenAmts(score, false);
		this.showHideElement({'removeHide' : ['overlayScoreBox', 'overlayWinnings', 'overlayGoalBox']});
		//this.updateButtons(score);
	},
			
	setEndScreenAmts : function(score, isWin){
		var currentChallenge = this.currentChallengeObject;		
		if(SLINGO.$('overlayGoal'))	SLINGO.$('overlayGoal').innerHTML = SLINGO.fmtNumber(currentChallenge.goal);
		if(SLINGO.$('overlayScore'))SLINGO.$('overlayScore').innerHTML = SLINGO.fmtNumber(score);
		
		if(SLINGO.$('overlayCoins'))	SLINGO.$('overlayCoins').innerHTML = SLINGO.fmtNumber(this.getCoinAward(score, isWin));
		if(SLINGO.$('overlayRank'))		SLINGO.$('overlayRank').innerHTML = SLINGO.fmtNumber(this.getRankAward(score, isWin));
	},
			
    setChallengesGoal : function(dc){
        for(var  challengePrefix in dc){
             //this.setChallengeGoal(key, {'goal_score': dc[challengePrefix].goal});
			 this.updateChallengeGoal(challengePrefix, dc[challengePrefix]);
        }
    },
			
	updateChallengeGoal : function(challengePrefix, challengeObject){
		var  prefix = challengeObject.challengePrefixForGame;
		if(prefix == 'Base') prefix = challengePrefix;
		var goalId =  prefix + 'Goal';
		var goalAmtId = prefix + 'GoalAmt';
		if(SLINGO.$(goalId)){
			if(challengeObject.hasBeaten){
				if(SLINGO.$(goalAmtId)){
					SLINGO.$(goalAmtId).innerHTML = 'DEFEATED!!';
				    }
				
				var checkMarkName = prefix + 'CheckMark';
				if(SLINGO.$(checkMarkName)){ 
				    SLINGO.addClassName(SLINGO.$(checkMarkName),'checked');
				}
			}else{
				if(SLINGO.$(goalAmtId))  
					SLINGO.$(goalAmtId).innerHTML = SLINGO.fmtNumber(challengeObject.goal);
			}			
		}
	},
	
	setGamePowerUps : function(){
			SLINGO.GameComm.setGameVar('config_reset', this.getCurrentPowerUpKeys());		
	},
	
	setGameChallenge : function(){
		var curPrefix = this.getCurrentChallengePrefix();
		var currentChallengeInfo = this.currentChallengeObject;
		
		//SLINGO.triggerError('GenDailyChallengeStage.setGameChallenge: ' + gaPfx.toLowerCase());
		if(!curPrefix){			
			curPrefix = this.currentChallengePrefix = this.defaultPrefix;
			currentChallengeInfo = this.currentChallengeObject = SLINGO.Game.dailyChallenges.challenges[this.defaultPrefix];
			SLINGO.triggerError('GenDailyChallengeObject::setChallengeProps: no current prefix, setting default prefix');
		}
		var gaPfx = (this.defaultPrefix == curPrefix) ? "base" : currentChallengeInfo.challengePrefixForGame;
		
		SLINGO.GameComm.setGameVar('set_challenge', gaPfx.toLowerCase());
		SLINGO.GameComm.setGameVar('game_prefix', curPrefix);
	},
	
	setGameGoal : function(){
		var curPrefix = this.getCurrentChallengePrefix();
		var dc = SLINGO.Game.dailyChallenges.challenges; 
		if (curPrefix in dc){
			SLINGO.triggerError('GenDailyChallengeStage.setGameGoal: ' + dc[curPrefix].goal);
			SLINGO.GameComm.setGameVar('challenge_score', dc[curPrefix].goal);
		}else{
			SLINGO.triggerError('GenDailyChallengeStage.setGameGoal() Could not set game goal');
		}
	},
	
	//Gets challengeInfo object for current challenge
	getCurrentChallengeInfo : function (gamePrefix) {
		return SLINGO.Game.dailyChallenges.challenges[gamePrefix];
	},
	
	/* Gets the current game prefix */
	getCurrentChallengePrefix : function(){
		return this.currentChallengePrefix;
	},
	
	getDefaultPrefix : function(){
		return this.defaultPrefix;
	},
	
	hasSubChallenges : function(){
		return this.numChallenges > 1;
	},
	
	getCurrentPowerUpKeys : function(){
		SLINGO.triggerError('GenDailyChallengeStage.getCurrentPowerUpKeys');
		var dc = SLINGO.Game.dailyChallenges.challenges;
		var prefix = this.getCurrentChallengePrefix();
		if(prefix){
			return dc[prefix].powerUpKeys;
		}else{
			SLINGO.triggerError('GenDailyChallengeStage.getCurrentPowerUpKeys no current prefix set! could not get powerups');
			return false;
		}
	},
	
	getCurrentPowerUpAliases : function(){		
		//Get the current game
		var dc = SLINGO.Game.dailyChallenges.challenges;
		for(var key in dc)
			if(dc[key].isCurrentChallenge)
				return dc[key].powerUpAliases;
	
		return false;
	},
	
	isChalWin : function(score){		
		if(!this.currentChallengeObject)
			SLINGO.triggerError('GenDailyChallengeStage.isChalWin: currentChallengeObject has not been set');
		
		var curPrefix = this.getCurrentChallengePrefix();
		var dcInfo = this.getCurrentChallengeInfo(curPrefix);

		switch(dcInfo.winType){
		    case 'GAME_SET':
			var isWin = parseInt(SLINGO.GameComm.getGameVar('game_won'));
			break;
			
		    case 'SCORE':
		    default:
			var isWin = score >= parseInt(this.currentChallengeObject.goal);
			break;
		}
		
		SLINGO.triggerError('GenDailyChallengeStage.isChalWin isWin: '+ isWin+', winType: '+dcInfo.winType);
		return isWin;
	},
	
	getCoinAward : function(score, isWin){
		if(!this.currentChallengeObject)
			return SLINGO.triggerError('GenDailyChallengeStage.getCoinAward: currentChallengeObject has not been set');
		
		var score = parseInt(score);
		//if(!this.updatedForWin){this.updateChallengeEnabledForGameWin(score);}
		if(isWin){
			if(!this.updatedForWin){this.updateChallengeEnabledForGameWin(score);}
		}
		var winCoins = (parseInt(this.currentChallengeObject.coins.win) + parseInt(this.currentChallengeObject.coins.play));
		var playCoins = parseInt(this.currentChallengeObject.coins.play);
		var coins = score 
						? (isWin
							? ((this.currentChallengeObject.isFirstWin) ? winCoins : playCoins)
							: playCoins) 
						: '0';	

		if(SLINGO.Game.coinConv){
			coins = coins * SLINGO.Game.coinConv;
			coins = Math.ceil(coins);
		}
		return coins;
	},
	
	getRankAward : function(score, isWin){
		if(!this.currentChallengeObject)
			return SLINGO.triggerError('GenDailyChallengeStage.getRankAward: currentChallengeObject has not been set');

		var score = parseInt(score);
		//if(!this.updatedForWin){this.updateChallengeEnabledForGameWin(score);}
		if(isWin){
			if(!this.updatedForWin){this.updateChallengeEnabledForGameWin(score);}
		}
		var winRank = (parseInt(this.currentChallengeObject.rank.win) + parseInt(this.currentChallengeObject.rank.play));
		var playRank = parseInt(this.currentChallengeObject.rank.play);
		var rank = score 
						? (isWin 
							? ((this.currentChallengeObject.isFirstWin) ? winRank : playRank)
							: playRank) 
						: '0';	

		if(SLINGO.Game.rankMultiplier){
			rank = rank * SLINGO.Game.rankMultiplier;
			rank = Math.ceil(rank);
		}
		return rank;
	},
	
	setLeaderBoardPrefix : function(){
		var prefix = this.getCurrentChallengePrefix();
		if((this.gameCompanion) && ('setPrefix' in this.gameCompanion)) this.gameCompanion.setPrefix(prefix);
	},
			
	setBtnEvents : function(challenges){
		for(var challengePrefix in challenges){
			var key = challenges[challengePrefix].challengePrefixForGame;
            if(challenges[challengePrefix].enabledChallenge){
                if(key == 'Base') key = challengePrefix;
                var buttonID	= key + "Button";
                if(SLINGO.$(buttonID)){
                    if(!SLINGO.Event.addEvent(SLINGO.$(buttonID), 'onclick', bindWithArgs(this, this._hideEndGameOverlay,  [challengePrefix, undefined, null])))
                        SLINGO.triggerError('GenDailyChallengeStage.updateButtons: Cannot add event to button for ' + currentChallengePrefix+"Button");
                }
            }
			
		}
	},
			
	showHideElement : function(elems){
		for(var action in elems){
			var eleArr = elems[action];			
			switch(action){
				case 'removeHide': 	var funct = SLINGO.removeClassName; var args = 'hide';	break;					
				case 'addHide': funct = SLINGO.addClassName; args = 'hide'; break;
				case 'display': funct = this.displayHideDiv; 	args = 'display'; break;
				case 'hide' : funct = this.displayHideDiv; args = 'hide'; break;
				default: /*if no action sent, go to next iteration*/ continue;
			}
			
			for(var i=0; i< eleArr.length; i++){
				var obj = SLINGO.$(eleArr[i]);
				if(obj){funct.apply(obj, [obj, args]); }
			}
		}
	},
			
	setClass : function(objName, classes){
		if(SLINGO.$(objName)){
				SLINGO.$(objName).className = (classes); 
		}else SLINGO.triggerError('GenDailyChallengeStage.setClass, null obj sent to func, obj name sent:' + objName + ' classes: ' + classes);	
	},	

	getImgName : function(which){
		switch(which){
			case "win": var imgClass = 'winImg'; break;
			case "lose": imgClass = 'loseImg'; break;
			case "welcome": imgClass = 'welcomeImg'; break;
			default: '';
		}
		
		return imgClass;
		/*if(SLINGO.$('dmBannerImg'))
			SLINGO.$('dmBannerImg').className = 'endGameButtons ' + imgClass;
		*/
	},
			
	showBaseChallengeCont : function(){

		if(SLINGO.$('playBtnCont')){
			var playBtn = SLINGO.$('playBtnCont').innerHTML;
			SLINGO.$('playBtnCont').parentNode.removeChild(SLINGO.$('playBtnCont'));
		}
		if(SLINGO.$('BaseChallengePlayBtnCont') ) {
			SLINGO.$('BaseChallengePlayBtnCont') .innerHTML += playBtn;
		}

		var playAgainBtn = this.defaultPrefix + "_PlayAgain";
		if(SLINGO.$(playAgainBtn)){
			if(!SLINGO.Event.addEvent(SLINGO.$(playAgainBtn), 'onclick', bindWithArgs(this, this._hideEndGameOverlay,  [this.defaultPrefix, undefined, null])))
					SLINGO.triggerError('GenDailyChallengeStage.updateButtons: Cannot add event to button for playAgainBtn');
		}
        
        if(SLINGO.$('baseChallengeContainer')){
            SLINGO.removeClassName(SLINGO.$('baseChallengeContainer'), 'hide');
        }
	},
	
	
	//Updates challengedEnabled status affected by current game win
	updateChallengeEnabledForGameWin : function(score) {
		SLINGO.triggerError('GenDailyChallengeStage.updateChallengeEnabledForGameWin');
		if(this.updatedForWin === true) return;
		var currentChallengePrefix = this.getCurrentChallengePrefix();
		var dc = SLINGO.Game.dailyChallenges.challenges;
		if(!dc[currentChallengePrefix].hasBeaten){
			dc[currentChallengePrefix].hasBeaten = 1;
			dc[currentChallengePrefix].isFirstWin = 1;
			this.updateChallengeGoal(currentChallengePrefix, dc[currentChallengePrefix]);
			
			if ('Base' == dc[currentChallengePrefix].challengePrefixForGame){
				var buttonID	= dc[currentChallengePrefix].challengePrefix + "Button";				
				if (SLINGO.$(buttonID)){
					if(!SLINGO.Event.addEvent(SLINGO.$(buttonID), 'onclick', bindWithArgs(this, this._hideEndGameOverlay, [currentChallengePrefix]))){
						SLINGO.triggerError('GenDailyChallengeStage.updateChallengeEnabledForGameWin: Cannot add event to button for ' + buttonID);
					}
				}
			}
			
		}else dc[currentChallengePrefix].isFirstWin = 0;
			
		
		//Update accordingly if current win enables new challenges.
		for(var key in dc){
			if(key == currentChallengePrefix)
				continue;
			
			//if already enabled no point in checking
			if(dc[key].enabledChallenge)
				continue;
			
			var preReqMet = 0;
			for(var i in dc[key].preReq) {
				if(typeof dc[key].preReq[i]!="string")
					continue;

				if(dc[dc[key].preReq[i]].hasBeaten)
					preReqMet = 1;
				else {
					preReqMet = 0;
					break;
				}
			}
			
			if(preReqMet){
				dc[key].enabledChallenge = 1;					
				var buttonID	= dc[key].challengePrefixForGame + "Button";
				var goalAmtID	= dc[key].challengePrefixForGame + "Goal";
				//var goalTxtID	= dc[key].challengePrefixForGame + "GoalTxt";
				//remove locked state
				if(SLINGO.$(buttonID))		SLINGO.removeClassName(SLINGO.$(buttonID), 'locked');
				//remove hide state for amounts
				if(SLINGO.$(goalAmtID))		SLINGO.removeClassName(SLINGO.$(goalAmtID), 'hide');
				
				//add onclick button to enabled challenge	
				if(!SLINGO.Event.addEvent(SLINGO.$(buttonID), 'onclick', bindWithArgs(this, this._hideEndGameOverlay, [key])))
					SLINGO.triggerError('GenDailyChallengeStage.updateButtons: Cannot add event to button for ' + buttonID);
			}
		}
		this.updatedForWin = true;
		return true;
	},
	
	//Changes the isCurrentChallenge property for currently active challenge
	changeCurrentGame : function(newCurrentChalPrefix) {
		var oldCurrentChallengePrefix = this.getCurrentChallengePrefix();
		
		//do we have to change to a new challenge?
		if(oldCurrentChallengePrefix == newCurrentChalPrefix)
			return true;
		
		//set new current challenge stuff
		SLINGO.Game.dailyChallenges.challenges[newCurrentChalPrefix].isCurrentChallenge = 1;
		this.currentChallengePrefix = newCurrentChalPrefix;
		this.currentChallengeObject = SLINGO.Game.dailyChallenges.challenges[newCurrentChalPrefix];
		
		if(!this.firstPageLoad){
			if(oldCurrentChallengePrefix)
				SLINGO.Game.dailyChallenges.challenges[oldCurrentChallengePrefix].isCurrentChallenge = 0;
		}	
		//set leaderboard prefix
		this.setLeaderBoardPrefix();			
        if((this.gameCompanion) && ('refreshGlobalScores' in this.gameCompanion)) this.gameCompanion.refreshGlobalScores(); 
        if((this.gameCompanion) && ('refreshFriendScores' in this.gameCompanion)) this.gameCompanion.refreshFriendScores();

		return true;
	},
	
	setDailyChallengeCoins : function(coins){
		if(coins.win)	SLINGO.Game.dailyChalCoins.win=parseInt(coins.win);
		if(coins.play)	SLINGO.Game.dailyChalCoins.play=parseInt(coins.play);
	},
	
	setDailyChallengeRanks : function(ranks){ 
		if(ranks.win)	SLINGO.Game.dailyChalRanks.win=parseInt(ranks.win);
		if(ranks.play)	SLINGO.Game.dailyChalRanks.play=parseInt(ranks.play);
	},
	
	/* Process the user stats that were retrieved from the game on game over. */
	handleUserStats : function(stats){
		// If there are no user stats defined, this game doesn't support them. Move along.
		if(SLINGO.Game.dailyChalUserStats === false) return;
		
		// Get user_stats from game and process updates into existing user stats object.			
		var stats = SLINGO.GameComm.getGameVar('user_data');
		if(stats && this.processUserStatsUpdates(stats))
			this.sendUserStatsReq();
	},
	
	/* Take updates from the game or the server into account. */
	processUserStatsUpdates : function(stats){
		// If there are no user stats defined, this game doesn't support them. Move along.
		var oldStats = SLINGO.Game.dailyChalUserStats;
		if(oldStats === false)return;

		// Convert the string to a JSON object.
		var s = stats;
		if(typeof stats != 'object'){
			try{
				var stats = SLINGO.JSON.parse(stats);
				if (typeof stats != 'object'){
					throw 'stats are not an object';
				}
			}catch(e){
				return SLINGO.triggerError('GenDailyChallengeStage.processUserStatsUpdates: invalid stats: '+s,SLINGO.EWARN);
			}
		}
		
		// Loop through JSON object from game and extract the new values.
		var newStats = {};
		for(var i = 0; i < stats.length; i++)
			newStats[stats[i].var_name] = stats[i].value;
			
		// Loop through the existing user stats and update any new records.
		for(var i = 0; i < oldStats.length; i++){
			// Make sure the old stat exists in the new stats.
			if(!(oldStats[i].var_name in newStats)){
				SLINGO.triggerError('GenDailyChallengeStage.processUserStatsUpdates: missing old stat in new stats: '+oldStats[i].var_name,SLINGO.EWARN);
				continue;
			}
			
			var oldVal = parseInt(oldStats[i].value);
			var oldVar = oldStats[i].var_name;
			
			// Only update lifetime stats if the new value is a record.
			if((oldVar.indexOf('lifetime_') === 0) || (oldVar == 'best_score')){
				var currTotalVar = oldVar.replace('lifetime_', '');
				if(!(currTotalVar in newStats)){
					SLINGO.triggerError('GenDailyChallengeStage.processUserStatsUpdates: missing curr total var in new stats: '+currTotalVar,SLINGO.EWARN);
					continue;
				}
				
				if(oldVal < parseInt(newStats[currTotalVar]))
					oldStats[i].value = newStats[currTotalVar];
			
			// Otherwise just overwrite the old stat with the new stat.
			} else oldStats[i].value = newStats[oldVar];
		}
				
		return true;
	},
	
	prepareUserStatsForGame : function(){
		for(var i = 0; i < SLINGO.Game.dailyChalUserStats.length; i++)
			if(SLINGO.Game.dailyChalUserStats[i].var_name.indexOf('total_') === 0)
				SLINGO.Game.dailyChalUserStats[i].value = '0';
	},
	
	/* Send an AJAX request to the server to update the user stats. */
	sendUserStatsReq : function(){
		// If there are no user stats defined, this game doesn't support them. Move along.
		var stats = SLINGO.Game.dailyChalUserStats;
		if(stats === false) return;
			
		// Set up the post variables to be logged.
		var post = {
			score 					: SLINGO.Player.lastScore,
			user_stats				: SLINGO.JSON.stringify(stats),
			game_id					: SLINGO.Game.gameId,
			challenge_game_prefix	: this.getCurrentChallengePrefix()
		};
		
		// Send the AJAX request.
		var d = new Date;
		if(!SLINGO.XHR.createAndSendRequest('sendUserStats_'+d.getTime(), bind(this, this.catchUserStatsReq), '/ajax/genericdailychallenge/process_user_stats/'+d.getTime(), 'POST', 'JSON', true, post))
			return SLINGO.triggerError('GenDailyChallengeStage.sendUserStatsReq: could not send XHR request',SLINGO.EWARN);
			
		return true;
	},
	
	/* Receive the AJAX response and process any updates to the user stats (if the user is playing in more than one window). */
	catchUserStatsReq : function(retv){
		if(retv.msg)
			SLINGO.triggerError('GenDailyChallengeStage.catchUserStatsReq: '+retv.msg,SLINGO.EWARN);
			
		if(retv.stats) {
			this.processUserStatsUpdates(retv.stats);
		}
	}
	
});

/* Alternate Daily Challenge stage for Daily Challenge-type games that have the end game screen in Flash. */
SLINGO.AltGenDailyChallengeStage = SLINGO.GenDailyChallengeStage.extend({
	_obj_class_name : 'AltGenDailyChallengeStage',
	
	_onInit : function(){
		this._super();

		/* Override powerups with test powerups provided in the URL. 
			Example: /sgs/play/501?powerups=coalToDiamond,luckyColorTopaz */
		if(SLINGO.Game.isDevMode == true){
			var srch = window.location.search;
			if(srch){
				srch = srch.replace('?powerups=','');
				if(srch){
					var srch = srch.split(',');
					if(srch){
						var inf = this.getCurrentChallengeInfo(this.getCurrentChallengePrefix());
						inf.powerUpAliases = srch;
					}
				}
			}
		}
	},

	_setState_GAME_OVER : function(){
		var score = SLINGO.GameComm.getGameVar('game_score');
		this._super();
		var isWin = this.isChalWin(score);
		SLINGO.GameComm.setGameVar('game_awards', this.getCoinAward(score, isWin)+','+this.getRankAward(score, isWin));
	},
	
	/* Override GenDailyChallengeStage.setGamePowerUps() since this uses the powerup alias. */
	setGamePowerUps : function(){
		var aliases = this.getCurrentPowerUpAliases();
		if(!aliases) return SLINGO.triggerError('AltGenDailyChallengeStage.setGamePowerUps: no powerup aliases found',SLINGO.EWARN);
		var aliases = aliases.join(',');
		SLINGO.GameComm.setGameVar('init_powerups', aliases);
		SLINGO.GameComm.setGameVar('config_reset', aliases);
	}
});

SLINGO.BlastStage = SLINGO.WrapperlessStage.extend({
	_obj_class_name : 'BlastStage',

	/* Slingo Blast specific page element handles */
	tuteElement			: null,

	musicOn				: true,		/* Whether or not the user has turned game music on or off. */
	sfxOn				: true,		/* Whether or not the user has turned game sound effects on or off. */

	lastSparkConfig		: '',
	newSparkConfig		: '',
	selectedFeatures	: [],		/* Array of feature ids that the user has currently selected. */
	maxFeatures			: 0,		/* The max number of features a user can select for this game. */
	nextSlotToFill		: 1,		/* The next powerup slot available to fill. */
	featureInfo			: {},		/* Information about each possible feature (just powerups right now) */
	featureUnlockOrder	: false,	/* An array of feature ids in the order in which they are unlocked. */
	powerupSlotOrder	: [],		/* An array of powerup slot feature ids sorted by the order in which they are unlocked. */
	unlockedFeature		: false,	/* The id of the feature the player just unlocked. */
	unlockedPrevFeature	: false,	/* The id of the last feature that was unlocked. */
	unlockedNextFeature	: false,	/* The id of the next feature the player will unlock. */
	blocksDetonated		: 0,		/* The number of blocks detonated from the game that just ended. */

	/* features pagination */
	numFeatures			: 0,		/* The number of features to be displayed on the config screen. */
	currFeaturePage		: 1,		/* The current page of features being displayed. */
	featuresPerPage		: 6,		/* The number of features to display per page on the poweurp config screen. */
	score				: false,

	postScoreURL 		: '/ajax/sgs/post_blast_score/',
	postEndGameURL 		: '/ajax/sgs/post_blast_endgame/',
	postSparkConfigURL	: '/ajax/sgs/post_blast_sparkconfig/',

	_onInit : function(){
		this._super();
		this.addChild('TOPBAR'	, new SLINGO.StageDisplay('gameTopBar'));
		this.addChild('HELP'	, new SLINGO.StageDisplay('blastTutorial'));
		this.addChild('POWERUPS', new SLINGO.AccordionMenu('powerupPanel'));	// this.sparkScr
		this.getChild('POWERUPS').setArea(SLINGO.$('powerupSelectionArea'));
		
		this._game_area_mgr.addStage('SPLASH'	, new SLINGO.StageDisplay('titleScreen'));		// this.logoScr
		this._game_area_mgr.addStage('UNLOCKED'	, new SLINGO.StageDisplay('unlockedFeature'));
		this._game_area_mgr.addStage('SUMMARY'	, new SLINGO.StageDisplay('gameSummary'));		// this.tallyScr
		this._game_area_mgr.addStage('ENDSCREEN'	, new SLINGO.StageDisplay('sgs_endGameMsg'));	
		
		this.tuteElement 	= SLINGO.$('tuteElement');
		this.lastSparkConfig 	= SLINGO.Game.blastSparkConfig;
		this.featureInfo 		= SLINGO.Game.blastFeatures;
		
		/* set up our onclick handlers */
		SLINGO.$('titlePlayNow').onclick 		= bind(this, this.showGameScreen);
		SLINGO.$('showHelpBtn').onclick 		= bind(this, this.toggleHelpScreen);
		SLINGO.$('tuteCloseBtn').onclick		= bind(this, this.toggleHelpScreen);
		SLINGO.$('summaryPlayNow').onclick		= bind(this, this._showEndGameScreen); //function(){SLINGO.GamePage.setGameCycleState('GAME_OVER');};
		SLINGO.$('powerupsPlayNow').onclick		= bind(this, this.startBlastGame);
		SLINGO.$('toggleMusicBtn').onclick		= bind(this, this.toggleMusic);
		SLINGO.$('toggleSfxBtn').onclick		= bind(this, this.toggleSfx);
		SLINGO.$('sparkLeftScrollBtn').onclick	= bind(this, this.showPrevFeatures);
		SLINGO.$('sparkRightScrollBtn').onclick	= bind(this, this.showNextFeatures);

		this.setFeatureUnlockOrder();
		this.initPowerupScreen();
		this.loadLastConfig();
		this.refreshGameElements();

	},
	_setState_GAME_OVER : function(){
		this.getChild('POWERUPS').setState('HIDE');
		var score = SLINGO.GameComm.getGameVar('game_score');
		if(score > 0){
			/*++SLINGO.Game.blastGamesLife;
			this.refreshGameElements();*/
			var newGames = SLINGO.Game.blastGamesLife + 1;
			var arg = {games_played : newGames};
			if(this.unlockedNextFeature && (this.featureInfo[this.unlockedNextFeature].games_to_unlock === newGames))
				arg.unlocked = [this.unlockedNextFeature];
			this.processServerUpdates(arg, true);
		}
		this.processBatchEvent(score);
		return this._super();
	},

	_setState_GAME_END : function(){
		this.getChild('TOPBAR').setState('HIDE');
		return this._super();
	},

	_setState_INTERMISSION_START : function() {
		this._game_area_mgr.getElement().style.height = '';
		return this._super()
	},
	/*	This will be called from SocketlessStage's _setState_GAME_START().
		If this is the user's first game, go straight to the game.
		Otherwise, show the splash screen. */
	_onGameStart : function(){
		return ((SLINGO.Game.blastGamesLife < 1) || (SLINGO.Player.getGamesPlayed() > 0))
			? this.showGameScreen()
			: this.showSplashScreen();
	},

	showSplashScreen : function(){
		return this.showGameAreaStage('SPLASH');
	},

	showGameScreen : function(){
		this.showGameAreaStage('GAME');
		this.getChild('TOPBAR').setState('SHOW');

		if(SLINGO.Game.blastGamesLife > 0)
			this.showPowerupScreen();
		else this.startBlastGame();

		this._game_area_mgr.getElement().style.height = '573px';
		this._game_area_mgr.getChild('GAME').getElement().style.height = '525px';
		return true;
	},

	startBlastGame : function(){
		/* Minimize the powerup panel to be underneath the game. */
		this.getChild('POWERUPS').setState('SHOW');
		this.getChild('POWERUPS').setState('MIN');

		/* Adjust the game area height so the minimized powerup panel can fit underneath the game. */
		this._game_area_mgr.getChild('GAME').getElement().style.height =
			SLINGO.Game.gameHeight
			+ SLINGO.DHTML.getObjectHeight(SLINGO.$('selectedPowerupPanel'))
			+ 'px';

		/* Format the new spark config, and empty the "cart". */
		var config = [];
		if(this.selectedFeatures){
			for(var i = 0; i < this.selectedFeatures.length; i++){
				var id = this.selectedFeatures[i];
				if(this.featureInfo && this.featureInfo[id] && this.featureInfo[id].swf)
					config.push(this.featureInfo[id].swf);
			}
			this.featuresForSummary = this.selectedFeatures;
			//this.selectedFeatures = [];
		}

		this.newSparkConfig = config.join(',');

		/* Send the game the spark config and lifetime number of games played for this user */
		SLINGO.GameComm.setGameVar('init_sparks', this.newSparkConfig+(config.length ? ',' : '')+'reelDevilSpark');
		SLINGO.GameComm.setGameVar('games_played', parseInt(SLINGO.Game.blastGamesLife) ? SLINGO.Game.blastGamesLife : 0);

		/* Reset the player's score from the last game */
		SLINGO.Player.lastScore = 0;

		/* Fire up the game and set the current spark config */
		this.startGame();
		this.initGameOverChecking();
		this.postSparkConfig();
		return true;
	},
			
	_showEndGameScreen : function(){
		var score = this.score;
		var playAgainButton = 'sgs_PlayAgain';
		
		if(SLINGO.$(playAgainButton)){
			if(!SLINGO.Event.addEvent(SLINGO.$(playAgainButton), 'onclick', bind(this, this._hideEndGameScreen)))
				SLINGO.triggerError('WrapperlessStage._showEndGameOverlay: Cannot add event to button for ' + playAgainButton);
		}		
		
		if(SLINGO.$('overlayScore')) SLINGO.$('overlayScore').innerHTML = SLINGO.fmtNumber(score);
		if(SLINGO.$('overlayCoins'))	SLINGO.$('overlayCoins').innerHTML = SLINGO.fmtNumber(this.getCoinAward(SLINGO.Player.lastScore));

		/* Added 2011.11.21 -NV */
		if(SLINGO.$('overlayRank'))		SLINGO.$('overlayRank').innerHTML = SLINGO.fmtNumber(this.getRankAward(score));
		
		this.showGameAreaStage('ENDSCREEN');
		var screenID = 'sgs_endGameMsg';
		if(SLINGO.$(screenID)){
			SLINGO.$(screenID).style.display = 'block';
			var oldScreenH = SLINGO.DHTML.getObjectHeight(screenID);
			var stageH = SLINGO.DHTML.getObjectHeight(this.stage.id);
			var screenH = (oldScreenH < stageH) ? stageH - 4 : oldScreenH;
			SLINGO.$(screenID).style.height = screenH + 'px';
			this.getChild('TOPBAR').setState('HIDE');
			//adjust top margin             
			var nodeArr = SLINGO.getElementsByClass('endGameContent');
			if(nodeArr.length > 0){
				var endContentId = nodeArr[0].id;
				var endContentH = SLINGO.DHTML.getObjectHeight(endContentId);
				if(screenH > endContentH){
					var topMargin =  (screenH - endContentH)/2;
					/* because of IE7 buggy margin top */
					var parentId = SLINGO.$(nodeArr[0].id).parentNode.id;
					if(SLINGO.$(parentId)){
						SLINGO.$(parentId).style.paddingTop= topMargin + "px";
						SLINGO.$(nodeArr[0].id).style.margin = '0 auto';
					}
				}
			}           
		}
		return;	
	},
			
	_hideEndGameScreen	: function(){
		SLINGO.GamePage.setGameCycleState('GAME_END');
	},

	showPowerupScreen : function(){
		var games = SLINGO.Game.blastGamesLife;
		var puScreen = this.getChild('POWERUPS');

		/* Determine if we need to display a First User Experience tutorial. */
		if(SLINGO.isPluginLoaded('Tutorial')){
			var steps = [];
			switch(games){
				case 1:
					/* Add the step where the user must select the Time Bonus powerup to advance. */
					var step = new SLINGO.TutorialStep();
					step.addClickableArea(SLINGO.$('blastItem_2'), SLINGO.$('firstPowerupTutorial'));
					steps.push(step);
					break;

				case 3:
					/* Add the step where the user must unselect the Time Bonus powerup. */
					var step = new SLINGO.TutorialStep();
					step.addClickableArea(SLINGO.$('blastItem_2'), SLINGO.$('removePowerupTutorial'));
					steps.push(step);

					/* Add the step where the user must select the Multipliers powerup. */
					var step2 = new SLINGO.TutorialStep();
					step2.addClickableArea(SLINGO.$('blastItem_3'), SLINGO.$('replacePowerupTutorial'));
					steps.push(step2);
					break;

				case 4:
					/* Add the step to remind the user that they can purchase powerups. */
					var step = new SLINGO.TutorialStep();
					step.addClickableArea(SLINGO.$('okBtn'), SLINGO.$('purchasePowerupTutorial'));
					step.addClickableArea(SLINGO.$('blastItem_6'), SLINGO.$('purchasePowerupTutorial'));
					steps.push(step);
					break;
			}

			if(steps.length){
				/* Set up the tutorial object. */
				var puTute = new SLINGO.Tutorial();
				puTute.setScrimParent(puScreen.getElement());
				puTute.addWatch('HIDE', bind(this, this.onTutorialEnd));

				/* Add the tutorial steps. */
				for(var i = 0; i < steps.length; i++)
					puTute.addStep(steps[i]);

				/* Cheating a little bit to make sure the clickable areas display above the scrim.
					The scrim and the powerup need to have the same closest positioned ancestor in order for the z-index to work. */
				SLINGO.$('sparkDisplayCont').style.position = 'static';
				SLINGO.$('sparkDisplayWindow').style.position = 'static';

				/* Start the tutorial. */
				puTute.setState('SHOW');
			}
		}

		// change look for tooltips get all the class tooltips
		var toolTips = SLINGO.getElementsByClass('toolTip');
		for(i=0; i<toolTips.length; i++){
			var children =  toolTips[i].childNodes;
			if(toolTips[i].firstChild.className == undefined){
				SLINGO.addClassName(toolTips[i], 'powerupTooltip', false);
			}
		}

		return puScreen.setState('SHOW');
	},

	/* Put the correct positions back on the powerup slider so it can function properly again. */
	onTutorialEnd : function(){

		SLINGO.$('sparkDisplayCont').style.position = 'relative';
		SLINGO.$('sparkDisplayWindow').style.position = 'absolute';
		SLINGO.$('purchasePowerupTutorial').style.display = 'none';
		return true;
	},

	showPrevFeatures : function(e, prevPage){
		if(this.currFeaturePage === 1) return;

		SLINGO.removeClassName(SLINGO.$('pageDot_'+this.currFeaturePage), 'pageDotOn');

		if(typeof prevPage == 'undefined') prevPage = this.currFeaturePage - 1;
		SLINGO.BlastSlider.init_BlastSlider(SLINGO.Event.getEvent(e), this.currFeaturePage, prevPage);

		this.currFeaturePage = prevPage;
		SLINGO.addClassName(SLINGO.$('pageDot_'+this.currFeaturePage), 'pageDotOn');

		/* If we're not on the last page, show the next arrow. */
		var pages = Math.ceil(this.numFeatures / this.featuresPerPage);
		if(this.currFeaturePage < pages)
			SLINGO.$('sparkRightScrollBtn').style.display = 'block';

		/* If we're on the first page, hide the previous arrow. */
		if(this.currFeaturePage === 1)
			SLINGO.$('sparkLeftScrollBtn').style.display = 'none';

		return true;
	},

	showNextFeatures : function(e, nextPage){
		/* If we're on the last page, go no further. */
		var pages = Math.ceil(this.numFeatures / this.featuresPerPage);
		if(this.currFeaturePage === pages) return false;

		/* Remove the highlight from the active page button. */
		SLINGO.removeClassName(SLINGO.$('pageDot_'+this.currFeaturePage), 'pageDotOn');

		/* Move to the next page. */
		var eventTarget = SLINGO.Event.getEventTarget(SLINGO.Event.getEvent(e));
		if(typeof nextPage == 'undefined') nextPage = this.currFeaturePage + 1;

		SLINGO.BlastSlider.init_BlastSlider(SLINGO.Event.getEvent(e), this.currFeaturePage, nextPage);

		/* Increment the page count and highlight the next page button. */
		this.currFeaturePage = nextPage;
		SLINGO.addClassName(SLINGO.$('pageDot_'+this.currFeaturePage), 'pageDotOn');

		/* If we're on the last page, hide the next arrow. */
		if(this.currFeaturePage === pages)
			SLINGO.$('sparkRightScrollBtn').style.display = 'none';

		/* If we're no longer on the first page, show the previous arrow. */
		if(this.currFeaturePage > 1)
			SLINGO.$('sparkLeftScrollBtn').style.display = 'block';

		return true;
	},

	showFeaturePage : function(e){
		var eventTarget = SLINGO.Event.getEventTarget(SLINGO.Event.getEvent(e));
		var pg = parseInt(eventTarget.getAttribute('FEATURES_PAGE'));
		if(!pg)
			return SLINGO.triggerError('BlastStage.showFeaturePage: FEATURES_PAGE attribute not found',SLINGO.EWARN);

		if(pg > this.currFeaturePage){
			//for(var i = this.currFeaturePage+1; i <= pg; i++)
				this.showNextFeatures(e, pg);
		} else if(pg < this.currFeaturePage){
			//for(var i = this.currFeaturePage-1; i >= pg; i--)
				this.showPrevFeatures(e, pg);
		}
		return true;
	},

	toggleHelpScreen : function(){
		if(SLINGO.inArray(this.getChild('HELP').getState(), ['INIT','HIDE'])){
			/*SLINGO.$('closeTutorial').style.display = 'block';*/
			SLINGO.GameComm.setGameVar('pause_game', '1');
			/*this.tuteElement.style.display = 'block';*/
			this.getChild('HELP').setState('SHOW');
			this.getChild('HELP').getElement().style.top = SLINGO.DHTML.getObjectHeight(this.getChild('TOPBAR').getElement())+1+'px';
			return true;
		}

		SLINGO.GameComm.setGameVar('pause_game','0');
		return this.getChild('HELP').setState('HIDE');
	},

	toggleMusic : function(){
		if(this.musicOn){
			this.musicOn = false;
			SLINGO.addClassName(SLINGO.$('toggleMusicBtn'), 'btnOff');
		} else {
			this.musicOn = true;
			SLINGO.removeClassName(SLINGO.$('toggleMusicBtn'), 'btnOff');
		}
		SLINGO.GameComm.toggleGameMusic();
		return true;
	},

	toggleSfx : function(){
		if(this.sfxOn){
			this.sfxOn = false;
			SLINGO.addClassName(SLINGO.$('toggleSfxBtn'), 'btnOff');
		} else {
			this.sfxOn = true;
			SLINGO.removeClassName(SLINGO.$('toggleSfxBtn'), 'btnOff');
		}
		SLINGO.GameComm.toggleGameSfx();
		return true;
	},

	showUnlockScreen : function(score){
		if(!this.unlockedFeature) 								return SLINGO.triggerError('BlastStage.showUnlockScreen: no new unlocked feature',SLINGO.EWARN);
		if(!this.featureInfo[this.unlockedFeature])				return SLINGO.triggerError('BlastStage.showUnlockScreen: invalid feature: '+this.unlockedFeature);
		if(this.featureInfo[this.unlockedFeature].displayed)	return;

		var f = this.featureInfo[this.unlockedFeature];
		if(SLINGO.$('unlockedImage'))	SLINGO.$('unlockedImage').innerHTML = f.icon ? '<img src="/images/common/blast/'+f.icon+'_lrg.png">' : '';
		if(SLINGO.$('unlockedName'))	SLINGO.$('unlockedName').innerHTML	= f.name ? f.name : '';
		if(SLINGO.$('unlockedDesc'))	SLINGO.$('unlockedDesc').innerHTML	= f.long_desc ? f.long_desc : '';

		this.refreshTopGameBar();
		this.featureInfo[this.unlockedFeature].displayed = true;
		this.unlockedFeature = false;
		this.unlockedNextFeature = false;
		this.unlockedPrevFeature = false;

		SLINGO.$('unlockedOkBtn').onclick = null;
		SLINGO.$('unlockedOkBtn').onclick = bindWithArgs(this, this.showSummaryScreen, [score])
		return this._game_area_mgr.showStage('UNLOCKED');
	},

	showSummaryScreen : function(score){
		if(this.updateTopScore(score))
			SLINGO.addClassName(SLINGO.$('gameSummary'), 'gameSummaryHighScore', true);
		else SLINGO.removeClassName(SLINGO.$('gameSummary'), 'gameSummaryHighScore');

		SLINGO.$('finalScore').innerHTML 		= parseInt(score) ? SLINGO.fmtNumber(score) : SLINGO.fmtNumber(SLINGO.Player.lastScore);
		SLINGO.$('endCoins').innerHTML			= SLINGO.Game.coinConv ? SLINGO.fmtNumber(SLINGO.Game.coinConv * score) : SLINGO.ftmNumber(score);
		SLINGO.$('endGamesPlayed').innerHTML	= SLINGO.fmtNumber(SLINGO.Game.blastGamesLife);

		var blocks = SLINGO.GameComm.getGameVar('total_detonations');
		SLINGO.$('endBlocks').innerHTML			= parseInt(this.blocksDetonated) ? SLINGO.fmtNumber(this.blocksDetonated) : 0;
		this.blocksDetonated = 0;

		if(this.featuresForSummary && (typeof this.featuresForSummary === 'object') && this.featuresForSummary.length){
			var imgs = '';
			for(var i = 0; i < this.featuresForSummary.length; i++)
				imgs += '<img src="/images/common/blast/'+this.featureInfo[this.featuresForSummary[i]].icon+'_sml.png">';
			this.featuresForSummary = [];
			SLINGO.$('endPowerupsTitle').style.display = 'block';
			SLINGO.$('endPowerupsIcons').style.display = 'block';
			SLINGO.$('endPowerupsIcons').innerHTML = imgs;
		} else {
			SLINGO.$('endPowerupsTitle').style.display = 'none';
			SLINGO.$('endPowerupsIcons').style.display = 'none';
		}

		return this.showGameAreaStage('SUMMARY');
	},

	showGameAreaStage : function(refId){
		if(!this._game_area_mgr.isChild(refId))
			return SLINGO.triggerError('BlastStage.showGameArea: stage does not exist: '+refId,SLINGO.EWARN);

		this.currGameAreaStage = refId;
		this._game_area_mgr.showStage(refId);
		return true;
	},

	_showEndGameOverlay : function(score){
		this.score = score;

		/*this.getChild('POWERUPS').setState('HIDE');*/
		return this.unlockedFeature
			? this.showUnlockScreen(score)
			: this.showSummaryScreen(score);
	},

	_catchPostScore : function(retv){
		this._super(retv);	
		this.processServerUpdates(retv);
	},

	processServerUpdates : function(retv, keepDisplay){
		/* Check the list of unlocked features in case a feature was unlocked in another window. */
		if(retv.unlocked && (typeof retv.unlocked === 'object')){
			var u = retv.unlocked;
			var newUnlocks = 0;
			var reInit = false;
			for(var i = 0; i < u.length; i++)
				if(this.featureInfo[u[i]] && !this.featureInfo[u[i]].unlocked){
					++newUnlocks;
					this.featureInfo[u[i]].unlocked = 1;
					if(!keepDisplay) this.featureInfo[u[i]].displayed = 1;

					if(this.featureInfo[u[i]].type.toUpperCase() === 'SLOT'){
						/*++this.maxFeatures;*/
						reInit = true;
						//SLINGO.$('blastItem_'+i).parentNode.removeChild(SLINGO.$('blastItem_'+i));
					}
				}
			if(reInit) this.initPowerupScreen();
		}

		var newGames = retv.games_played > SLINGO.Game.blastGamesLife;

		/* If the user is playing in another window, update any changes to their information on this page. */
		if(retv.bux) this.updatePlayerBux(retv.bux);
		if(retv.top_score) this.updateTopScore(retv.top_score);
		if(newGames) this.updateGamesPlayed(retv.games_played);
		if(newGames || newUnlocks) this.refreshGameElements();
		return true;
	},

	updateGamesPlayed : function(num){
		if(!num) return SLINGO.triggerError('BlastStage.updateGamesPlayed: number not provided',SLINGO.EWARN);

		SLINGO.Game.blastGamesLife = num;
		if(SLINGO.$('totalGamesPlayed'))
			SLINGO.$('totalGamesPlayed').innerHTML = SLINGO.fmtNumber(num);
	},

	updateTopScore : function(score){
		if(parseInt(score) > SLINGO.Game.blastTopScore){
			SLINGO.Game.blastTopScore = score;
			return true;
		}
		return false;
	},

	updatePlayerBux : function(bux){
		if(!bux) return SLINGO.triggerError('BlastStage.updateGamesPlayed: bux not provided',SLINGO.EWARN);
		SLINGO.Player.playerBux = bux;
		SLINGO.UserTools.updateBuxView(bux.toString());
	},

	setLastSparkConfig : function(config){
		this.lastSparkConfig = config;
	},

	setNewSparkConfig : function(config){
		this.newSparkConfig = config;
	},

	getAllFeatures : function(){
		return (this.featureInfo && (typeof this.featureInfo == 'object'))
			? this.featureInfo
			: SLINGO.triggerError('BlastStage.getAllFeatures: no features found',SLINGO.EWARN);
	},

	getFeature : function(id){
		return (this.featureInfo && (id in this.featureInfo))
			? this.featureInfo[id]
			: SLINGO.triggerError('BlastStage.getFeature: feature does not exist: '+id,SLINGO.EWARN);
	},

	isFeatureUnlocked : function(id){
		if(!this.featureInfo[id])
			return SLINGO.triggerError('SLINGO.BlastStage.isFeatureUnlocked: feature does not exist: '+id,SLINGO.EWARN);

		return this.featureInfo[id].unlocked;
	},

	isFeatureNewlyUnlocked : function(id){
		if(!this.featureInfo[id])
			return SLINGO.triggerError('BlastStage.isFeatureNewlyUnlocked: feature does not exist: '+id,SLINGO.EWARN);

		return (this.featureInfo[id].games_to_unlock === SLINGO.Game.blastGamesLife) && !this.featureInfo[id].displayed;
	},

	canAffordFeature : function(id){
		if(!this.featureInfo[id])
			return SLINGO.triggerError('SLINGO.BlastStage.canAffordFeature: feature does not exist: '+id,SLINGO.EWARN);

		return SLINGO.Player.getPlayerBux() >= this.featureInfo[id].unlock_cost;
	},

	loadLastConfig : function(){
		if(!this.lastSparkConfig) return;

		var config = this.lastSparkConfig.split(',');
		if(!config) return;

		/* if the user previously had this selected, add it to the preload list */
		for(var i in this.featureInfo)
			if(SLINGO.inArray(this.featureInfo[i].swf, config) && SLINGO.$('blastItem_'+i))
				this.toggleFeature(i);
	},

	refreshGameElements : function(){
		this.handleGameUnlocks();
		/*this.updateFeatureEnable();*/
		this.refreshTopGameBar();
		return true;
	},

	updateFeatureEnable : function(){
		/* Get ready to load previous config if there is one */
		var toLoad = [];
		if(this.lastSparkConfig)
			var config = this.lastSparkConfig.split(',');
		if(!config) var config = [];

		/* Enable/disable a feature based on what the user is eligible for */
		for(var i in this.featureInfo){
			/* if the user previously had this selected, add it to the preload list */
			if(SLINGO.inArray(this.featureInfo[i].swf, config) && SLINGO.$('blastItem_'+i))
				toLoad.push(i);

			this.handleFeatureEnable(i);

			switch(this.featureInfo[i].type){
				case 'SLOT':
					this.handleSlotEnable(i);
					break;
			}
		}

		/* Load previous config items if there are any */
		if(toLoad.length)
			for(var i = 0; i < toLoad.length; i++)
				this.toggleFeature(toLoad[i]);

		return true;
	},

	setFeatureUnlockOrder : function(){
		this.featureUnlockOrder = new Array();
		for(var i in this.featureInfo){
			if(!this.featureUnlockOrder.length)
				this.featureUnlockOrder.push(i);
			else {
				var added = false;
				var old = this.featureUnlockOrder;
				for(var j = 0; j < old.length; j++){
					if(this.featureInfo[i].games_to_unlock < this.featureInfo[old[j]].games_to_unlock){
						this.featureUnlockOrder.splice(j, 0, i);
						added = true;
						break;
					}
				}
				if(!added) this.featureUnlockOrder.push(i);
			}
		}
		return true;
	},

	initPowerupScreen : function(){
		SLINGO.$('powerupPageBtns').innerHTML = '';
		SLINGO.$('sparkDisplayWindow').innerHTML = '';
		SLINGO.$('sparkLeftScrollBtn').style.display = 'none';
		SLINGO.$('sparkRightScrollBtn').style.display = 'block';
		this.maxFeatures = 0;
		this.numFeatures = 0;
		this.currFeaturePage = 1;
		if(SLINGO.isPluginLoaded('BlastSlider')) SLINGO.BlastSlider.resetSlider();

		var pageNum = 1;
		var perPage = 0;
		for(var k = 0; k < this.featureUnlockOrder.length; k++){
			var i = this.featureUnlockOrder[k];
			if(this.featureInfo[i].unlocked && (i !== this.unlockedNextFeature)) this.featureInfo[i].displayed = 1;

			/* Create an HTML element for each feature. */
			var typ = this.featureInfo[i].type.toUpperCase();
			var fn = 'initElement_'+typ;

			/* Create the "page" container if it doesn't exist already. */
			if(!SLINGO.$('sparkDisplay_'+pageNum)){
				var d = document.createElement('div');
				d.id = 'sparkDisplay_'+pageNum;
				d.className = 'sparkDisplay';
				d.style.cssFloat = 'left';
				SLINGO.$('sparkDisplayWindow').appendChild(d);

				var a = document.createElement('a');
				a.id = 'pageDot_'+pageNum;
				a.className = 'pageDot'+(this.currFeaturePage === pageNum ? ' pageDotOn' : '');
				a.setAttribute('FEATURES_PAGE', pageNum);
				a.onclick = bind(this, this.showFeaturePage);
				SLINGO.$('powerupPageBtns').appendChild(a);
			}

			/* If the powerup is successfully created, add it to the "page". */
			if(this.initElement_POWERUP(i, pageNum)){
				++perPage;

				/* If this page has already reached its max, we have to move onto the next one. */
				if(perPage >= this.featuresPerPage){
					++pageNum;
					perPage = 0;
				}
			}
			if((typ != 'POWERUP') && (fn in this))
				this[fn](i);
		}

		SLINGO.$('sparkDisplayWindow').style.width = (pageNum * 394)+'px';
		SLINGO.$('sparkDisplayWindow').style.left = '0px';
		return true;
	},

	initElement_POWERUP : function(i, pg){
		if(!this.featureInfo[i]) return SLINGO.triggerError('BlastStage.initElement_POWERUP: invalid feature: '+i);

		/* Don't show powerup slots that are already unlocked. */
		if((this.featureInfo[i].type.toUpperCase() === 'SLOT') && this.featureInfo[i].unlocked)
			return false;

		++this.numFeatures;

		var item = document.createElement('div');
		item.className = 'blastItem toolTipAnchor';
		item.id = 'blastItem_'+i;
		item.setAttribute('tiptext', this.featureInfo[i].desc);

		var checkMark = document.createElement('div');
		checkMark.id = 'checkMark';
		checkMark.innerHTML = '&nbsp;';
		item.appendChild(checkMark);

		var itemName = document.createElement('div');
		itemName.className = 'itemName';
		itemName.innerHTML = '&nbsp;';/*this.featureInfo[i].name;*/
		item.appendChild(itemName);

		var itemNameLocked = document.createElement('div');
		itemNameLocked.className = 'itemNameLocked';
		itemNameLocked.innerHTML = 'Unlock at<br>'+SLINGO.fmtNumber(this.featureInfo[i].games_to_unlock)+' Games';
		item.appendChild(itemNameLocked);

		var imageCont = document.createElement('div');
		imageCont.className = 'itemImage';
		item.appendChild(imageCont);

			var image = document.createElement('img');
			image.src = this.featureInfo[i].icon ? '/images/common/blast/'+this.featureInfo[i].icon+'.png' : '/images/admin/blank.gif';
			imageCont.appendChild(image);

		var imageContLocked = document.createElement('div');
		imageContLocked.className = 'itemImageLocked';
		item.appendChild(imageContLocked);

			var image = document.createElement('img');
			image.src = this.featureInfo[i].icon ? '/images/common/blast/'+this.featureInfo[i].icon+'_locked.png' : '/images/admin/blank.gif';
			imageContLocked.appendChild(image);

		/*var itemPrice = document.createElement('div');
		itemPrice.className = 'itemPrice';
		itemPrice.innerHTML = 'Own';
		item.appendChild(itemPrice);*/

		var itemPriceLocked = document.createElement('div');
		itemPriceLocked.className = 'itemPriceLocked';
		item.appendChild(itemPriceLocked);
			/*
			var ctaBtn = document.createElement('a');
			ctaBtn.className = 'ctaBtn ctaBtnOrange';

				var ctaBtnLeft = document.createElement('span');
				ctaBtnLeft.className = 'ctaBtnLeft';
				ctaBtn.appendChild(ctaBtnLeft);

				var ctaBtnMiddle = document.createElement('span');
				ctaBtnMiddle.className = 'ctaBtnMiddle';
				ctaBtnMiddle.innerHTML = 'Buy '+SLINGO.fmtNumber(this.featureInfo[i].unlock_cost)+' <img src="/images/common/blast/ico_buxmini.png" align="absmiddle" alt="Slingo Bux">';
				ctaBtn.appendChild(ctaBtnMiddle);

				var ctaBtnRight = document.createElement('span');
				ctaBtnRight.className = 'ctaBtnRight';
				ctaBtn.appendChild(ctaBtnRight);

			itemPriceLocked.appendChild(ctaBtn);
			*/
		SLINGO.$('sparkDisplay_'+pg).appendChild(item);
		
		this.handleFeatureEnable(i);
		
		item.onclick = bindWithArgs(this, this.toggleFeature, [i]);
		

		if(SLINGO.isPluginLoaded('ToolTip')) SLINGO.ToolTip.checkForTip(item);
		return true;
	},

	initElement_SLOT : function(i){
		if(!this.featureInfo[i]) return SLINGO.triggerError('BlastStage.initElement_SLOT: invalid feature: '+i);

		var obj = {id : i, filled : false};
		if(!this.powerupSlotOrder.length)
			this.powerupSlotOrder[1] = obj;
		else this.powerupSlotOrder.push(obj);

		/* Create the slot display. */
		var isUnlocked = this.featureInfo[i].unlocked;
		if(isUnlocked) ++this.maxFeatures;

		var slot = document.createElement('div');
		slot.id = 'slotItem_'+i;
		slot.className = 'slotItem'+(!isUnlocked ? ' slotItemLocked' : '');

			var slotLocked = document.createElement('div');
			slotLocked.className = 'slotLocked';
			slotLocked.innerHTML = 'Unlock at<br>'+this.featureInfo[i].games_to_unlock+' Games';
			slot.appendChild(slotLocked);

			var slotImage = document.createElement('div');
			slotImage.id = 'slotInfo_'+i;
			slotImage.className = 'slotInfo';
			slot.appendChild(slotImage);

			/*  
			var slotPrice = document.createElement('div');
			slotPrice.className = 'slotPrice';
			//slotPrice.innerHTML = 'or <img src="/images/common/blast/ico_buxmini.png" alt="Slingo Bux" align="absmiddle"> '+this.featureInfo[i].unlock_cost;
			slot.appendChild(slotPrice);

				var ctaBtn = document.createElement('a');
				ctaBtn.className = 'ctaBtn ctaBtnOrange';

					var ctaBtnLeft = document.createElement('span');
					ctaBtnLeft.className = 'ctaBtnLeft';
					ctaBtn.appendChild(ctaBtnLeft);

					var ctaBtnMiddle = document.createElement('span');
					ctaBtnMiddle.className = 'ctaBtnMiddle';
					ctaBtnMiddle.innerHTML = SLINGO.fmtNumber(this.featureInfo[i].unlock_cost)+' <img src="/images/common/blast/ico_buxmini.png" align="absmiddle" alt="Slingo Bux">';
					ctaBtn.appendChild(ctaBtnMiddle);

					var ctaBtnRight = document.createElement('span');
					ctaBtnRight.className = 'ctaBtnRight';
					ctaBtn.appendChild(ctaBtnRight);

				slotPrice.appendChild(ctaBtn);

		if(!isUnlocked)
			slot.onclick = bindWithArgs(this, this.unlockFeature, [i]);
		*/
			SLINGO.$('chamberHolder').appendChild(slot);

		return true;
	},

	toggleFeature : function(id){
		var pstate = this.getChild('POWERUPS').getState();
		/* Don't allow this if the powerup screen is minimized. */
		if(pstate === 'MIN')
			return;
		
		if((pstate === 'SHOW') && !this.isFeatureUnlocked(id))
			return this.unlockFeature(id);
		
		if(!parseInt(id))
			return SLINGO.triggerError('SLINGO.BlastStage.toggleFeature: invalid id: '+id,SLINGO.EWARN);

		if(!SLINGO.$('blastItem_'+id))
			return SLINGO.triggerError('SLINGO.BlastStage.toggleFeature: invalid node: '+id,SLINGO.EWARN);
		var item = SLINGO.$('blastItem_'+id);

		/* if this spark is already selected, remove it */
		if(SLINGO.inArray(id, this.selectedFeatures)){
			/* Empty the powerup slot. */
			var slotPos = this.featureInfo[id].slot;
			var slotId = this.powerupSlotOrder[slotPos].id;
			SLINGO.removeClassName(SLINGO.$('slotItem_'+slotId), 'slotItemActive');
			SLINGO.$('slotItem_'+slotId).onclick = null;
			SLINGO.$('slotInfo_'+slotId).innerHTML = '';
			this.featureInfo[id].slot = false;
			this.powerupSlotOrder[slotPos].filled = false;

			/* Set the feature back to unselected in the store. */
			SLINGO.removeClassName(item, 'blastItemActive');

			/* remove the spark id from the selected sparks array */
			var s = [];
			for(var i = 0; i < this.selectedFeatures.length; i++)
				if(this.selectedFeatures[i] != id)
					s.push(this.selectedFeatures[i]);
			this.selectedFeatures = s;

		/* otherwise add it */
		} else {
			if(this.selectedFeatures.length >= this.maxFeatures){
				alert('You can only select '+this.maxFeatures+' powerup'+(this.maxFeatures > 1 ? 's' : '')+' at a time. Click on a powerup slot to unlock the ability to select more powerups at a time!');
				return;
			}

			/* Determine the next available powerup slot. */
			var slotId = false;
			var slotPos = false;
			for(var i = 1; i <= this.powerupSlotOrder.length; i++)
				if(!this.powerupSlotOrder[i].filled){
					slotPos = i;
					slotId = this.powerupSlotOrder[i].id;
					break;
				}
			if(!slotId) return SLINGO.triggerError('BlastStage.toggleFeature: could not find an available slot',SLINGO.EWARN);

			/* Add the feature to the next available powerup slot. */
			SLINGO.addClassName(SLINGO.$('slotItem_'+slotId), 'slotItemActive');
			SLINGO.$('slotItem_'+slotId).onclick = bindWithArgs(this, this.toggleFeature, [id]);
			SLINGO.$('slotInfo_'+slotId).innerHTML = '<img src="/images/common/blast/'+this.featureInfo[id].icon+'_sml.png">';
			this.featureInfo[id].slot = slotPos;
			this.powerupSlotOrder[slotPos].filled = true;

			/* Set the feature to selected in the store. */
			SLINGO.addClassName(item, 'blastItemActive');

			this.selectedFeatures.push(id);
		}

		/* disable any features they can/can't afford now that they've removed/added a feature */
//		for(var i in this.featureInfo)
//			if(id !== i)
//				this.handleFeatureEnable(i);
	},

	unlockFeature : function(id){
		var pstate = this.getChild('POWERUPS').getState();
		/* Don't allow this if the powerup screen is minimized. */
		if(pstate === 'MIN')
			return;

		if(!this.featureInfo[id])
			return SLINGO.triggerError('BlastStage.unlockFeature: feature does not exist: '+id,SLINGO.EWARN);

		if(this.isFeatureUnlocked(id))
			return alert('You have already unlocked '+this.featureInfo[id].name+'.');

		return alert('This feature is locked!');
		
		/*
		if(!this.canAffordFeature(id))
			return alert('You do not have enough Slingo Bux to purchase '+this.featureInfo[id].name+'.');

		var ok = confirm('Are you sure you want to unlock '+this.featureInfo[id].name+' for '+this.featureInfo[id].unlock_cost+' Slingo Bux?');
		if(!ok) return;

		if(!SLINGO.isPluginLoaded('Overlay'))
			SLINGO.triggerError('AdvantageStage.catchBuyClick: overlay plugin not loaded',SLINGO.EWARN);
		else {
			SLINGO.Overlay.createOverlay(
				'blast_puloading',
				{
					cid		: this._game_area_mgr.getChild('GAME').id,
					content	: '<center><h2>Processing your power up purchase.</h2><br><img src="/images/common/loading100x100.gif"></center><br><br><h4>Please do not close your browser or navigate away from this page until complete.</h4>',
					vcenter	: true
				}
			);
		}

		var d = new Date();
		var post = {feature_id : id};
		if(!SLINGO.XHR.createRequest('unlockFeature'+d.getTime(), '/ajax/sgs/blast_unlock/'+d.getTime(), 'POST', 'JSON', true, post))
			return SLINGO.triggerError('SLINGO.BlastStage.postSparkConfig: unable to create a valid XHR request.', SLINGO.EWARN);

		if(!SLINGO.XHR.activateRequest('unlockFeature'+d.getTime(), bind(this, this.catchUnlockFeature)))
			return SLINGO.triggerError('SLINGO.BlastStage.postSparkConfig: failed to activate the xhr request.', SLINGO.EWARN);
		*/
		return true;
	},

	catchUnlockFeature : function(retv){
		if(retv.not_member)
			return alert('You are not logged in. Please log in to Slingo.com and try again.');

		if(retv.error)
			return alert(retv.error);

		this.processServerUpdates(retv);

		SLINGO.Overlay.close({id:'blast_puloading',type:'remove'});
		if(retv.msg) alert(retv.msg);
		return true;
	},

	handleFeatureEnable : function(id){
		if(SLINGO.$('blastItem_'+id)){
			var bi = SLINGO.$('blastItem_'+id);
			if(this.isFeatureUnlocked(id))
				SLINGO.removeClassName(bi, 'blastItemLocked');
			else SLINGO.addClassName(bi, 'blastItemLocked', true);
		}
	},

	handleSlotEnable : function(id){
		if(SLINGO.$('slotItem_'+id)){
			var si = SLINGO.$('slotItem_'+id);
			if(this.isFeatureUnlocked(id)){
				if(SLINGO.hasClassName(si, 'slotItemLocked')){
					SLINGO.removeClassName(si, 'slotItemLocked');
					si.onclick = null;
				}
			} else SLINGO.addClassName(si, 'slotItemLocked', true);
		}
	},

	handleGameUnlocks : function(){
		var games = SLINGO.Game.blastGamesLife;
		var origNext = this.unlockedFeatureNext;

		this.unlockedFeature = false;
		this.unlockedNextFeature = false;
		this.unlockedPrevFeature = false;

		/* Enable any powerups they've unlocked by playing a certain number of games. */
		for(var i in this.featureInfo){
			/* Determine if this feature has just become unlocked this game. */
			if(this.isFeatureNewlyUnlocked(i)){
				this.unlockedFeature = i;
				this.featureInfo[i].unlocked = 1;

			/* Determine if this feature has become unlocked due to the number of games increasing. */
			} else if(this.featureInfo[i].games_to_unlock < games){
				this.featureInfo[i].unlocked = 1;
				this.featureInfo[i].displayed = 1;

			/* Determine what the next feature to unlock is. */
			} else if(!this.isFeatureUnlocked(i)){
				if(!this.unlockedNextFeature)
					this.unlockedNextFeature = i;
				else if(this.featureInfo[this.unlockedNextFeature].games_to_unlock > this.featureInfo[i].games_to_unlock)
					this.unlockedNextFeature = i;
			}

			/* Determine if this feature was the last one to be unlocked by reaching x number of games. */
			if(this.isFeatureUnlocked(i) && (this.featureInfo[i].games_to_unlock < games)){
				if(!this.unlockedPrevFeature)
					this.unlockedPrevFeature = i;
				else if(this.featureInfo[this.unlockedPrevFeature].games_to_unlock < this.featureInfo[i].games_to_unlock)
					this.unlockedPrevFeature = i;
			}

			/* Regardless of when/how it was unlocked, we want to make sure the display is correct. */
			if(this.isFeatureUnlocked(i)){
				this.handleFeatureEnable(i);
				switch(this.featureInfo[i].type){
					case 'SLOT':
						this.handleSlotEnable(i);
						break;
				}
			}
		}

		/* If we're on the last unlockable feature. */
//		if(this.isFeatureUnlocked(this.unlockedNextFeature))
//			this.unlockedNextFeature = false;

		SLINGO.triggerError('BlastStage.handleGameUnlocks: blastGamesLife: '+games+', unlockedFeature: '+this.unlockedFeature+', unlockedNextFeature: '+this.unlockedNextFeature+', unlockedPrevFeature: '+this.unlockedPrevFeature,SLINGO.EWARN);
		return true;
	},

	refreshTopGameBar : function(){
		SLINGO.triggerError('BlastStage.refreshTopGameBar',SLINGO.EWARN);
		if(SLINGO.$('totalGamesPlayed'))
			SLINGO.$('totalGamesPlayed').innerHTML = SLINGO.fmtNumber(SLINGO.Game.blastGamesLife);

		if(!SLINGO.$('nextUnlocked'))		return SLINGO.triggerError('BlastStage.refreshTopGameBar: nextUnlocked not found',SLINGO.EWARN);
		if(!SLINGO.$('pubarFill'))			return SLINGO.triggerError('BlastStage.refreshTopGameBar: pubarFill not found',SLINGO.EWARN);

		var prev = this.unlockedPrevFeature ? this.getFeature(this.unlockedPrevFeature) : false;
		var next = this.unlockedNextFeature ? this.getFeature(this.unlockedNextFeature) : false;
		SLINGO.$('nextUnlocked').innerHTML 		= (next && next.name) ? next.name : 'none';
		SLINGO.$('pubarIcon').innerHTML			= (next && next.icon) ? '<img src="/images/common/blast/'+next.icon+'_mini.png">' : '';
		SLINGO.$('pubarFill').style.width		= (next && prev)
													? ((SLINGO.Game.blastGamesLife / next.games_to_unlock) * 100 +'%') //((((SLINGO.Game.blastGamesLife - prev.games_to_unlock) / (next.games_to_unlock - prev.games_to_unlock)) * 100)+'%')
													: (prev ? '100%' : '1%');
		if(SLINGO.$('pubarFill').style.width == '0%') SLINGO.$('pubarFill').style.width = '1%';
		return true;
	},

	/* Record the users last spark configuration */
	postSparkConfig : function(){
		if(this.newSparkConfig.toLowerCase() != this.lastSparkConfig.toLowerCase()){
			var d = new Date();
			var post = {
				'spark_config' 	: this.newSparkConfig,
				'token' 		: SLINGO.Player.getPlayerToken(),
				'game_id'		: SLINGO.Game.gameId,
				'room_id'		: SLINGO.Game.roomId
			};

			var reqh = this.postSparkConfigURL;
			if(!SLINGO.XHR.createRequest('postSparks', reqh+d.getTime(), 'POST', 'JSON', true, post))
				return SLINGO.triggerError('SLINGO.BlastStage.postSparkConfig: unable to create a valid XHR request.', SLINGO.EWARN);

			if(!SLINGO.XHR.activateRequest('postSparks', bind(this, this.catchPostSparkConfig)))
				return SLINGO.triggerError('SLINGO.BlastStage.postSparkConfig: failed to activate the xhr request.', SLINGO.EWARN);

		}
	},

	/* Expects retv.msg (on error), retv.coins, retv.token */
	catchPostSparkConfig : function(retv){
		if(retv.msg){
			/*if(retv.diff_msg){
				alert('Another game session is already in progress. Please be sure that you only have one game window open at a time.');
				return this._sendToLobby();
			}*/

			/* GP Edit - added token values to report. */
			if(!retv.no_report){
				var emsg = 'GEN:JS-GENERATEDTICKET,TYPE:BLASTCONFIG,ERROR:'+retv.msg;
				if(retv.ptok) emsg += ',pgtok:'+retv.ptok;
				if(retv.stok) emsg += ',shtok:'+retv.stok;
				if(retv.ssid) emsg += ',ssid:'+retv.ssid;
				if(retv.misc) emsg += ','+retv.misc;

				return this.relay_reportError(emsg);
			}

			SLINGO.triggerError('SLINGO.BlastStage.catchPostSparkConfig: '+retv.msg);
		}else
			/* The post has finished so set the new to the last for next time */
			this.lastSparkConfig = this.newSparkConfig;
	},

	/* Get the last BATCH_EVENT sent from the Platform Adapter, extract what we need to display, and log the rest. */
	processBatchEvent : function(score){
		var msg = SLINGO.GameComm.getPlatformMsgs('BATCH_EVENT');
		if(!msg) return SLINGO.triggerError('BlastStage.processBatchEvent: BATCH_EVENT array not found',SLINGO.EWARN);

		var msg = msg.pop();
		if(!msg) return SLINGO.triggerError('BlastStage.processBatchEvent: BATCH_EVENT array is empty',SLINGO.EWARN);

		var info = eval('('+msg+')');
		if(typeof info !== 'object') return SLINGO.triggerError('BlastStage.processBatchEvent: info object not found: '+(typeof info),SLINGO.EWARN);

		// Pull out the number of detonated blocks for display on the game summary screen.
		if(('allData' in info) && ('TD' in info.allData)) this.blocksDetonated = info.allData.TD;
		else SLINGO.triggerError('BlastStage.processBatchEvent: total detonations not found',SLINGO.EWARN);

		return this.postEndGame(info, score);
	},

	/* Log the game data */
	postEndGame : function(data, score){
		var d = new Date();
		var logData = '';
		for(var i in data.allData){
			if(typeof data.allData[i] == 'object'){
				for(var j in data.allData[i]){
					logData = logData+''+j+'='+data.allData[i][j]+'&';
				}
			}else{
				logData = logData+''+i+'='+data.allData[i]+'&';
			}
		}

		logData = logData.substring(0,(logData.length - 1));
		var post = {
			'token' 	: SLINGO.Player.getPlayerToken(),
			'log_data'	: logData,
			'game_id'	: SLINGO.Game.gameId,
			'room_id'	: SLINGO.Game.roomId,
			'score'		: score,
			'powerups'	: this.newSparkConfig
		};
		
		

		var reqh = this.postEndGameURL;
		if(!SLINGO.XHR.createRequest('postEndGame'+d.getTime(), reqh+d.getTime(), 'POST', 'JSON', true, post))
			return SLINGO.triggerError('SLINGO.BlastStage.postEndGame: unable to create a valid XHR request.', SLINGO.EWARN);

		if(!SLINGO.XHR.activateRequest('postEndGame'+d.getTime(), bind(this, this.catchPostEndGame)))
			return SLINGO.triggerError('SLINGO.BlastStage.postEndGame: failed to activate the xhr request.', SLINGO.EWARN);
	},

	/* Expects retv.msg (on error), retv.coins, retv.token */
	catchPostEndGame : function(retv){
		if(retv.msg) 		SLINGO.triggerError('BlastStage.catchPostEndGame: '+retv.msg,SLINGO.EWARN);
		if(!retv.logged) 	SLINGO.triggerError('BlastStage.catchPostEndGame: end game was not logged',SLINGO.EWARN);
	}

});

SLINGO.DavidHoytStage = SLINGO.AltGenDailyChallengeStage.extend({
	startGamePoll : false,
	
	_obj_class_name : 'DavidHoytStage',
	
	setGameGoal : function(){
		var curPrefix = this.getCurrentChallengePrefix();		
		var goal;
		
		var uts = parseInt(SLINGO.GamePage.requestInfo.pageReqTime,10) * 1000;
		var ld = new Date(uts);
		var utc = ld.getTime() + (ld.getTimezoneOffset() * 60000);
		var d = new Date(utc + (3600000*parseInt(SLINGO.GamePage.requestInfo.timezoneOffset,10)));
		
		/*var uts = parseInt(SLINGO.GamePage.requestInfo.pageReqTime,10) * 1000;
		var offset = parseInt(SLINGO.GamePage.requestInfo.timezoneOffset,10) * 1000;
		var d = new Date(uts + offset);*/
		if (curPrefix.match(/DevilDC$/)){
			goal = ((d.getFullYear()-1) % 100) * 10000 + (d.getMonth()+1)*100 + d.getDate(); 
		}else if (curPrefix.match(/CherubDC$/)){
			goal = ((d.getFullYear()-2) % 100) * 10000 + (d.getMonth()+1)*100 + d.getDate();
		}else{
			goal = (d.getFullYear() % 100) * 10000 + (d.getMonth()+1)*100 + d.getDate();
		}
		
		//goal = '140901';
		
		SLINGO.triggerError('DavidHoytStage.setGameGoal: ' + goal);
		SLINGO.GameComm.setGameVar('challenge_target', goal);
	},
	
	setChallengeGoal : function (){
		// david hoyt games don't have any goal text
		return;
	},
	
	updateChallengeGoal : function(challengePrefix, challengeObject){
		this._super(challengePrefix, challengeObject);
		var prefix = challengeObject.challengePrefixForGame;
		if(prefix == 'Base'){
			prefix = challengePrefix;
		}
		var goalId =  prefix + 'Goal';
		if(SLINGO.$(goalId)){
			if(challengeObject.hasBeaten){
				SLINGO.$(goalId).innerHTML = 'DEFEATED!!';
			}
		}
	},	
	
	getPercent : function(score){
		return Math.max(1 - Math.max(0, score - this.minScore) / (this.maxScore - this.minScore), 0);
	},
	
	getCoinAward : function(score, isWin){
		var bonus = 0;
		
		if(this.currentChallengeObject && this.currentChallengeObject.isFirstWin){
		    bonus = Math.floor(this.minCoinBonus + (this.maxCoinBonus - this.minCoinBonus) * this.getPercent(score));

		    if(SLINGO.Game.coinConv){
			    bonus = Math.ceil(bonus * SLINGO.Game.coinConv);
		    }
		}
		
		return this._super(score, isWin) + bonus;
	},
	
	getRankAward : function(score, isWin){
		var bonus = 0;
		
		if(this.currentChallengeObject && this.currentChallengeObject.isFirstWin){
		    var bonus = Math.floor(this.minRankBonus + (this.maxRankBonus - this.minRankBonus) * this.getPercent(score));

		    if(SLINGO.Game.rankMultiplier){
			    bonus = Math.ceil(bonus * SLINGO.Game.rankMultiplier);
		    }
		}
		
		return this._super(score, isWin) + bonus;
	},
	
	_postScore : function(score){		
		this.postScoreArgs['wlv'] =  SLINGO.GameComm.getGameVar('game_won');
		return this._super(score);
	}
});

SLINGO.InGameLeaderboardElement = {
	_lbList			:	false,
	_currUser		:	false,
	_friendNumPage	:	false,
	_friendNumRec	:	false,
	_reqListType	:	[],
	//_globalNumPage	:	false,
	getLeaderboard : function(objs){
		// This requests the leaderboard and then sends it to the game
		SLINGO.triggerError('SLINGO.InGameLeaderboardElement.getLeaderboard',SLINGO.EWARN);
		this._updateLeaderboard(objs);
	},
            
    getFBLeaderboard : function(textObj){
		SLINGO.triggerError('SLINGO.InGameLeaderboardElement.getFBLeaderboard: textObj sent: is '+textObj);
		try{
			var textObj = SLINGO.JSON.parse(textObj);
			if (typeof textObj != 'object'){
				throw 'textObj sent is an object';
			}
		}catch(e){
			SLINGO.triggerError('SLINGO.InGameLeaderboardElement.getFBLeaderboard: invalid textObj sent: '+textObj,SLINGO.EWARN);
			return ;			
		}		
		
		//set page numbers requested
		//get objects requested...
		var objectsRequested = [];
		for(var i=0; i < textObj.query.length; i++){
			switch(textObj.query[i].type){
				case 'friend_board': 
					this._friendNumPage = textObj.query[i].pos;
					this._friendNumRec = textObj.query[i].num;
					this._reqListType.push(textObj.query[i].type);
					break;
				//case 'global_board': this._globalNumPage = textObj.query[i].pos;  break;
				default: this._reqListType.push(textObj.query[i].type);//nothing
			}
		}
		
		if(!this._lbList || !this._currUser){
			if(typeof SLINGO.Facebook == 'object'){
				SLINGO.Facebook.fbGetAppScores(function(response){ SLINGO.InGameLeaderboardElement.updateFBLeaderboardCallback(response)} );
			}else{
			   SLINGO.triggerError('SLINGO.InGameLeaderboardElement.getFBLeaderboard:  FACEBOOK OBJECT DOES NOT EXIST',SLINGO.EWARN); 
			}
		}else{
			//get the num and position requested
			lbData = {};
			if(this._friendNumPage){
				//page requested for friends...
				if(!(this._friendNumPage in this._lbList)){
					console.log('SLINGO.InGameLeaderboardElement.getFBLeaderboard:  page #' + this._friendNumPage + ' # requested not found in list: ',  this._lbList); 
					if(1 in this._lbList){
						lbData.friend_board = this._lbList[1];
						lbData.friend_board_pos = 1;
						lbData.friend_board_eol = (2 in this._lbList) ? false: true;
					}else lbData.friend_board = {};				
				}else {
					lbData.friend_board = this._lbList[this._friendNumPage];
					lbData.friend_board_pos = this._friendNumPage;
					var nextP = parseInt(lbData.friend_board_pos) +1;
					lbData.friend_board_eol = (nextP in this._lbList) ? false: true;
				}
			}
			
			
			lbData.error=0;
			lbData.type='fb';
			lbData.current_user = this._currUser;
			SLINGO.GameComm.setGameVar('leaderboard', SLINGO.JSON.stringify(lbData));
		}		
    },
	
	_updateLeaderboard  : function(textObj){
		SLINGO.triggerError('GameComm.InGameLeaderboardElement._updateLeaderboard post: ' + SLINGO.JSON.stringify(textObj),SLINGO.EWARN);
		var parsedObj = this.parseReqObj(textObj);
		SLINGO.triggerError('GameComm.InGameLeaderboardElement._updateLeaderboard parsed obj is: ' + SLINGO.JSON.stringify(parsedObj),SLINGO.EWARN);
		if(parsedObj.error){
			SLINGO.triggerError('GameComm.InGameLeaderboardElement._updateLeaderboard error: ' + parsedObj.error,SLINGO.EWARN);
			return;
		}else{
			if(parsedObj.post){
				var d = new Date();
				SLINGO.XHR.createAndSendRequest(
					'relay_leaderboard' + d.getTime(), 
					SLINGO.InGameLeaderboardElement._catchLeaderboardRequest, 
					'/ajax/gameservice/leaderboard/' + d.getTime(), 
					'POST', 
					'JSON',
					true,
					parsedObj.post
				);
			}else{
				SLINGO.triggerError('GameComm.InGameLeaderboardElement._updateLeaderboard error: no parsed post returned',SLINGO.EWARN);
				return;
			}
		}		
	},
	
	_catchLeaderboardRequest : function(retv){
		// change the leaderboard
		// send the new leaderboard to the game
		SLINGO.GameComm.setGameVar('leaderboard', SLINGO.JSON.stringify(retv));
	},
    
    updateFBLeaderboardCallback : function(response){
        SLINGO.triggerError('updateFBLeaderboardCallback: ' +  SLINGO.JSON.stringify(response));
		var lbData = {};
		for(var listType in SLINGO.InGameLeaderboardElement._reqListType){
			if(listType !== 'xfind'){
				if(SLINGO.InGameLeaderboardElement._reqListType[listType] == 'friend_board'){
					if(SLINGO.InGameLeaderboardElement._friendNumRec && SLINGO.InGameLeaderboardElement._friendNumPage){
						//go through object, add profile pic url
						if(response.data){
							var lbPage = [];
							var lbListHolder = {};
							var page = 1;
							var limit = SLINGO.InGameLeaderboardElement._friendNumRec;
							for(var i=0; i < response.data.length; i++){				
								if(i !== 'xfind'){
									if(i == limit){
										lbListHolder[page] = lbPage;
										lbPage = [];
										page++;
										limit+=SLINGO.InGameLeaderboardElement._numRec;
									}
									var player = {};
									player.uid = response.data[i].id;
									player.username = response.data[i].user.first_name + " " + response.data[i].user.last_name;
									player.top_day = response.data[i].score;
									player.user_icon_small = "https://graph.facebook.com/" + response.data[i].user.id + "/picture?width=128&height=128";
									player.rank = i+1;
									lbPage.push(player);
									if(response.data[i].user.id == SLINGO.Facebook.FBid){
										lbData.current_user = player;
										SLINGO.InGameLeaderboardElement._currUser = player;
									}
									if(i == response.data.length-1){ lbListHolder[page] = lbPage; }
								}				
							}						
							
							SLINGO.InGameLeaderboardElement._lbList = lbListHolder;
							if(!(SLINGO.InGameLeaderboardElement._friendNumPage in SLINGO.InGameLeaderboardElement._lbList)){
								if(1 in SLINGO.InGameLeaderboardElement._lbList){
									lbData.friend_board = SLINGO.InGameLeaderboardElement._lbList[1];
									lbData.friend_board_pos = 1;
									lbData.friend_board_eol = (2 in SLINGO.InGameLeaderboardElement._lbList) ? false: true;
								}
							}else{
								lbData.friend_board = SLINGO.InGameLeaderboardElement._lbList[SLINGO.InGameLeaderboardElement._friendNumPage];
								lbData.friend_board_pos = SLINGO.InGameLeaderboardElement._friendNumPage;
								var nextP = parseInt(lbData.friend_board_pos) + 1;
								lbData.friend_board_eol = (nextP in SLINGO.InGameLeaderboardElement._lbList) ? false : true;
							}							
						}else{ lbData.error = 'FB API failed to provide list';	}
					}
				}else if(SLINGO.InGameLeaderboardElement._reqListType[listType] == 'current_user'){
					if(response.data){
						for(var i=0; i < response.data.length; i++){				
							if(i !== 'xfind'){
								if(response.data[i].user.id == SLINGO.Facebook.FBid){
								var player = {};
								player.uid = response.data[i].id;
								player.username = response.data[i].user.first_name + " " + response.data[i].user.last_name;
								player.top_day = response.data[i].score;
								player.user_icon_small = "https://graph.facebook.com/" + response.data[i].user.id + "/picture?width=128&height=128";
								player.rank = i+1;
								SLINGO.InGameLeaderboardElement._currUser = player;
								lbData.current_user = player;
								}
							}				
						}						
					}else{ lbData.error = 'FB API failed to provide list';	}
				}else{ SLINGO.triggerError('updateFBLeaderboardCallback: unknown object requested');	}
			}
		}		
		//set type and error if none set before
		lbData.type='fb';
		if(!('error' in lbData)) lbData.error=0;		
		//set key with info
		SLINGO.GameComm.setGameVar('leaderboard', SLINGO.JSON.stringify(lbData));
	},
			
	parseReqObj : function(textObj){
		var returnParsedObj = {};
		try{
			var reqObj = SLINGO.JSON.parse(textObj);
			if (typeof reqObj != 'object'){
				throw 'textObj sent is an object';
			}
		}catch(e){
			returnParsedObj.error = 'SLINGO.InGameLeaderboardElement.parseReqObj: invalid textObj sent';
			return returnParsedObj;
			//return SLINGO.triggerError('SLINGO.InGameLeaderboardElement.parseReqObj: invalid textObj sent: '+textObj,SLINGO.EWARN);
		}		

		var objList = [];
		var post = {gameid: SLINGO.Game.gameId};
		if(SLINGO.Game.isDailyChallenge){			
			if (typeof reqObj.prefix == 'undefined' || reqObj.prefix == "base"){
				//get the default daily challenge
				var dc = SLINGO.Game.dailyChallenges.challenges;
				for(var key in dc) {
					if(dc[key].isDefaultChallenge)
						post.gameprefix = key;
				}			
			}else{
				post.gameprefix = reqObj.prefix;
			}
		}		
		
		//SLINGO.triggerError('GameComm.InGameLeaderboardElement._updateLeaderboard post: ' + SLINGO.JSON.stringify(post),SLINGO.EWARN);
		
		/* Convert a reqObj into a valid request */
		try{
			if (typeof reqObj.query == 'undefined'){
				throw "no query defined";
			}
			// reqObj.query is an indexed array with objects in it.  Each 
			// object contains its own definitions
			for (var i = 0; i < reqObj.query.length; i++){
				if (typeof reqObj.query[i].type == 'undefined'){
					throw "bad query";
				}
				var objName = reqObj.query[i].type;
				objList.push(objName);
				if (typeof reqObj.query[i].pos != 'undefined'){
					post['pos[' + objName + ']'] = parseInt(reqObj.query[i].pos);	
				}
				if (typeof reqObj.query[i].num != 'undefined'){
					post['num[' + objName + ']'] = parseInt(reqObj.query[i].num);	
				}
				if (typeof reqObj.query[i].extra != 'undefined'){
					// go through each extra to find what to add.
					post['xtra[' + objName + ']'] = reqObj.query[i].extra.join(',');
				}
			}
		}catch(err){
			returnParsedObj.error = 'SLINGO.InGameLeaderboardElement.parseReqObj: error' + err;
			return returnParsedObj;
		}
		
		post['objs'] = objList.join(',');
		returnParsedObj.post = post;
		return returnParsedObj;
	}
};

/* object to manage the rules of a game cycle */
SLINGO.GameCycleMgr = SLINGO.StateObj.extend({
	_obj_class_name 	: 'GameCycleMgr',
	_stage_mgr 			: null,
	_cycle_type 		: null,

	_stage_id 			: 'stageCont',
	_preroll_stage_id 	: 'prerollStage',
	_preload_stage_id	: 'preloadStage',
	
	_preroll_cookie_name: 'slpskip',

	tornyElmId : 'gamePageTournamentInfoElement',

	/* display flags - true=shown, false=hidden */
	shoflg	: {
		proll	: false,
		pload	: false,
		banr 	: false,
		game 	: false,
		lobby 	: false,
		inter 	: false,
		stage	: false,
		ugBanr	: false
	},
	setState : function(state) {
		this._super(state);
	},
	setCycleType : function(type){
		if(this._cycle_type)
			return SLINGO.triggerError('GameCycleMgr.setCycleType: cycle type already defined: '+this._cycle_type,SLINGO.EWARN);

		this._cycle_type = type;
		this.setStageMgr();
	},

	setStageMgr : function(){
		this._stage_mgr = new SLINGO.StageMgr(this._stage_id);

		var ps;
		if(SLINGO.Game.adBeaconWatchEnabled) {
			ps = new SLINGO.PrerollStage_BEACON(this._preroll_stage_id);
		} else {
			ps = new SLINGO.PrerollStage(this._preroll_stage_id);
		}
		this._stage_mgr.addStage('PREROLL', ps);

		var ps = new SLINGO.PreloadStage(this._preload_stage_id);
		this._stage_mgr.addStage('PRELOAD', ps);
		
		this._initGameStage();
		
		this._stage_mgr.getChild('PREROLL').addWatch('END',function(){SLINGO.GamePage.setGameCycleState('PREROLL_END')});
		
		this._stage_mgr.getChild('PRELOAD').addWatch('END',function(){SLINGO.GamePage.setGameCycleState('PRELOAD_END')});
		this._stage_mgr.getChild('PRELOAD').addWatch('TIMEOUT', function(){SLINGO.GameComm.relay_reportError('GEN:JS-GENERATEDTICKET,TYPE:PRELOAD,ERROR:DownloadTimeout')});		
		this._stage_mgr.getChild('GAME').addWatch('INTERMISSION_END', function(){SLINGO.GamePage.setGameCycleState('INTERMISSION_END')});
		
	},

	getStageMgr : function(){
		return (this._stage_mgr && (this._stage_mgr instanceof SLINGO.StageMgr))
			? this._stage_mgr
			: false;
	},

	updatePBar : function(prog){
		var mgr = this.getStageMgr();
		if(!mgr) return SLINGO.triggerError('GameCycleMgr.updatePBar: no stage mgr found',SLINGO.EWARN);
		var state = this.getState();
		switch(state){
			case 'PREROLL_START'		:
				var stage = mgr.getChild('PREROLL');	
				break;
			case 'INTERMISSION_START'	: var stage = mgr.getChild('GAME');		break;
//			default						: return SLINGO.triggerError('GameCycleMgr.updatePBar: no stage for you, invalid state: '+state,SLINGO.EWARN);
		}
		
		if('updatePBar' in stage) {
			return stage.updatePBar(prog);
		} else {
			return SLINGO.triggerError('GameCycleMgr.updatePBar: updatePBar function not found in stage',SLINGO.EWARN);
		}
	},

	_initGameStage : function(){
		var stageId = 'stage';
		switch(this._cycle_type){
			case 'SGS'			: var gs = new SLINGO.SGSStage(stageId);					break;
			case 'SOCKETLESS'	: var gs = new SLINGO.SocketlessStage(stageId);				break;
			case 'WRAPPERLESS'	: var gs = new SLINGO.WrapperlessStage(stageId);			break;
			case 'BLAST'		: var gs = new SLINGO.BlastStage(stageId);					break;
			case 'ADVANTAGE'	: var gs = new SLINGO.AdvantageStage(stageId);				break;
			case 'SDC'			: var gs = new SLINGO.SDCStage(stageId);					break;
			case 'SDCONFB'		: var gs = new SLINGO.FacebookSDCStage(stageId);			break;
			case 'SOLITAIREDC'	: var gs = new SLINGO.SolitaireDCStage(stageId);			break;
			case 'FIVECARDDC'	: var gs = new SLINGO.FiveCardDCStage(stageId);				break;
			case 'TWIKIDC'      : var gs = new SLINGO.AltGenDailyChallengeStage(stageId);	break; //var gs = new SLINGO.Stage(stageId);				break;
			case 'BLITZDC'      : var gs = new SLINGO.BlitzDailyChallengeStage(stageId);	break;
			case 'ISLANDDC'     : var gs = new SLINGO.IslandDCStage(stageId);				break;
			case 'ALLSTARSDC'   : var gs = new SLINGO.AllStarsDCStage(stageId);				break;
			case 'CHIPFLIPDC'   : var gs = new SLINGO.ChipFlipDCStage(stageId);				break;
			case 'ZENMATCHDC'	: var gs = new SLINGO.ZenMatchDCStage(stageId);				break;
			case 'DEEPSEADC'	: var gs = new SLINGO.DeepSeaDCStage(stageId);				break;
			case 'BINGOFLIPDC'	: var gs = new SLINGO.BingoFlipDCStage(stageId);			break;
			case 'PACHINKODC'	: var gs = new SLINGO.PachinkoDCStage(stageId);				break;
			case 'MATCHMAKERDC'	: var gs = new SLINGO.MatchmakerDCStage(stageId);			break;
			case 'SPELLBUGSDC'	: var gs = new SLINGO.SpellBugsDCStage(stageId);			break;
			case 'WORDROUNDUPDC'	: var gs = new SLINGO.WordRoundupDCStage(stageId);			break;
			case 'WORDFRENZYDC'	: var gs = new SLINGO.WordFrenzyDCStage(stageId);			break;
			case 'PICTURIFFICDC'	: var gs = new SLINGO.PicturifficDCStage(stageId);			break;
			case 'GENDC'		: var gs = new SLINGO.GenDailyChallengeStage(stageId);		break;
			case 'ALTGENDC'		: var gs = new SLINGO.AltGenDailyChallengeStage(stageId);	break;
			default				: return SLINGO.triggerError('GameCycleMgr._initGameStage: invalid cycle type: '+this._cycle_type,SLINGO.EWARN);
		}
		this._stage_mgr.addStage('GAME', gs);
	},

	_loadTournamentInfo : function(){
		if(!SLINGO.Game.isTourn || SLINGO.Game.isSample) return;
		if(!SLINGO.isPluginLoaded('Tournament')) return SLINGO.triggerError('GameCycleMgr._loadTournamentInfo: tournament plugin not loaded',SLINGO.EWARN);
		SLINGO.Tournament.requestInfoBox(SLINGO.Game.gameId, bind(this, this._postTournamentInfo));
	},

	_postTournamentInfo : function(txt){
		var e = SLINGO.$(this.tornyElmId);
		if(e) e.parentNode.removeChild(e);
		var endMark = SLINGO.$('tournamentMarker');
		if(endMark){
			var d = document.createElement('div');
			d.id = d.name = this.tornyElmId;
			endMark.parentNode.insertBefore(d,endMark);
			d.innerHTML = txt;
			if(SLINGO.ToolTip && txt.match(SLINGO.ToolTip.TIPCLASS)) SLINGO.ToolTip.init();
		}
	},
	
	_hasPreroll : function(){
		var args = SLINGO.getParsedQueryString();
		var traffic = args && ('traffic' in args) && (args.traffic === 'true');
		return (!traffic || SLINGO.getCookie(this._preroll_cookie_name));
	},
	
	_setSkippedPreroll : function(){
		var d = new Date;
		return SLINGO.setCookie(this._preroll_cookie_name, d.getTime(), 30);
	},

	_setState_PREROLL_START : function(){
		if(this.shoflg.proll)
			return SLINGO.triggerError('GameCycleMgr._setState_PREROLL_START: preroll already flagged as shown',SLINGO.EWARN);
			
		if(SLINGO.Game.isPreloadTest && !this._hasPreroll()){
			return true;
		}

		this._stage_mgr.showStage('PREROLL');
		this.shoflg.proll = true;
		
	
		/* TODO - fire google analytics event */
		return true;
	},
	_setState_PREROLL_END : function(){
		if(this.shoflg.stage){
			return SLINGO.triggerError('GameCycleMgr._setState_PREROLL_END: stage already flagged as shown',SLINGO.EWARN);
		}

		if(!this._hasPreroll()) this._setSkippedPreroll();
				
		this._stage_mgr.hideStage('PREROLL');
		this._stage_mgr.remChild('PREROLL');
		this.shoflg.proll = false;
		return true;
	},
	
	_setState_PRELOAD_START : function(){
		if(this.shoflg.pload)
			return SLINGO.triggerError('GameCycleMgr._setState_PRELOAD_START: preload already flagged as shown',SLINGO.EWARN);

		/* Removed 2012.09.08 -NV */
		/*if(this._stage_mgr.getChild('PRELOAD').getProgress() === 100){
			SLINGO.triggerError('GameCycleMgr._setState_PRELOAD_START: skipping preload, game is loaded',SLINGO.EWARN);
			return true;
		}*/
		this._stage_mgr.showStage('PRELOAD');
		this._stage_mgr.setChildState('PRELOAD','START');
		this.shoflg.pload = true;
		return true;
	},
	
	_setState_PRELOAD_END : function(){
		SLINGO.triggerError('GameCycleMgr._setState_PRELOAD_END',SLINGO.EWARN);
		if(this.shoflg.stage){
			return SLINGO.triggerError('GameCycleMgr._setState_PRELOAD_END: stage already flagged as shown',SLINGO.EWARN);
		}
		if(this.gameReadyPoll){
			clearTimeout(this.gameReadyPoll);
			this.gameReadyPoll = null;
		}
		
		/* TODO - should we fire an error overlay here? */
		if(SLINGO.GameComm.gameReadyCount > SLINGO.GameComm.maxReadyCount){
			return SLINGO.Game.isPreloadTest
				? SLINGO.GameComm.relay_reportError('GEN:JS-GENERATEDTICKET,TYPE:GAMEREADY,ERROR:GameNotReady')
				: SLINGO.triggerError('GameCycleMgr._setState_PRELOAD_END: game not ready',SLINGO.EWARN);
		} else if(!SLINGO.GameComm.isGameReady()){
			this.gameReadyPoll = setTimeout('SLINGO.GamePage.setGameCycleState("PRELOAD_END")', 2000);
			SLINGO.GameComm.gameReadyCount++;
			return false;
		}
		
		 this._loadTournamentInfo();
		return true;
	},
	_setState_GAME_START : function(){
		if(!this.shoflg.stage) {
            this.shoflg.stage = true;
            this._stage_mgr.showStage('GAME');
            this._stage_mgr.remChild('PRELOAD');
            this.shoflg.pload = false;
		}
		this._stage_mgr.setChildState('GAME', 'GAME_START');
		this.shoflg.game = true;
		
		return true;
	},

	_setState_GAME_OVER : function(){
		if(!this.shoflg.game)
			return SLINGO.triggerError('GameCycleMgr._setState_GAME_OVER: game not flagged as shown',SLINGO.EWARN);
		this._stage_mgr.setChildState('GAME', 'GAME_OVER');
		return true;
	},
	
	_setState_GAME_PLAY_AGAIN : function(){
		if(!this.shoflg.game)
			return SLINGO.triggerError('GameCycleMgr._setState_GAME_PLAY_AGAIN: game not flagged as shown',SLINGO.EWARN);

		this._stage_mgr.setChildState('GAME', 'GAME_PLAY_AGAIN');
		return true;
	},

	_setState_GAME_END : function(){
		if(!this.shoflg.game)
			return SLINGO.triggerError('GameCycleMgr._setState_GAME_END: game flagged as hidden',SLINGO.EWARN);

		this._stage_mgr.setChildState('GAME', 'GAME_END');
		return true;
	},

	_setState_INACTIVE : function(){
		if(!this.shoflg.game)
			return SLINGO.triggerError('GameCycleMgr._setState_GAME_END: game flagged as hidden',SLINGO.EWARN);

		this._stage_mgr.setChildState('GAME', 'INACTIVE');
		return true;
	},
	
	_setState_INTERMISSION_START : function(){
		if(this.shoflg.inter){
                    //continue;
                    //return SLINGO.triggerError('GameCycleMgr._setState_INTERMISSION_START: inter already flagged as shown',SLINGO.EWARN);
                }

		//this._stage_mgr.getChild('GAME').getStage('INTER').addWatch('END', function(){SLINGO.GamePage.setGameCycleState('INTERMISSION_END')});
		this._stage_mgr.setChildState('GAME', 'INTERMISSION_START');
		this.shoflg.game = false;
		this.shoflg.inter = true;
		this._loadTournamentInfo();
		/* TODO - fire Google Analytics event */

		return true;
	},

	_setState_INTERMISSION_END : function(){
		if(!this.shoflg.inter)
			return SLINGO.triggerError('GameCycleMgr._setState_INTERMISSION_END: inter flagged as hidden',SLINGO.EWARN);
		
		
		/* TODO - fire Google Analytics event */
		this.shoflg.inter = false;
		return true;
	},

	_setState_LOBBY_START : function(){
		SLINGO.triggerError('GameCycleMgr._setState_LOBBY_START',SLINGO.EWARN);

		if(!this.shoflg.stage) {
            this.shoflg.stage = true;
            this._stage_mgr.showStage('GAME');
            this._stage_mgr.remChild('PRELOAD');
            this.shoflg.pload = false;
		}
		
		this._stage_mgr.setChildState('GAME', 'LOBBY_START');
		return true;
	},

	_setState_LOBBY_END : function(){
		SLINGO.triggerError('GameCycleMgr._setState_LOBBY_END',SLINGO.EWARN);
		this._stage_mgr.setChildState('GAME', 'LOBBY_END');
		return true;
	},
	
	_onUpdState : function(st){		
		if(!st) return SLINGO.triggerError('GameCycleMgr._onUpdState: no state provided',SLINGO.EWARN);
		
		st = st.toUpperCase();
		switch(st){
			case 'PREROLL_START'	: return !this._hasPreroll() ? this.setState('PREROLL_END') : true;
			case 'PREROLL_END'		: return this.setState('PRELOAD_START');
			case 'PRELOAD_START'	: break;
			case 'PRELOAD_END'		: 
				if(this._cycle_type != 'SGS')
					return this.setState('GAME_START');
				break;
			case 'INTERMISSION_END' : 
				return this.setState('GAME_START');
				break;
			case 'LOBBY_START'		: break;
			case 'LOBBY_END'		:
				return this.setState('GAME_START');
				break;
			case 'GAME_START'		: break;
			case 'GAME_OVER'		: 
				//_gaq.push(['_trackEvent','Games','Play',SLINGO.Game.gameName]);
				ga('send', 'event', 'Games', 'Play', SLINGO.Game.gameName, {'nonInteraction': 1});
			   var stage = this._stage_mgr.getChild('GAME');
			   if(!stage._hasEndGameOverlay() ){
				   if (!SLINGO.GamePage.getGameComm().getGameVar('play_again_enable') ){
					   return this.setState('GAME_END');
				   }else{
					   return true;
				   }
			   }else{
				 if(SLINGO.Game.isWrapperless){
					if(!SLINGO.GamePage.getGameComm().getGameVar('play_again_enable') )
					   return true;
					//var score = SLINGO.GamePage.getGameComm().getGameVar('game_score');
				 }else{
					  score = SLINGO.GamePage.getGameComm().score;
					  if(score){
						   stage._showEndGameOverlay(score);
					  }else 
							SLINGO.triggerError('GameCycleMgr._onUpdState: invalid score from game: ' + score ,SLINGO.EWARN);
				 }				
			   }
				break;
			case 'GAME_PLAY_AGAIN'  :
				var stage = this._stage_mgr.getChild('GAME');
				if(stage._hasEndGameOverlay()){
					var score = SLINGO.GameComm.getGameVar('game_score');
					stage._showEndGameOverlay(score);
				} else {
					return this.setState('GAME_END');					
				}
				break;
			case 'GAME_END'			: 
				return this.setState('INTERMISSION_START'); 
				break;

		}
		
		return true;
	}

});

SLINGO.StageElement = SLINGO.DisplayElement.extend({
	_obj_class_name : 'StageElement',

	_setState_SHOW : function(){
		var elem = this.getElement();
		if(SLINGO.hasClassName(elem, 'prHide'))
			SLINGO.removeClassName(elem, 'prHide');

		return true;
	}

});

SLINGO.GamePageElement = SLINGO.DisplayElement.extend({
	_obj_class_name : 'GamePageElement',
	
	_setState_SHOW : function(){
		var elem = this.getElement();
		elem.style.display = 'block';
		return true;
	},
	
	_setState_HIDE : function(){
		var elem = this.getElement();
		elem.style.display = 'none';
		return true;
	}
	
});

SLINGO.GameElement = SLINGO.StageDisplay.extend({
	_obj_class_name : 'GameElement',

	_setState_SHOW : function(){
		var elem = this.getElement();
		if(SLINGO.Game.isWide){
			elem.style.display = 'block';
			elem.style.clear = 'both';
		}

		var w = SLINGO.Game.gameWidth + 'px';
		var h = SLINGO.Game.gameHeight + 'px';
		elem.style.marginLeft = '0px';
		SLINGO.GameComm.getFlashMovie().style.width = w;
		SLINGO.GameComm.getFlashMovie().style.height = h;

		// @@@ no more 1 px
		if(SLINGO.inArray(elem.style.width, ['', '2px'])) elem.style.width = w;
		if(SLINGO.inArray(elem.style.height, ['', '2px'])) elem.style.height = h;

		if(SLINGO.Game.resetGameH){
			elem.style.height = SLINGO.Game.resetGameH + 'px';
		}
		
		
		//check overlay height
		/*var overlayDiv = SLINGO.getElementsByClass('endGameMsg');
		if(overlayDiv.length > 0){
			if(endScreenH && SLINGO.Game.gameHeight < endScreenH){
				if(SLINGO.Game.resetGameH){
					
				}
				if(!SLINGO.Game.isBlast )
					elem.style.height = endScreenH + 'px';
			}
		}*/
		
		if(this.defaultStage)
			this.showStage(this.defaultStage);

		if(SLINGO.Game.isSample)
			elem.style.marginTop = '0px';

		return true;
	},

	_setState_HIDE : function(){
		//@@@this.getElement().style.height = SLINGO.GameComm.getFlashMovie().style.height = '1px';
		this.getElement().style.height = SLINGO.GameComm.getFlashMovie().style.height = '2px';
		this.getElement().style.height = SLINGO.GameComm.getFlashMovie().style.width = '2px';
		return true;
	}

});

SLINGO.GameCompanionElement = SLINGO.StageElement.extend({
	_obj_class_name : 'GameCompanionElement',

	_setState_SHOW : function(){
		this._super();

		var elem = this.getElement();
		if(!SLINGO.Game.isWide){
			elem.style.width = '225px';
			if(elem.style.display === 'none') elem.style.display = 'block';		/* Added 2012.11.16 -NV */
		} else {
			elem.style.width = SLINGO.Game.gameWidth + 'px';
			elem.style.display = 'block';
			elem.style.clear = 'both';
		}

		/* Removed 2012.11.16.. not sure what it was for.. -NV */
		/*if(SLINGO.Game.isSample)
		/	elem.style.height = SLINGO.Game.gameHeight + 'px';*/

		return true;
	},
	
	/* added 2012.11.16 -NV */
	_setState_HIDE : function(){
		if(this.getState() !== 'SHOW')
			return SLINGO.triggerError('GameCompanionElement._setState_HIDE: element is not currently displayed',SLINGO.EWARN);
			
		var elem = this.getElement();
		elem.style.display = 'none';
		return true;
	}

});

SLINGO.ChatElement = SLINGO.GameCompanionElement.extend({
	_obj_class_name : 'ChatElement',

	chatApp : null,

	/* chat on/off button info */
	chatToggleBtn	: null,
	chatBtnVal 		: "Chat is STATE",
	chatSwchBtnVal	: "TYPE Chat",
	chatNode		: null,
	chatOffImg		: null,
	chatOffImgSrc 	: '/images/common/muzzled.jpg',
	chatOffImgId 	: 'chatoffFill',
	chatOffImgCur	: null,
	chatHeight		: null,

	_onInit : function(){
		this.chatApp = SLINGO.$('chatFrame');
	},

	_setState_SHOW : function(){
		this._super();
		this.chatToggleBtn = SLINGO.$('generalChatSwitchBtn');

		/* override button onclick event */
		this.chatToggleBtn.onclick = '';
		this.chatToggleBtn.onclick = bind(this, this.toggleChat);
		return true;
	},

	_setState_ON : function(){
		this.chatToggleBtn.innerHTML = this.chatBtnVal.replace('STATE','ON');
		if(SLINGO.Game.isWide) this.chatApp.style.height = this.chatHeight + 'px';
		return true;
	},

	toggleChat : function(){
		switch(this.getState()){
			case 'SHOW':
			case 'ON':
				return this.setState('OFF');
			case 'OFF':
				return this.setState('ON');
			default:
				return SLINGO.triggerError('ChatElement.toggleChat: invalid state: '+this.getState());
		}
	},

	_switchChatOff : function(){
		this.chatNode = this.chatApp.cloneNode(true);
		if(!this.chatOffImg && !this._getChatOffImg()) return SLINGO.triggerError('ChatElement._setState_OFF: failed to create the chat off Img',SLINGO.EWARN);
		this.chatOffImgCur = this.chatOffImg.cloneNode(true);
		var h = SLINGO.DHTML.getObjectHeight(this.chatApp);
		if(SLINGO.Game.isWide && !this.chatHeight) this.chatHeight = h;
		this.chatOffImgCur.style.height = h + 'px';
		this.chatApp.parentNode.style.backgroundColor='#fff';
		return true;
	},

	_switchButtonOff : function(){
		this.chatToggleBtn.innerHTML = this.chatBtnVal.replace('STATE', 'OFF');
		return true;
	},

	_getChatOffImg : function(){
		this.chatOffImg = document.createElement('DIV');
		this.chatOffImg.id = this.chatOffImgId;
		var img = document.createElement('IMG');
		img.src = this.chatOffImgSrc;
		img.id = this.chatOffImgId + '_img';
		img.alt = 'Chat is currently turned off.';
		this.chatOffImg.appendChild(img);
		return this.chatOffImg;
	}

});

SLINGO.WebChatElement = SLINGO.ChatElement.extend({
	_obj_class_name : 'WebChatElement',

	chatObj : null,

	_onInit : function(){
		this._super();

		if(!SLINGO.isPluginLoaded('ChatMgr'))
			return SLINGO.triggerError('WebChatElement._onInit: ChatMgr plugin not found',SLINGO.EWARN);

		this.chatObj = SLINGO.ChatMgr;

		if(typeof SLINGO.ChatMgr.setDCCallback == 'function')
			SLINGO.ChatMgr.setDCCallback(bind(this, this.toggleChatOff));

		if(typeof SLINGO.ChatMgr.setEMCallback == 'function')
			SLINGO.ChatMgr.setEMCallback(bind(this, this.displayChatErrMsg));

		return true;
	},

	_setState_SHOW : function(){
		this._super();

		if(this.getState() !== 'INIT' && this.getState() !== 'END')
			return SLINGO.triggerError('WebChatElement._setState_SHOW: invalid state: '+this.getState(),SLINGO.EWARN);

		if(!this.chatObj)
			return SLINGO.triggerError('WebChatElement._setState_START: chat obj not found',SLINGO.EWARN);

		this.chatObj.init();
		return true;
	},

	_setState_ON : function(){
		if(this.getState() !== 'OFF')
			return SLINGO.triggerError('WebChatElement._setState_ON: invalid state: '+this.getState(),SLINGO.EWARN);

		if(!this.chatObj)
			return SLINGO.triggerError('WebChatElement._setState_ON: chat obj not found',SLINGO.EWARN);

		if(this.chatObj.joinOk)
			return SLINGO.triggerError('WebChatElement._setState_ON: joinOk not true',SLINGO.EWARN);

		this.chatObj.join();

		this.chatOffImgCur.parentNode.removeChild(this.chatOffImgCur);
		this.chatApp.style.display = 'block';

		return this._super();
	},

	_setState_OFF : function(){
		if(!SLINGO.inArray(this.getState(), ['SHOW','ON']))
			return SLINGO.triggerError('WebChatElement._setState_OFF: can\'t stop chat, invalid state: '+this.getState(),SLINGO.EWARN);

		if(!this.chatObj)
			return SLINGO.triggerError('WebChatElement._setState_OFF: chat obj not found',SLINGO.EWARN);

		if(!this.chatObj.joinOk)
			return SLINGO.triggerError('WebChatElement._setState_OFF: joinOk is true',SLINGO.EWARN);

		this.chatObj.disconnect();
		return true;
	},

	toggleChatOff : function(){
		this._switchChatOff();
		this.chatApp.style.display = 'none';
		this.getElement().appendChild(this.chatOffImgCur);
		if(SLINGO.Game.isWide && SLINGO.$('chatoffFill_img')){
			this.chatOffImgCur.style.width = '100%';
			SLINGO.$('chatoffFill_img').style.margin = '10px';
		}

		this._switchButtonOff();
		return true;
	},

	displayChatErrMsg : function(msg){
		if(!msg || !SLINGO.$('chatoffFill')) return;

		if(SLINGO.$('chatoffFillMsg'))
			SLINGO.$('chatoffFillMsg').parentNode.removeChild(SLINGO.$('chatoffFillMsg'));

		var d = document.createElement('div');
		d.id = 'chatoffFillMsg';
		d.style.fontWeight = 'bold';
		d.style.textAlign = 'justify';

		if(SLINGO.Game.isWide){
			d.style.width = '40%';
			d.style.margin = '10px';
		} else {
			d.style.width = '91%';
			d.style.margin = '0px 10px';
		}

		d.innerHTML = msg;
		SLINGO.$('chatoffFill').appendChild(d);
		return true;
	}

});

SLINGO.FlashChatElement = SLINGO.ChatElement.extend({
	_obj_class_name : 'FlashChatElement',

	_setState_ON : function(){
		this.chatApp = this.chatNode.cloneNode(true);
		this.chatOffImgCur.parentNode.style.backgroundColor='#000';
		this.getElement().replaceChild(this.chatApp, this.chatOffImgCur);
		return this._super();
	},

	_setState_OFF : function(){
		this._switchChatOff();
		this.getElement().replaceChild(this.chatOffImgCur, this.chatApp);
		this._switchButtonOff();
		if(SLINGO.Game.isWide) this.chatOffImgCur.style.height = '0px';
		return true;
	}

});

SLINGO.ScoreLeaderboardElement = SLINGO.GameCompanionElement.extend({
	_obj_class_name : 'ScoreLeaderboardElement',

	refresh_btn_id		: 'sideLBRefresh',
	load_more_btn_id	: 'loadNextPage',

	load_more_reqh		: '/ajax/sgs/load_more_friends/',

	friends_lb_reqh		: '/ajax/sgs/sb_friends_lb/',
	friends_tab_id		: 'friendsList',
	friends_wrapper_id 	: 'sideLBScoreListFriendsWrapper',
	friends_list_id		: 'sideLBScoreListFriends',

	global_lb_reqh		: '/ajax/sgs/sb_global_lb/',
	global_tab_id		: 'globalList',
	global_wrapper_id 	: 'sideLBScoreListGlobalWrapper',

	_onInit : function(){
		if(SLINGO.Game.gameId == 155) return true;	/* TODO - make this work for Blast as well */

		/* Add the switch tab events. */
		if(SLINGO.$(this.friends_tab_id)) 	SLINGO.$(this.friends_tab_id).onclick = bindWithArgs(this, this.showHideDiv, [this.friends_wrapper_id]);
		if(SLINGO.$(this.global_tab_id))	SLINGO.$(this.global_tab_id).onclick = bindWithArgs(this, this.showHideDiv, [this.global_wrapper_id]);

		/* Add the refresh button event. */
		if(SLINGO.$(this.refresh_btn_id)) 	SLINGO.$(this.refresh_btn_id).onclick = bind(this, this.refreshLBScores);

		/* Add the load more button event. */
		if(SLINGO.$(this.load_more_btn_id)) SLINGO.$(this.load_more_btn_id).onclick = bind(this, this.loadMoreFriends);
	
		return true;
	},

	showHideDiv : function(divToShow){
		switch(divToShow){
			case this.global_wrapper_id:
				SLINGO.$(this.global_wrapper_id).style.display = 'block';
				SLINGO.$(this.friends_wrapper_id).style.display = 'none';

				SLINGO.$(this.global_tab_id).style.backgroundColor = '#BBB';
				SLINGO.$(this.global_tab_id).style.color = '#000';

				SLINGO.$(this.friends_tab_id).style.backgroundColor = '';
				SLINGO.$(this.friends_tab_id).style.color = '';
				break;

			case this.friends_wrapper_id:
				if(SLINGO.$(this.friends_wrapper_id)) 	SLINGO.$(this.friends_wrapper_id).style.display = 'block';
				if(SLINGO.$(this.global_wrapper_id)) 	SLINGO.$(this.global_wrapper_id).style.display = 'none';

				if(SLINGO.$(this.friends_tab_id)){
					SLINGO.$(this.friends_tab_id).style.backgroundColor = '#BBB';
					SLINGO.$(this.friends_tab_id).style.color = '#000';
				}

				if(SLINGO.$(this.global_tab_id)){
					SLINGO.$(this.global_tab_id).style.backgroundColor = '';
					SLINGO.$(this.global_tab_id).style.color = '';
				}
				break;

			default:
				SLINGO.$(this.friends_wrapper_id).style.display = 'block';
				SLINGO.$(this.global_wrapper_id).style.display = 'none';

				SLINGO.$(this.friends_tab_id).style.backgroundColor = '#BBB';
				SLINGO.$(this.friends_tab_id).style.color = '#000';

				SLINGO.$(this.global_tab_id).style.backgroundColor = '';
				SLINGO.$(this.global_tab_id).style.color = '';
				break;
		}

		return true;
	},

	refreshLBScores : function(){
		return (SLINGO.$(this.global_wrapper_id) && (SLINGO.$(this.global_wrapper_id).style.display == 'block'))
			? this.refreshGlobalScores()
			: this.refreshFriendScores();
	},

	refreshFriendScores : function(){
		var d = new Date;
		var n = 'friendslbrf_'+ d.getTime();
		var path = this.friends_lb_reqh+SLINGO.Game.gameId+'/'+d.getTime();
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchFriendLBScores), path)){
			SLINGO.$(this.friends_wrapper_id).innerHTML = '<h2 id="msg">Updating Friends Scores....</h2>';
			this.showHideDiv(this.friends_wrapper_id);
			return true;
		}else return SLINGO.triggerError("ScoreLeaderboardElement.refreshFriendScores: could not send ajax request",SLINGO.EWARN);
	},

	catchFriendLBScores : function(txt){
		SLINGO.$(this.friends_wrapper_id).innerHTML = txt;
                
                /* Add the load more button event. */
		if(SLINGO.$(this.load_more_btn_id)) SLINGO.$(this.load_more_btn_id).onclick = bind(this, this.loadMoreFriends);
		return true;
	},

	refreshGlobalScores : function(){
		var d = new Date;
		var n = 'globallbrf_'+d.getTime();
		var path = this.global_lb_reqh+SLINGO.Game.gameId+'/'+d.getTime();
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchGlobalScores), path)){
			SLINGO.$(this.global_wrapper_id).innerHTML = '<h2 id="msg">Updating Global Scores.....</h2>';
			this.showHideDiv(this.global_wrapper_id);
			return true;
		}else return SLINGO.triggerError('ScoreLeaderboardElement.refreshGlobalScores: could not send ajax request',SLINGO.EWARN);
	},

	catchGlobalScores : function(txt){
		SLINGO.$(this.global_wrapper_id).innerHTML = txt;
		return true;
	},

	loadMoreFriends : function(){
		var btn = SLINGO.$(this.load_more_btn_id);

		var uid = btn.getAttribute('uid');
		if(!uid) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no uid provided',SLINGO.EWARN);

		var username = btn.getAttribute('username');
		if(!username) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no username provided',SLINGO.EWARN);

		var userTopScore = btn.getAttribute('usertopscore');
		if(!userTopScore) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no usertopscore provided',SLINGO.EWARN);

		var rank = btn.getAttribute('rank');
		if(!rank) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no rank provided',SLINGO.EWARN);

		var isUserPlacedInList = btn.getAttribute('isuserplacedinlist');
		if(!isUserPlacedInList) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no isuserplacedinlist provided',SLINGO.EWARN);

		var secondaryTopScore = btn.getAttribute('secondarytopscore');
		var orderBy = btn.getAttribute('orderby');
		
		var d = new Date;
		var n = 'loadmorelbrf_'+d.getTime();
		var path = this.load_more_reqh+SLINGO.Game.gameId+'/'+uid + '/' + username + '/' + userTopScore + '/' + rank + '/' + isUserPlacedInList + '/';
		if(secondaryTopScore){
			path += secondaryTopScore+'/';
			if(orderBy) path += orderBy+'/';
		}
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchMoreFriends), path)){
			return true;
		}else return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: could not send ajax request',SLINGO.EWARN);
	},

	catchMoreFriends : function(txt){
		//removing previous load more friends div
		var child = SLINGO.$(this.load_more_btn_id);
                var parent = SLINGO.$(this.friends_list_id);
                parent.removeChild(child);

		SLINGO.$(this.friends_list_id).innerHTML += txt;
                
                /* Add the load more button event. */
		if(SLINGO.$(this.load_more_btn_id)) SLINGO.$(this.load_more_btn_id).onclick = bind(this, this.loadMoreFriends);
		return true;
	}

});

SLINGO.GenDailyChallengeScoreLeaderboardElement = SLINGO.ScoreLeaderboardElement.extend({
	_obj_class_name : 'GenDailyChallengeScoreLeaderboardElement',	
	prefix		:  null,	
	setPrefix : function(prefix){
		this.prefix = prefix;
	},

	refreshFriendScores : function(){
		var d = new Date;
		var n = 'friendslbrf_'+ d.getTime();
		var path = this.friends_lb_reqh+SLINGO.Game.gameId+'/'+ this.prefix+'/'+d.getTime();
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchFriendLBScores), path)){
			SLINGO.$(this.friends_wrapper_id).innerHTML = '<h2 id="msg">Updating Friends Scores....</h2>';
			this.showHideDiv(this.friends_wrapper_id);
			return true;
		}else return SLINGO.triggerError("SDCScoreLeaderboardElement.refreshFriendScores: could not send ajax request",SLINGO.EWARN);
	},

	refreshGlobalScores : function(){ 
		var d = new Date;
		var n = 'globallbrf_'+d.getTime();
		var path = this.global_lb_reqh+SLINGO.Game.gameId+'/'+ this.prefix+'/'+d.getTime();
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchGlobalScores), path)){
			SLINGO.$(this.global_wrapper_id).innerHTML = '<h2 id="msg">Updating Global Scores.....</h2>';
			this.showHideDiv(this.global_wrapper_id);
			return true;
		}else return SLINGO.triggerError('SDCScoreLeaderboardElement.refreshGlobalScores: could not send ajax request',SLINGO.EWARN);
	},
	
	loadMoreFriends : function(){
		var btn = SLINGO.$(this.load_more_btn_id);

		var uid = btn.getAttribute('uid');
		if(!uid) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no uid provided',SLINGO.EWARN);

		var username = btn.getAttribute('username');
		if(!username) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no username provided',SLINGO.EWARN);

		var userTopScore = btn.getAttribute('usertopscore');
		if(!userTopScore) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no usertopscore provided',SLINGO.EWARN);

		var rank = btn.getAttribute('rank');
		if(!rank) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no rank provided',SLINGO.EWARN);

		var isUserPlacedInList = btn.getAttribute('isuserplacedinlist');
		if(!isUserPlacedInList) return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: no isuserplacedinlist provided',SLINGO.EWARN);
		
		var prefix = btn.getAttribute('prefix');
		if(prefix){
			var post = {'prefix' : prefix};
		}

		var d = new Date;
		var n = 'loadmorelbrf_'+d.getTime();
		var path = this.load_more_reqh+SLINGO.Game.gameId+'/'+uid + '/' + username + '/' + userTopScore + '/' + rank + '/' + isUserPlacedInList + '/';
		if(SLINGO.XHR.createAndSendRequest(n, bind(this, this.catchMoreFriends), path, 'POST', 'TEXT', true, post)){
			return true;
		}else return SLINGO.triggerError('ScoreLeaderboardElement.loadMoreFriends: could not send ajax request',SLINGO.EWARN);
	}
});

SLINGO.LobbyElement = SLINGO.StageDisplay.extend({
	_obj_class_name : 'LobbyElement',
	_progTxt		: null,
	lobbyAd			: {
		node : null
	},
	/* element widths for manipulating the stage */
	eWidths : {
		curr 	: null,
		base 	: null,
		lobby	: null,
		game	: null,
		inter 	: null
	},
	adCycle : {
		lobby : {
			cycle		: true,//if we want the ads to cycle at all
			interval	: null,//handle for the setInterval call
			minTimeReq	: 20,//minimum time in secs left in the lobby to call another video
			cycleTime	: 45000,//time in millisecs inbetween new ad calls
			cycleCount	: 0,//to be incremented with each ad cycle run
			wideLayout	: false,//flag to denote if a wide layout with no skyscraper for a video with companion is called
			origWidth	: null,
			addWidth	: 160//number of pixels to increase the stage width by to allow for companion ad
		}
	},
	banrAd 		: {
		node	: null
	},
	_onInit : function(){
		this.chat		  = SLINGO.$('chatContainer');
		this.lobbyAd.node = SLINGO.$('lobbyAd');
		/*
		this.banr		  = SLINGO.$('bannerElm');
		this.banrAd.node  = SLINGO.$('skyscraper');
		this.banrAd.node.src 	= '';
		*/
		this.lobby		  = SLINGO.$('lobbyElm');
		this.stageCont	  = SLINGO.$('stageCont');
	},
	
	_setState_SHOW : function(){
		this.stage 		  = SLINGO.GamePage.getGameCycleMgr().getStageMgr().getChild('GAME');

		var elem = this.getElement();
		elem.style.display = 'block';
		if(SLINGO.hasClassName(elem, 'prHide'))
			SLINGO.removeClassName(elem, 'prHide');

		if(!this.eWidths.lobby){
			this.eWidths.lobby = SLINGO.DHTML.getObjectWidth(this.lobby);
		}
		
		this.setLobbyAd();
		if(this.adCycle.lobby.cycle)
			this.adCycle.lobby.interval = setInterval(SLINGO.bind(this,this.setLobbyAd), this.adCycle.lobby.cycleTime);

		return true;
	},

	_setState_HIDE : function(){
		if(this.getState() !== 'SHOW') return;
		
		if(this.adCycle.lobby.interval)
			clearInterval(this.adCycle.lobby.interval);
		
		var elem = this.getElement();
		elem.style.display = 'none';
		if(!SLINGO.hasClassName(elem, 'prHide'))
			SLINGO.addClassName(elem, 'prHide');
		return true;
	},
	setLobbyAd : function(){
		/* get the current progress sent from the SGS */
		//var progTxt = SLINGO.$('progTxtTarg');
		//var timeStr = progTxt.innerHTML.toLowerCase();
		var timeStr = SLINGO.GameComm._progTxt.toLowerCase();
		var showVideo = false;
		var vSrc = false;
		var vSrcTime = false;
		var vSrcOrd = Math.random()*10000000000000000;
				
		/* if we are in any of the 3 phases of the lobby, there's time to show a video */
		if((timeStr.indexOf('spin') != -1) || (timeStr.indexOf('finishing') != -1) || (timeStr.indexOf('round') != -1)){
			showVideo = true;
			vSrcTime = '30';
		}

		/* if we are in the count down to the game, check and see if there's enough time */
		if(timeStr.indexOf('seconds') != -1){
			var pattern = /[0-9]+/;
			var remainingTime = parseInt(timeStr.match(pattern));
			if(remainingTime >= this.adCycle.lobby.minTimeReq){
				showVideo = true;
				/* we need to see if there's enough time to show a 30 second video or not */
				vSrcTime = remainingTime > 30 ? '30' : '15';
			}
		}

		/* first make sure the src is cleared */
		this.lobbyAd.node.src = '';

		if(showVideo){
			/* if we have time for a 30 second video, and this is the first cycle, so a video with companion */
			if(this.adCycle.lobby.cycleCount == 0 && vSrcTime == 30){
				vSrcTime += 'wc';
				this.adCycle.lobby.wideLayout = true;
			} else
				this.adCycle.lobby.wideLayout = false;//this is the default value but we need it here for the cycle

			/* we need to make the proper replacements in the src string */
			vSrc = SLINGO.Game.lobbyVideoSrc;
			vSrc = vSrc.replace('ordtime', vSrcOrd);
			vSrc = vSrc.replace('videotime', 'lobby'+vSrcTime);
			this.lobbyAd.node.src = !showVideo ? this.lobbyAd.node.src : vSrc;

		}
		this.setLobbyLayout();
		this.adCycle.lobby.cycleCount += 1;
	},
	setLobbyLayout : function(){
		var w = (this.eWidths.lobby + parseInt(this.chat.style.width));
		if(!this.adCycle.lobby.wideLayout){
			//put everything back where it belongs
			this.showBanner();
			SLINGO.$('stageRightCol').style.width = this.eWidths.lobby + 'px';
			SLINGO.$('lobbyElm').style.width = this.adCycle.lobby.origWidth;
			this.lobbyAd.node.style.width = 300 + 'px';
		} else {
			//set up for a wide ad with no skyscraper
			this.hideBanner();
			SLINGO.$('stageRightCol').style.width = this.eWidths.lobby + this.adCycle.lobby.addWidth + 'px';
			SLINGO.$('lobbyElm').style.width = this.eWidths.lobby + this.adCycle.lobby.addWidth + 'px';
			w = w + this.adCycle.lobby.addWidth;
			this.lobbyAd.node.style.width = this.eWidths.lobby + this.adCycle.lobby.addWidth + 'px';
		} 
		this.stage._setStageWidth(w);
	},
	showBanner : function(){		
		/* Added 2011.05.12 for when flyout is open and ad is hidden -NV */
		if(false){
			SLINGO.triggerError('SGS.showBanner: static ad refresh disabled',SLINGO.EWARN);
			return;
		}
		
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			SLINGO.$(SLINGO.GamePage.Ads[i].containerId).style.display = 'block';
			SLINGO.$(SLINGO.GamePage.Ads[i].iframeId).src = SLINGO.GamePage.Ads[i].adSource;
		}
		
		/* fire the google analytics event */
		//this._trackGAEvent('GMPGEVT', 'SKYSCRPRAD', this.gameName, 1);
			
		return true;
	},

	hideBanner : function(){
		for(var i = 0; i < SLINGO.GamePage.Ads.length; i++) {
			SLINGO.$(SLINGO.GamePage.Ads[i].containerId).style.display = 'none';
			SLINGO.$(SLINGO.GamePage.Ads[i].iframeId).src = '';
		}
		return true;
	}
});

SLINGO.IntermissionElement = SLINGO.StageDisplay.extend({
	_obj_class_name : 'IntermissionElement',

	_pbar_obj : null,
	_pbar_ready : null,
	_pbar_duration : 90, // 40

	_video_obj : null,

	_onInit : function(){
		this.addChild('PBARCONTAINER', new SLINGO.DisplayElement('stageAdPBar'));
		this.addChild('VIDEOAD', new SLINGO.VideoAdElement('videoAd'));
	},

	_setState_SHOW : function(){
		var state = this.getState();
		if(!SLINGO.inArray(this.getState(),['INIT','HIDE', 'END'])) // @todo maybe end statement should not goes here
			return SLINGO.triggerError('IntermissionElement._setState_SHOW: invalid state: '+state,SLINGO.EWARN);

		var elem = this.getElement();
		if(SLINGO.hasClassName(elem, 'prHide')) SLINGO.removeClassName(elem, 'prHide');
		elem.style.visibility = 'visible';
		elem.style.display = 'block';
		return true;
	},

	_setState_HIDE : function(){
		var state = this.getState();
		if(state !== 'END')
			//return SLINGO.triggerError('IntermissionElement._setState_HIDE: invalid state: '+state,SLINGO.EWARN);
			SLINGO.triggerError('IntermissionElement._setState_HIDE: invalid state: '+state,SLINGO.EWARN);

		if(this._pbar_ready)
			this._pbar_obj.reset();

		this.setChildState('VIDEOAD', 'END');
		this.getElement().style.display = 'none';

		return true;
	},

	_setState_START : function(){
		var state = this.getState();
		if(state !== 'SHOW')
			return SLINGO.triggerError('IntermissionElement._setState_START: invalid state: '+state,SLINGO.EWARN);

		this.initPBar();
		this.getChild('VIDEOAD').setSrc(SLINGO.Game.interSrc);
		this.setChildState('VIDEOAD', 'START');

		return true;
	},

	_setState_END : function(){
		return true;
	},

	_onUpdState : function(){
		if(this.getState() === 'SHOW')
			this.setState('START');
	},

	initPBar : function(){
		if(!this._pbar_ready){
			this._pbar_obj = new SLINGO.PBar.ProgressBar(this.getChild('PBARCONTAINER').getElement());
			if(!this._pbar_obj) return SLINGO.triggerError('IntermissionElement.initPBar:failed to initialize the intermission progress bar',SLINGO.EWARN);
			this._pbar_obj.INTEROBJ_REF = this;
			this._pbar_obj.LOAD_TEXT = 'Intermission:';
			this._pbar_obj.progressBarClass = 'pBarFill';
            this._pbar_obj.TOTAL_SECONDS = SLINGO.Game.isDevMode === true ? 10 : this._pbar_duration;		

			var cycleType = SLINGO.GamePage.getGameCycleType();
			if(!cycleType) return SLINGO.triggerError('IntermissionElement.initPBar: no game cycle type found',SLINGO.EWARN);

			switch(cycleType){
				case 'SGS':
					if(SLINGO.GamePage.getGameComm().gameReady) {
						this._pbar_obj.TIME_BASED_UPDATES	= false;
						SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
					} else 
						SLINGO.GamePage.intermissionStarted = true;
					break;

				case 'SOCKETLESS':
					if(SLINGO.GamePage.getGameComm().gameReady) {
						SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
					} else 
						SLINGO.GamePage.intermissionStarted = true;
					
				default:
					this._pbar_obj.TIME_BASED_UPDATES	= true;
					this._pbar_obj.endCallFunction		= this.catchPBarEnd;
					break;
			}

			this._pbar_obj.init();
			this._pbar_ready = true;
		}

		if(!this._pbar_ready)
			return SLINGO.triggerError('IntermissionElement.initPBar: progress bar not ready',SLINGO.EWARN);

		this._pbar_obj.launch();
		return true;
	},

	catchPBarEnd : function(){
		if('INTEROBJ_REF' in this)
			return this.INTEROBJ_REF.catchPBarEnd();
		else this.setState('END');
	}

});

SLINGO.IntermissionElement_BEACON = SLINGO.StageDisplay.extend({
	_obj_class_name : 'IntermissionElement_BEACON',

        _pbarEnabled : false,
	_pbar_elm_id : null,
	_pbar_obj : null,
	_pbar_ready : null,
	_pbar_duration : 90, // 40
	_skipButtonActivateTime : 35,
	_nonBeaconDuration : 90, // 40
	_adErrorBufferTime : 10,
	
	_video_obj : null,
	_video_elm_id : null,
	_video_src : null,
        
        _loader_elm_id : null,
        _lastLoaderTick : 0,
	
	_melAdLoadedCallbackRef : null,
        _melAdStartedCallbackRef : null,
	_melAdStoppedCallbackRef : null,
	_melAdErrorCallbackRef : null,
	
	_adErrorOccurred : false,
	_beaconActive : false,
        _skipButtonEnabled : false, /* whether or not to even display the skip button -NV */
	_skipButtonActive : false,
	_skipEventsAdded : false,
	init : function(mainContainer, opts){
            
            console.log("VALUE OF TEST VAR: "+SLINGO.GamePage.testValue);
            
            
		if(opts && (typeof opts == 'object')){
			var keys = ['pbar_elm_id','video_elm_id','video_src','loader_elm_id'];
			for(k in keys){
				var propName = '_'+keys[k];
				if((keys[k] in opts) && opts[keys[k]] && (propName in this))
					this[propName] = opts[keys[k]];
			}
		}
						
		var elms = {};
                if(SLINGO.$(this._pbar_elm_id)) elms.PBARCONTAINER = new SLINGO.DisplayElement(this._pbar_elm_id);
                else SLINGO.triggerError('IntermissionElement_BEACON.init: no pbar elem',SLINGO.EWARN);
                
                if(!this._pbarEnabled){
                    SLINGO.$(this._pbar_elm_id).style.display = 'none';
                    if(this._loader_elm_id && SLINGO.$(this._loader_elm_id)){
                        SLINGO.$(this._loader_elm_id).parentNode.style.textAlign = 'left';
                        SLINGO.$(this._loader_elm_id).parentNode.style.marginLeft = '10px';
                    }
                }
		
		if(SLINGO.$(this._video_elm_id)) elms.VIDEOAD = new SLINGO.VideoAdElement(this._video_elm_id);
		else SLINGO.triggerError('IntermissionElement_BEACON.init: no video elem',SLINGO.EWARN);
		
		this._super(mainContainer, elms);
	},

	_setState_SHOW : function(){
		var state = this.getState();
		if(!SLINGO.inArray(this.getState(),['INIT','HIDE', 'END'])) // @todo maybe end statement should not goes here
			return SLINGO.triggerError('IntermissionElement_BEACON._setState_SHOW: invalid state: '+state,SLINGO.EWARN);
		
		this._beaconActive = false;
		this._skipButtonActive = false;
		
                if(this._skipButtonEnabled){
                    var skipBtnNode;

                    skipBtnNode = SLINGO.$('skipAdBtnPreRoll');
                    if(skipBtnNode) {
                            SLINGO.removeClassName(skipBtnNode,'active');	
                            if(!this._skipEventsAdded) {
                                    SLINGO.Event.addEvent(skipBtnNode,'onclick',bind(this,this.skipAd));
                            }
                    }
                    skipBtnNode = SLINGO.$('skipAdBtnInter');
                    if(skipBtnNode) {
                            SLINGO.removeClassName(skipBtnNode,'active');	
                            if(!this._skipEventsAdded) {
                                    SLINGO.Event.addEvent(skipBtnNode,'onclick',bind(this,this.skipAd));
                            }
                    }	

                    this._skipEventsAdded = true; // Assuming everything went well up there...
                }
		
		if(SLINGO.MessageEventListener) {
			this._melAdLoadedCallbackRef  = SLINGO.MessageEventListener.registerCallback('ANY', 'AdLoaded', bind(this, this.catchAdLoaded));
                        this._melAdStartedCallbackRef  = SLINGO.MessageEventListener.registerCallback('ANY', 'AdStarted', bind(this, this.catchAdStarted));
			this._melAdStoppedCallbackRef = SLINGO.MessageEventListener.registerCallback('ANY', 'AdStopped', bind(this, this.catchAdStopped));
			this._melAdErrorCallbackRef = SLINGO.MessageEventListener.registerCallback('ANY','AdError',bind(this,this.catchAdError))
		
			if(!SLINGO.MessageEventListener.isListening) {
				SLINGO.MessageEventListener.start();
			}
		} 
		var elem = this.getElement();
                if(SLINGO.hasClassName(elem, 'prHide')) SLINGO.removeClassName(elem, 'prHide');
		elem.style.visibility = 'visible';
		elem.style.display = 'block';
                
                if(this._loader_elm_id && SLINGO.$(this._loader_elm_id))
                    SLINGO.$(this._loader_elm_id).style.visibility = 'visible';
                
		return true;
	},

	_setState_HIDE : function(){
		var state = this.getState();
		if(state !== 'END')
			//return SLINGO.triggerError('IntermissionElement._setState_HIDE: invalid state: '+state,SLINGO.EWARN);
			SLINGO.triggerError('IntermissionElement_BEACON._setState_HIDE: invalid state: '+state,SLINGO.EWARN);

		if(this._pbar_ready)
			this._pbar_obj.reset();

		this.setChildState('VIDEOAD', 'END');
		this.getElement().style.display = 'none';

		return true;
	},

	_setState_START : function(){
		var state = this.getState();
		if(state !== 'SHOW')
			return SLINGO.triggerError('IntermissionElement_BEACON._setState_START: invalid state: '+state,SLINGO.EWARN);

		this.initPBar();
		this.getChild('VIDEOAD').setSrc(this._video_src);
		this.setChildState('VIDEOAD', 'START');

		return true;
	},

	_setState_END : function(){
		return true;
	},

	_onUpdState : function(){
		if(this.getState() === 'SHOW')
			this.setState('START');
	},
	
	setPBarElement : function(elmId){
		if(!elmId) return SLINGO.triggerError('IntermissionElement_BEACON.setPBarElm: invalid elmId: '+elmId,SLINGO.EWARN);
		
		this._pbar_elm_id = elmId;
		return true;
	},
	
	setVideoSrc : function(vidSrc){
		if(!vidSrc) return SLINGO.triggerError('IntermissionElement_BEACON.setVideoElm: invalid vidSrc: '+vidSrc,SLINGO.EWARN);
			
		this._video_src = vidSrc;
		return true;
	},

	initPBar : function(){
            	if(!this._pbar_ready){
			this._pbar_obj = new SLINGO.PBar.ProgressBar(this.getChild('PBARCONTAINER').getElement());
			if(!this._pbar_obj) return SLINGO.triggerError('IntermissionElement_BEACON.initPBar:failed to initialize the intermission progress bar',SLINGO.EWARN);
			this._pbar_obj.INTEROBJ_REF = this;
			this._pbar_obj.LOAD_TEXT = 'Intermission:';
			this._pbar_obj.progressBarClass = 'pBarFill';
			this._pbar_obj.TOTAL_SECONDS = SLINGO.Game.isDevMode === true ? 10 : this._pbar_duration;
			this._pbar_obj.addTickCallback(bind(this,this.onProgressTick));

			var cycleType = SLINGO.GamePage.getGameCycleType();
			if(!cycleType) return SLINGO.triggerError('IntermissionElement.initPBar_BEACON: no game cycle type found',SLINGO.EWARN);

			switch(cycleType){
				case 'SGS':
					if(SLINGO.GamePage.getGameComm().gameReady) {
						this._pbar_obj.TIME_BASED_UPDATES	= false;
						SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
					} else 
						SLINGO.GamePage.intermissionStarted = true;
					break;

				case 'SOCKETLESS':
					if(SLINGO.GamePage.getGameComm().gameReady) {
						SLINGO.GamePage.getGameComm().getFlashMovie().sgs_CallActionscriptMethod('startIntermission','true');
					} else 
						SLINGO.GamePage.intermissionStarted = true;

				default:
					this._pbar_obj.TIME_BASED_UPDATES	= true;
					this._pbar_obj.endCallFunction		= bind(this,this.catchPBarEnd);
					break;
			}

			this._pbar_obj.init();
			this._pbar_ready = true;
		}

		if(!this._pbar_ready)
			return SLINGO.triggerError('IntermissionElement_BEACON.initPBar: progress bar not ready',SLINGO.EWARN);

		this._pbar_obj.launch();
		return true;
	},

	catchPBarEnd : function(){
            if(!this._beaconActive){
		this.cleanUpCallbacks();

		if('INTEROBJ_REF' in this)
			return this.INTEROBJ_REF.catchPBarEnd();
		else this.setState('END');
            }
	},
	onProgressTick : function(elapsedTime) {
		/* If there's an AdError with <= _adErrorBufferTime */
		var secondsElapsed = (elapsedTime / 1000);
		if(this._adErrorOccurred  && (secondsElapsed <= this._adErrorBufferTime) ) {
			this._adErrorOccurred = false;
			this._beaconActive = false;
			
			this.setChildState('VIDEOAD', 'REFRESH');
			
			if(this._melAdLoadedCallbackRef == null)
				this._melAdLoadedCallbackRef  = SLINGO.MessageEventListener.registerCallback('ANY', 'AdLoaded', bind(this, this.catchAdLoaded));
                            
                        if(this._melAdStartedCallbackRef == null)
				this._melAdStartedCallbackRef  = SLINGO.MessageEventListener.registerCallback('ANY', 'AdStarted', bind(this, this.catchAdStarted));
		
			if(this._melAdStoppedCallbackRef == null)
				this._melAdStoppedCallbackRef = SLINGO.MessageEventListener.registerCallback('ANY', 'AdStopped', bind(this, this.catchAdStopped));
		} else if(this._adErrorOccurred) {
			this._adErrorOccurred = false;
			this.cleanUpCallbacks();
			this.setState('END');	
		}
		/* Activate Skip Button when it's time */
                if(this._skipButtonEnabled && !this._skipButtonActive && (secondsElapsed >= this._skipButtonActivateTime)) {
			if(this._beaconActive) {
				var skipBtnNode;
				
				skipBtnNode = SLINGO.$('skipAdBtnPreRoll');
				if(skipBtnNode)
					SLINGO.addClassName(skipBtnNode,'active');

				skipBtnNode = SLINGO.$('skipAdBtnInter');
				if(skipBtnNode)
					SLINGO.addClassName(skipBtnNode,'active');			
				
				this._skipButtonActive = true;
			}
		}
		/* If beacon isn't set, abort after _nonBeaconDuration seconds */
		if(!this._beaconActive && (secondsElapsed >= this._nonBeaconDuration)) {
			this.cleanUpCallbacks();
			this.setState('END');
		}
                
                var tick = Math.floor(secondsElapsed);
                if((tick !== this.lastLoaderTick) && this._loader_elm_id && SLINGO.$(this._loader_elm_id)){
                    var vis = SLINGO.$(this._loader_elm_id).style.visibility;
                    if(vis === 'visible'){
                        this.lastLoaderTick = tick;
                        var text = SLINGO.$(this._loader_elm_id).innerHTML;
                        var len = text.length === 3 ? 1 : text.length+1;
                        var newText = '';
                        for(var i = 0; i < len; i++)
                            newText += '.';
                        SLINGO.$(this._loader_elm_id).innerHTML = newText;
                    }
                }
	},
	catchAdLoaded : function() {
		this._beaconActive = true;
		SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdLoaded', this._melAdLoadedCallbackRef);
		this._melAdLoadedCallbackRef = null;
	},
        catchAdStarted : function() {
                SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdStarted', this._melAdStartedCallbackRef);
		this._melAdStartedCallbackRef = null;
                
                if(this._loader_elm_id && SLINGO.$(this._loader_elm_id)) 
                    SLINGO.$(this._loader_elm_id).style.visibility = 'hidden';
        },
	catchAdStopped : function() {
		this.cleanUpCallbacks();
		this.setState('END');		
	},
	catchAdError : function() {
		this._adErrorOccurred = true;
	},
	cleanUpCallbacks : function() {
		if(this._melAdLoadedCallbackRef != null) {
			SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdLoaded', this._melAdLoadedCallbackRef);
			this._melAdLoadedCallbackRef = null;
		} 
                if(this._melAdStartedCallbackRef != null) {
			SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdStarted', this._melAdStartedCallbackRef);
			this._melAdStartedCallbackRef = null;
		} 
		if(this._melAdStoppedCallbackRef !=  null) {
			SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdStopped', this._melAdStoppedCallbackRef);
			this._melAdStoppedCallbackRef = null;
		}
		if(this._melAdErrorCallbackRef != null) {
			SLINGO.MessageEventListener.unregisterCallback('ANY', 'AdError', this._melAdErrorCallbackRef);
			this._melAdErrorCallbackRef = null;
		}
	},
	skipAd : function() {
		if(this._skipButtonActive) {
			var skipBtnNode;
			
			skipBtnNode = SLINGO.$('skipAdBtnPreRoll');
			if(skipBtnNode)
				SLINGO.removeClassName(skipBtnNode,'active');

			skipBtnNode = SLINGO.$('skipAdBtnInter');
			if(skipBtnNode)
				SLINGO.removeClassName(skipBtnNode,'active');		

			this.cleanUpCallbacks();
			this.setState('END');
		}
	},
        setDuration : function(duration) {
            this._pbar_duration = duration;
            this._nonBeaconDuration = duration;
            
            console.debug('DURATION OF VIDEO AD PBAR: ' + this._pbar_duration);
            console.debug('DURATION OF VIDEO AD nonBeacon: ' + this._nonBeaconDuration);
        }

});


SLINGO.AdElement = SLINGO.DisplayElement.extend({
	_obj_class_name : 'AdElement',
	
	_setState_SHOW : function(){
		if(!SLINGO.inArray(this.getState(), ['INIT','HIDE', 'END'])) // @todo maybe end statement should not goes here
			return SLINGO.triggerError('AdElement._setState_SHOW: invalid state: '+this.getState(),SLINGO.EWARN);
			
		this.getElement().style.display = 'block';
		return true;
	},
	
	_setState_HIDE : function(){
		if(this.getState() === 'HIDE')
			return SLINGO.triggerError('AdElement._setState_HIDE: invalid state: '+this.getState(),SLINGO.EWARN);
		
		this.getElement().style.display = 'none';
		return true;
	}

});

SLINGO.IframeAdElement = SLINGO.AdElement.extend({
	_obj_class_name : 'IframeAdElement',

	iframeElm : null,
	iframeSrc : null,

	adContentClass : 'adContent',
	reloadElmClass : 'reloadAd',

	covered : false,

	_onInit : function(){
		/* Add the refresh onclick event to any "reload ad" links. */
		var reload = SLINGO.getElementsByClass(this.reloadElmClass, this.getElement());
		if(reload)
			for(var i in reload)
				reload[i].onclick = bind(this, this.reloadAd);

		/* Add this object reference to the ad so other plugins can manage the ad state. (ex. a flyout is covering the ad) */
		var adContent = SLINGO.getElementsByClass(this.adContentClass, this.getElement());
		if(adContent)
			for(var i in adContent)
				adContent[i].OBJ_INST_REF = this;
	},

	setIframe : function(node, src){
		if(!SLINGO.$(node))
			return SLINGO.triggerError('IframeAdElement.setIframe: invalid element',SLINGO.EWARN);

		this.iframeElm = node;
		this.iframeSrc = src;
		return true;
	},

	reloadAd : function(){
		return this.setState('REFRESH');
	},

	isDisplayed : function(){
		return SLINGO.inArray(this.getState(), ['SHOW','REFRESH','UNCOVER']);
	},

	isCovered : function(){
		return this.covered;
	},

	_setState_SHOW : function(){
		if(!SLINGO.inArray(this.getState(), ['INIT','HIDE','UNCOVER', 'END'])) // @todo maybe end statement should not goes here
			return SLINGO.triggerError('IframeElement._setState_SHOW: invalid state: '+this.getState(),SLINGO.EWARN);

		if(!this.iframeSrc)
			return SLINGO.triggerError('IframeElement._setState_SHOW: no iframe src defined',SLINGO.EWARN);

		if(this.getElement().style.display !== 'block')
			this.getElement().style.display = 'block';

		if(!this.isCovered() && (this.iframeElm.src != this.iframeSrc))
			this.iframeElm.src = this.iframeSrc;

		return true;
	},

	_setState_HIDE : function(){
		if(this.getState() === 'HIDE')
			return SLINGO.triggerError('IframeElement._setState_HIDE: invalid state: '+this.getState(),SLINGO.EWARN);

		this.iframeElm.src = '';
		this.getElement().style.display = 'none';

		return true;
	},

	_setState_REFRESH : function(){
		if(!this.isDisplayed())
			return SLINGO.triggerError('IframeElement._setState_REFRESH: invalid state: '+this.getState(),SLINGO.EWARN);

		if(!this.iframeSrc)
			return SLINGO.triggerError('IframeElement._setState_REFRESH: no iframe src defined',SLINGO.EWARN);

		this.iframeElm.src = '';
		this.iframeElm.src = this.iframeSrc;

		return true;
	},

	_setState_CLEAR : function(){
		this.iframeElm.src = '';
		return true;
	},

	/* Use if an open overlay/flyout is going to cover the ad. */
	_setState_COVER : function(){
		this.covered = true;
		return true;
	},

	/* Use if a closed overlay/flyout isn't covering the ad anymore. */
	_setState_UNCOVER : function(){
		this.covered = false;
		return true;
	}

});

SLINGO.VideoAdElement = SLINGO.AdElement.extend({
	_obj_class_name : 'VideoAdElement',

	src : null,

	setSrc : function(src){
		this.src = src;
	},

	_setState_START : function(){
		this.getElement().src = this.src;
		return true;
	},

	_setState_END : function(){
		this.getElement().src = '';
		return true;
	},
	_setState_REFRESH : function(){
		this.getElement().src = '';
		this.getElement().src = this.src;

		return true;
	},
	_onUpdState : function(state){
		SLINGO.triggerError('VideoAdElement._onUpdState: '+state+', for element: '+this.getElement().id,SLINGO.EWARN);
	}

});

SLINGO.GPTAdElement = SLINGO.AdElement.extend({
        _obj_class_name : 'GPTAdElement',

        src : null,
	slot: null,
	size: null,
	opt_div: null,

	covered : false,
	
        _onInit : function(){
		console.log("*VJ* GPTAdElement::Init");
                /* Add the refresh onclick event to any "reload ad" links. */
                var reload = SLINGO.getElementsByClass(this.reloadElmClass, this.getElement());
                if(reload)
                        for(var i in reload)
                                reload[i].onclick = bind(this, this.reloadAd);

                /* Add this object reference to the ad so other plugins can manage the ad state. (ex. a flyout is covering the ad) */
                var adContent = SLINGO.getElementsByClass(this.adContentClass, this.getElement());
                if(adContent)
                        for(var i in adContent)
                                adContent[i].OBJ_INST_REF = this;
        },


        setSrc : function(src){
		console.log("*VJ* GPTAdElement::setSrc " + src);
                this.src = src;
        },

	setGPTData: function(slot, size, div){
		console.log("*VJ* GPTAdElement::setGPTData " + slot + "," + size + "," + div);
		this.slot = slot;
		this.size = size;
		this.opt_div = div;

		// init GPT
		googletag.cmd.push(function() {
			console.log("*VJ* GPTAdElement::push " + div);
			var sz = size.split('x');
			googletag.defineSlot(slot, [parseInt(sz[0], 10), parseInt(sz[1], 10)], div).addService(googletag.pubads());
		});
	},

	reloadAd : function(){
		console.log("*VJ* GPTAdElement::reloadAd");
                return this.setState('REFRESH');
        },

        isDisplayed : function(){
		console.log("*VJ* GPTAdElement::isDisplayed");
                return SLINGO.inArray(this.getState(), ['SHOW','REFRESH','UNCOVER']);
        },

        isCovered : function(){
		console.log("*VJ* GPTAdElement::isCovered");
                return this.covered;
        },

        _setState_SHOW : function(){
		console.log("*VJ* GPTAdElement::SHOW " + this.opt_div);
		console.log("*VJ* getElement: " + this.getElement());

                if(!SLINGO.inArray(this.getState(), ['INIT','HIDE','UNCOVER', 'END'])) // @todo maybe end state should not goes here
                        return SLINGO.triggerError('IframeElement._setState_SHOW: invalid state: '+this.getState(),SLINGO.EWARN);

                //VJ if(!this.iframeSrc)
                //VJ        return SLINGO.triggerError('IframeElement._setState_SHOW: no iframe src defined',SLINGO.EWARN);

                if(this.getElement().style.display !== 'block')
                        this.getElement().style.display = 'block';

                //VJ if(!this.isCovered() && (this.iframeElm.src != this.iframeSrc))
                //VJ        this.iframeElm.src = this.iframeSrc;
		var div = this.opt_div;
		googletag.cmd.push(function() {
			console.log("*VJ* GPTAdElement::ADSHOW " + div);
			googletag.display(div);
		});

                return true;
        },

        _setState_HIDE : function(){
		console.log("*VJ* GPTAdElement::HIDE");

                if(this.getState() === 'HIDE')
                        return SLINGO.triggerError('IframeElement._setState_HIDE: invalid state: '+this.getState(),SLINGO.EWARN);

                //VJ this.iframeElm.src = '';
                this.getElement().style.display = 'none';

                return true;
        },

        _setState_REFRESH : function(){
		console.log("*VJ* GPTAdElement::REFRESH");
		return true;

                if(!this.isDisplayed())
                        return SLINGO.triggerError('IframeElement._setState_REFRESH: invalid state: '+this.getState(),SLINGO.EWARN);

                if(!this.iframeSrc)
                        return SLINGO.triggerError('IframeElement._setState_REFRESH: no iframe src defined',SLINGO.EWARN);

                this.iframeElm.src = '';
                this.iframeElm.src = this.iframeSrc;

                return true;
        },

        _setState_CLEAR : function(){
		console.log("*VJ* GPTAdElement::CLEAR");
		return true;

                this.iframeElm.src = '';
                return true;
        },

        /* Use if an open overlay/flyout is going to cover the ad. */
        _setState_COVER : function(){
		console.log("*VJ* GPTAdElement::COVER");
		return true;

                this.covered = true;
                return true;
        },

        /* Use if a closed overlay/flyout isn't covering the ad anymore. */
        _setState_UNCOVER : function(){
		console.log("*VJ* GPTAdElement::UNCOVER");
		return true;

                this.covered = false;
                return true;
        },

        _setState_START : function(){
		console.log("*VJ* GPTAdElement::START");
                //VJ this.getElement().src = this.src;
                return true;
        },

        _setState_END : function(){
		console.log("*VJ* GPTAdElement::END");
                //VJ this.getElement().src = '';
                return true;
        },
        _onUpdState : function(state){
		console.log("*VJ* GPTAdElement::_OnUpdState");
                SLINGO.triggerError('VideoAdElement._onUpdState: '+state+', for element: '+this.getElement().id,SLINGO.EWARN);
        }

});

SLINGO.AccordionMenu = SLINGO.StageDisplay.extend({

	/* the area taht will become hidden when the screen is minimized */
	hiddenArea	: null,

	initd : false,

	setArea : function(hid){
		if(!hid)
			return SLINGO.triggerError('AccordionMenu:.setArea: no hidden area specified',SLINGO.EWARN);

		this.hiddenArea = hid;
		return true;
	},

	_setState_SHOW : function(){
		if(this.getState() === 'SHOW')
			return SLINGO.triggerError('AccordionMenu._setState_SHOW: already flagged as shown',SLINGO.EWARN);

		this._super();

		if(!this.initd){
			var nd = this.getElement();
			if(!
				SLINGO.Effect.addEffect(this.hiddenArea, 'ACCORDION_POSY', this.hiddenArea.id+'_acc', 1, SLINGO.DHTML.getObjectHeight(this.hiddenArea.id), 'MAX')
			 ||!
				SLINGO.Effect.addEffect(this.hiddenArea, 'ALPHA', this.hiddenArea.id+'_alpha', 0, 100, 50, 250)
			)return SLINGO.triggerError('AccordionMenu.setArea: unable to add effects', SLINGO.EWARN);

			if(!SLINGO.Effect.createAction(
				this.hiddenArea,
				this.hiddenArea.id+'_close',
				[
					SLINGO.Effect.createActionObject(this.hiddenArea, this.hiddenArea.id+'_acc', 'TO_MIN'),
					SLINGO.Effect.createActionObject(this.hiddenArea, this.hiddenArea.id+'_alpha', 'TO_MIN')
				]
			))return SLINGO.triggerError('AccordionMenu.setArea: failed to create an effect action',SLINGO.EWARN);

			if(!SLINGO.Effect.createAction(
				this.hiddenArea,
				this.hiddenArea.id + '_open',
				[
					SLINGO.Effect.createActionObject(this.hiddenArea, this.hiddenArea.id+'_acc', 'MAX'),
					SLINGO.Effect.createActionObject(this.hiddenArea, this.hiddenArea.id+'_alpha', 'MAX')
				]
			))return SLINGO.triggerError('AccordionMenu: failed to create an effect Action',SLINGO.EWARN);

			this.initd = true;
		}

		return SLINGO.Effect.runAction(this.hiddenArea, this.hiddenArea.id+'_open');
	},

	_setState_MIN : function(){
		if(this.getState() !== 'SHOW')
			return SLINGO.triggerError('AccordionMenu._setState_MIN: element not being displayed',SLINGO.EWARN);

		return SLINGO.Effect.runAction(this.hiddenArea, this.hiddenArea.id+'_close');
	}

});


/* the GameMsg object can currently either send() text to the console of *
* alert() a message. Usage will keep the errors in IE from forgotten    *
* console.log() calls from happening. It also will allow us to turn     *
* them on and off completely by setting the debug level.                */
SLINGO.GameMsg = {
	debugLevel : 1,

	send : function(msg){
		if(this.debugLevel){
			var d = new Date();
			var ds = d.getHours() + ':' + d.getMinutes() + ':' + d.getSeconds() + ':' + d.getMilliseconds();
			var s = '> PAGE: [' + ds + '] ' + msg;
			if('console' in window || (!window.all && typeof console != 'undefined'))
			console.log(s);
		}

		return true;
	},

	alert : function(msg){
		if(this.debugLevel)
		alert(msg);

		return true;
	}
};

/************************************************************************
/************************************************************************
/* Definitions of GamePage class constants
/************************************************************************
/************************************************************************/





/************************************************************************
/************************************************************************
/* Definitions of GamePage class methods
/************************************************************************
/************************************************************************/



/* OK, GamePage plugin finished. */
SLINGO.transitLoadComplete = true;
